/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_cb_ioim_macros.h BFA IOIM driver interface macros.
 */

#ifndef __BFA_HCB_IOIM_MACROS_H__
#define __BFA_HCB_IOIM_MACROS_H__

#include <bfa_os_inc.h>
/*
 * #include <linux/dma-mapping.h>
 *
 * #include <scsi/scsi.h> #include <scsi/scsi_cmnd.h> #include
 * <scsi/scsi_device.h> #include <scsi/scsi_host.h>
 */
#include "bfad_im_compat.h"
#include "bfad_drv.h"

/*
 * task attribute values in FCP-2 FCP_CMND IU
 */
#define SIMPLE_Q    0
#define HEAD_OF_Q   1
#define ORDERED_Q   2
#define ACA_Q	    4
#define UNTAGGED    5

#define BFA_SG_FIRST(DIO) (void *)scsi_sglist((struct scsi_cmnd *)DIO)

#if (!defined(__VMKLNX__) && \
	(LINUX_VERSION_CODE > 	KERNEL_VERSION(2, 6, 26))) ||	\
	(defined(__VMKLNX__) && (ESX_VERS > ESX41_VERS))
#define BFA_SG_NEXT(SG, DIO, SGEID)	\
	SG = (void *)sg_next((struct scatterlist *)SG)
#else
#define BFA_SG_NEXT(SG, DIO, SGEID)	\
	SG = bfa_ioim_get_sg(DIO, SGEID)
#endif

#if (defined(__VMKLNX__) && (ESX_VERS > ESX41_VERS))
#define BFA_SG_RESET(SG)	sg_reset((struct scatterlist *)SG)
#else
#define BFA_SG_RESET(SG)
#endif

static inline lun_t
bfad_int_to_lun(uint32_t luno)
{
	union {
		uint16_t	scsi_lun[4];
		lun_t		bfa_lun;
	} lun;

	lun.bfa_lun     = 0;
	lun.scsi_lun[0] = bfa_os_htons(luno);

	return (lun.bfa_lun);
}

/**
 * Get LUN for the I/O request
 */
#define bfa_cb_ioim_get_lun(__dio)	\
	bfad_int_to_lun(((struct scsi_cmnd *)__dio)->device->lun)

/**
 * Get CDB for the I/O request
 */
static inline uint8_t *
bfa_cb_ioim_get_cdb(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

	return ((uint8_t *) cmnd->cmnd);
}

/**
 * Get I/O direction (read/write) for the I/O request
 */
static inline fcp_iodir_t
bfa_cb_ioim_get_iodir(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	enum dma_data_direction dmadir;

	dmadir = cmnd->sc_data_direction;
	if (dmadir == DMA_TO_DEVICE)
		return FCP_IODIR_WRITE;
	else if (dmadir == DMA_FROM_DEVICE)
		return FCP_IODIR_READ;
	else
		return FCP_IODIR_NONE;
}

/**
 * Get IO size in bytes for the I/O request
 */
static inline uint32_t
bfa_cb_ioim_get_size(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

	return (scsi_bufflen(cmnd));
}

/**
 * Get timeout for the I/O request
 */
static inline uint8_t
bfa_cb_ioim_get_timeout(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	/*
	 * TBD: need a timeout for scsi passthru
	 */
	if (cmnd->device->host == NULL)
		return 4;

	return 0;
}

static inline void *
bfa_ioim_get_sg(struct bfad_ioim_s *dio, int sgeid)
{
	struct scatterlist *sg = NULL;
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

#if (!defined(__VMKLNX__) && \
	(LINUX_VERSION_CODE >   KERNEL_VERSION(2, 6, 26))) ||   \
	(defined(__VMKLNX__) && (ESX_VERS > ESX41_VERS))
	int i;
	scsi_for_each_sg(cmnd, sg, sgeid, i) {}
#else
	sg = (struct scatterlist *)scsi_sglist(cmnd) + sgeid;
#endif
	return (void*)sg;
}

static inline void *
bfa_cb_ioim_get_kva(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	return (void*)scsi_sglist(cmnd);
}

/**
 * Get SG element virtual mem for the I/O request
 */
static inline void *
bfa_cb_ioim_phys_to_virt(struct bfad_ioim_s *dio, void *sg)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	void *addr = NULL;

	if (scsi_sg_count(cmnd)) {
		addr = phys_to_virt(sg_dma_address((struct scatterlist *)sg));
	} else {
#ifdef __VMKERNEL_MODULE__
		addr = phys_to_virt(cmnd->request_bufferMA);
#else
		addr = phys_to_virt(cmnd->SCp.dma_handle);
#endif
	}

	return addr;
}

static inline uint32_t
bfa_cb_ioim_get_sg_count(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

	return scsi_sg_count(cmnd);
}

static inline bfi_addr_t
bfa_cb_ioim_get_sgaddr(struct bfad_ioim_s *dio, void *sg)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	uint64_t	addr;

	if (scsi_sg_count(cmnd)) {
		addr = bfa_sgaddr_le(sg_dma_address((struct scatterlist *)sg));
	} else {
#ifdef __VMKERNEL_MODULE__
		addr = (uint64_t) cmnd->request_bufferMA;
#else
		addr = bfa_sgaddr_le(cmnd->SCp.dma_handle);
#endif
	}

	return (*(union bfi_addr_u *) &addr);
}

static inline uint32_t
bfa_cb_ioim_get_sglen(struct bfad_ioim_s *dio, void *sg)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	uint32_t	len;

	if (scsi_sg_count(cmnd)) {
		len = sg_dma_len((struct scatterlist *)sg);
	} else {
		len = scsi_bufflen(cmnd);
	}

	return len;
}

/**
 * Get Command Reference Number for the I/O request. 0 if none.
 */
static inline uint8_t
bfa_cb_ioim_get_crn(struct bfad_ioim_s *dio)
{
	return 0;
}

/**
 * Get SAM-3 priority for the I/O request. 0 is default.
 */
static inline uint8_t
bfa_cb_ioim_get_priority(struct bfad_ioim_s *dio)
{
	return 0;
}

/**
 * Get task attributes for the I/O request. Default is FCP_TASK_ATTR_SIMPLE(0).
 */
static inline uint8_t
bfa_cb_ioim_get_taskattr(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	uint8_t	task_attr = UNTAGGED;

	if (cmnd->device->tagged_supported) {
		switch (cmnd->tag) {
		case HEAD_OF_QUEUE_TAG:
			task_attr = HEAD_OF_Q;
			break;
		case ORDERED_QUEUE_TAG:
			task_attr = ORDERED_Q;
			break;
		default:
			task_attr = SIMPLE_Q;
			break;
		}
	}

	return task_attr;
}

/**
 * Get CDB length in bytes for the I/O request. Default is FCP_CMND_CDB_LEN(16).
 */
static inline uint8_t
bfa_cb_ioim_get_cdblen(struct bfad_ioim_s *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

	return (cmnd->cmd_len);
}

/**
 * Assign queue to be used for the I/O request. This value depends on whether
 * the driver wants to use the queues via any specific algorithm. Currently,
 * this is not supported.
 */
#if (defined(__VMKLNX__) && (ESX_VERS > ESX50_VERS))
uint8_t bfa_cb_ioim_get_reqq(struct bfad_ioim_s *dio);
#else
#define bfa_cb_ioim_get_reqq(__dio) BFA_FALSE
#endif

/**
 * @}
 */

#endif /* __BFA_HCB_IOIM_MACROS_H__ */
