/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfad_2.6.c Linux driver PCI interface module.
 */
#include <linux/kthread.h>
#include "bfad_drv.h"
#include "bfad_im.h"
#include "bfad_tm.h"
#include "bfad_ipfc.h"

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9)) || \
    (LINUX_VERSION_CODE >= KERNEL_VERSION(3,2,0))
#include <linux/moduleparam.h>
#endif

module_param(os_name, charp, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(os_name, "OS name of the hba host machine");
module_param(os_patch, charp, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(os_patch, "OS patch level of the hba host machine");
module_param(host_name, charp, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(host_name, "Hostname of the hba host machine");
module_param(num_rports, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(num_rports, "Max number of rports supported per port (physical/logical), default=1024");
module_param(num_ioims, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(num_ioims, "Max number of ioim requests, default=2000");
module_param(num_tios, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(num_tios, "Max number of fwtio requests, default=0");
module_param(num_tms, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(num_tms, "Max number of task im requests, default=128");
module_param(num_fcxps, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(num_fcxps, "Max number of fcxp requests, default=64");
module_param(num_ufbufs, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(num_ufbufs, "Max number of unsolicited frame buffers, default=64");
module_param(reqq_size, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(reqq_size, "Max number of request queue elements, default=256");
module_param(rspq_size, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(rspq_size, "Max number of response queue elements, default=64");
module_param(num_sgpgs, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(num_sgpgs, "Number of scatter/gather pages, default=2048");
module_param(rport_del_timeout, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(rport_del_timeout, "Rport delete timeout, default=90 secs, Range[>0]");
module_param(bfa_lun_queue_depth, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(bfa_lun_queue_depth, "Lun queue depth, default=32, Range[>0]");
module_param(bfa_io_max_sge, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(bfa_io_max_sge, "Max io scatter/gather elements , default=255");
module_param(log_level, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(log_level, "Driver log level, default=3, Range[Critical:1|Error:2|Warning:3|Info:4]");
module_param(ioc_auto_recover, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(ioc_auto_recover, "IOC auto recovery, default=1, Range[off:0|on:1]");
module_param(linkup_delay, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(linkup_delay, "Link up delay, default=30 secs for boot port. Otherwise 10 secs in RHEL4 & 0 for [RHEL5, SLES10, ESX40] Range[>0]");
module_param(msix_disable_cb, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(msix_disable_cb, "Disable Message Signaled Interrupts for QLogic-415/425/815/825 cards, default=0, Range[false:0|true:1]");
module_param(msix_disable_ct, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(msix_disable_ct, "Disable Message Signaled Interrupts if possible for QLogic-1010/1020/804/1007/1741 cards, default=0, Range[false:0|true:1]");
module_param(fdmi_enable, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(fdmi_enable, "Enables fdmi registration, default=1, Range[false:0|true:1]");
module_param(pcie_max_read_reqsz, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(pcie_max_read_reqsz, "PCIe max read request size, default=0 (use system setting), Range[128|256|512|1024|2048|4096]");
module_param(max_xfer_size, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(max_xfer_size, "default=32MB, Range[64k|128k|256k|512k|1024k|2048k]");
module_param(max_rport_logins, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(max_rport_logins, "Max number of logins to initiator and target rports on a port (physical/logical), default=1024");
#if (defined(__VMKLNX__) && (ESX_VERS > ESX50_VERS))
module_param(vmklnx_multiq, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(vmklnx_multiq, "Enables registering multiple storage io queues with vmkernel. default=1, Range[disable:0|enable:1]");
#endif

void *
bfad_os_dma_alloc(struct bfad_s *bfad, struct bfa_mem_dma_s *meminfo_elem,
		dma_addr_t      *phys_addr)
{
	return dma_alloc_coherent(&bfad->pcidev->dev,
		meminfo_elem->mem_len,
		phys_addr, GFP_KERNEL);
}

void
bfad_os_dma_free(struct bfad_s *bfad, struct bfa_mem_dma_s *meminfo_elem)
{
	dma_free_coherent(&bfad->pcidev->dev,
			meminfo_elem->mem_len,
			meminfo_elem->kva,
			(dma_addr_t) meminfo_elem->dma);
}

void
bfad_os_kthread_create(struct bfad_s *bfad)
{

	bfad->bfad_tsk = kthread_create(bfad_worker, (void *) bfad, "%s", "bfad_worker");
	if (IS_ERR(bfad->bfad_tsk)) {
		printk(KERN_INFO "bfad[%d]: Kernel thread creation failed!\n",
				bfad->inst_no);
		bfa_sm_send_event(bfad, BFAD_E_KTHREAD_CREATE_FAILED);
	}

	bfa_sm_send_event(bfad, BFAD_E_INIT);
}

void
bfad_os_kthread_stop(struct bfad_s *bfad)
{
	kthread_stop(bfad->bfad_tsk);
}

void
bfad_os_kthread_wakeup(struct bfad_s *bfad)
{
	wake_up_process(bfad->bfad_tsk);
}

int
bfad_os_kthread_should_stop(void)
{
	return kthread_should_stop();
}

#ifdef CONFIG_PM
/**
 * @brief
 * PCI suspend entry.
 */
static int
bfad_os_suspend(struct pci_dev *pdev, pm_message_t state)
{
	struct bfad_s         *bfad = pci_get_drvdata(pdev);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 12))
	pci_power_t     device_state;
#else
	uint32_t        device_state;
#endif

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 12))
	device_state = pci_choose_state(pdev, state);
#else
	switch (state) {
	case 1:		/* S1 */
		device_state = 1; /* D1 */ ;
		break;
	case 3:		/* S3 */
	case 4:		/* S4 */
		device_state = 3; /* D3 */ ;
		break;
	default:
		return -EAGAIN /* FIXME */;
		break;
	}
#endif

	bfad_os_pci_save_state(pdev);

	init_completion(&bfad->suspend);

	bfa_sm_send_event(bfad, BFAD_E_STOP);

	bfa_detach(&bfad->bfa);
	bfad_hal_mem_release(bfad);
	bfad->bfad_flags &= ~BFAD_DRV_INIT_DONE;

	wait_for_completion_timeout(&bfad->suspend, 30);
	pci_disable_device(pdev);
	pci_set_power_state(pdev, device_state);

	return 0;
}

/**
 * @brief
 * PCI resume entry.
 */
static int
bfad_os_resume(struct pci_dev *pdev)
{
	struct bfad_s	*bfad = pci_get_drvdata(pdev);
	int retval;

	/* Resume the device power state to D0 */
	pci_set_power_state(pdev, 0);

	/* Restore all device PCI configuation space to its original state. */
	bfad_os_pci_restore_state(pdev);

	if (pci_enable_device(pdev))
		goto out;
	pci_set_master(pdev);

	retval = bfad_drv_init(bfad);

	if (retval != BFA_STATUS_OK)
		goto out;

	bfa_sm_send_event(bfad, BFAD_E_CREATE);

	return 0;

out:
	printk(KERN_ERR "bfad[%d] resume failed\n", bfad->inst_no);
	return 1;

}
#endif

#ifdef SUPPORT_PCI_AER
/*
 * PCI error recovery support, this fearure is only availbe for kernel
 * .6.16 or later.
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
/**
 * @brief
 * PCI Error Recovery entry, error detected.
 */
static	pci_ers_result_t
bfad_pci_error_detected(struct pci_dev *pdev, pci_channel_state_t state)
{
	struct bfad_s		*bfad = pci_get_drvdata(pdev);
	unsigned long		flags;
	pci_ers_result_t	ret = PCI_ERS_RESULT_NONE;

	dev_printk(KERN_ERR, &pdev->dev,
		"error detected state: %d - flags: 0x%x\n",
		state, bfad->bfad_flags);

	switch (state) {
	case pci_channel_io_normal: /* non-fatal error */

		spin_lock_irqsave(&bfad->bfad_lock, flags);
		bfad_flags_clear(bfad, BFAD_EEH_BUSY);
		/*
		 * Suspend/fail all bfa operations
		 */
		bfa_suspend(&bfad->bfa);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		del_timer_sync(&bfad->hal_tmo);
		ret = PCI_ERS_RESULT_CAN_RECOVER;
		break;

	case pci_channel_io_frozen: /* fatal error */

		spin_lock_irqsave(&bfad->bfad_lock, flags);
		 bfad_flags_set(bfad, BFAD_EEH_BUSY);
		/*
		 * Suspend/fail all bfa operations
		 */
		bfa_suspend(&bfad->bfa);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);

		spin_lock_irqsave(&bfad->bfad_lock, flags);
		init_completion(&bfad->comp);
		bfa_fcs_stop(&bfad->bfa_fcs);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		wait_for_completion(&bfad->comp);

		bfad_remove_intr(bfad);
		del_timer_sync(&bfad->hal_tmo);
		pci_disable_device(pdev);

		ret = PCI_ERS_RESULT_NEED_RESET;
		break;

	case pci_channel_io_perm_failure: /* PCI Card is DEAD */

		spin_lock_irqsave(&bfad->bfad_lock, flags);
		bfad_flags_set(bfad, BFAD_EEH_BUSY);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		bfad_flags_set(bfad, BFAD_EEH_PCI_CHANNEL_IO_PERM_FAILURE);

		/* If the error_detected handler is called with the reason
		 * pci_channel_io_perm_failure - it will subsequently call
		 * pci_remove() entry point to remove the pci device from the
		 * system - So defer the cleanup to pci_remove(); cleaning up
		 * here causes inconsistent state during pci_remove().
		 */

		ret = PCI_ERS_RESULT_DISCONNECT;
		break;

	default:
		bfa_assert(0);
	}

	return ret;

}

int
restart_bfa(struct bfad_s *bfad)
{
	unsigned long flags;
	struct pci_dev *pdev = bfad->pcidev;

	bfa_attach(&bfad->bfa, bfad, &bfad->ioc_cfg, &bfad->meminfo,
		   &bfad->hal_pcidev);

	/* Enable Interrupt and wait bfa_init completion */
	if (bfad_setup_intr(bfad)) {
		dev_printk(KERN_WARNING, &pdev->dev,
			"%s: bfad_setup_intr failed\n",
			bfad->pci_name);
		bfa_sm_send_event(bfad, BFAD_E_INIT_FAILED);
		return -1;
	}

	init_completion(&bfad->comp);

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_init(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

#ifdef CONFIG_PCI_MSI
	/* Set up interrupt handler for each vectors */
	if ((bfad->bfad_flags & BFAD_MSIX_ON) &&
		bfad_install_msix_handler(bfad)) {
			dev_printk(KERN_WARNING, &pdev->dev,
				"%s: install_msix failed.\n",
				bfad->pci_name);
	}
#endif
	bfad_init_timer(bfad);

	wait_for_completion(&bfad->comp);

	bfad_drv_start(bfad);

	return 0;
}

/**
 * @brief
 * PCI Error Recovery entry, re-initialize the chip.
 */

static	pci_ers_result_t
bfad_pci_slot_reset(struct pci_dev *pdev)
{
	struct bfad_s	*bfad = pci_get_drvdata(pdev);
	uint8_t b;

	dev_printk(KERN_ERR, &pdev->dev,
		"bfad_pci_slot_reset flags: 0x%x\n", bfad->bfad_flags);

	if (pci_enable_device(pdev)) {
		dev_printk(KERN_ERR, &pdev->dev, "Cannot re-enable "
			"PCI device after reset.\n");
		return PCI_ERS_RESULT_DISCONNECT;
	}

	bfad_os_pci_restore_state(pdev);

#if 0
	{
		uint8_t b;
		uint32_t i;

		printk("slot_reset_0: ");
			for (i = 0; i < 256; i++) {
			pci_read_config_byte(pdev, i, &b);
			printk("%s%02x", (i%16) ? " " : "\n", b);
		}
		printk("\n");
	}
#endif

	/*
	 * WORK AROUND: Read some byte (e.g. DMA max. payload size which can't
	 * be 0xff any time) to make sure -
	 * we did not hit another PCI error in the middle of recovery.
	 * If we did, then declare permanent failure. Because the outcome
	 * of the function is either recovered/failed. Continuing with
	 * recovery will not work anyway.
	 */
	pci_read_config_byte(pdev, 0x68, &b);
	if (b == 0xff) {
		dev_printk(KERN_ERR, &pdev->dev,
		"slot_reset failed ... got another PCI error !\n");
		goto out_disable_device;
	}

	/*
	 * In the newer kernels, the pci_restore_state clears the
	 * device saved_state flag, so, pci_save_state is required
	 */
	bfad_os_pci_save_state(pdev);

	pci_set_master(pdev);

	if (pci_set_dma_mask(bfad->pcidev, DMA_BIT_MASK(64)) != 0)
		if (pci_set_dma_mask(bfad->pcidev, DMA_BIT_MASK(32)) != 0)
			goto out_disable_device;

	BFAD_CLEANUP_PCIE_AER(pdev);

	if (restart_bfa(bfad) == -1)
		goto out_disable_device;

	BFAD_ENABLE_PCIE_AER(pdev);

	dev_printk(KERN_WARNING, &pdev->dev,
		"slot_reset completed  flags: 0x%x!\n", bfad->bfad_flags);

	return PCI_ERS_RESULT_RECOVERED;

out_disable_device:
	pci_disable_device(pdev);

	return PCI_ERS_RESULT_DISCONNECT;

}

static pci_ers_result_t
bfad_pci_mmio_enabled(struct pci_dev *pdev)
{
	unsigned long   flags;
	struct bfad_s *bfad = pci_get_drvdata(pdev);

	dev_printk(KERN_INFO, &pdev->dev, "mmio_enabled\n");

	/*
	 * Fetch FW diagnostic information
	 */
	bfa_get_debug_info(&bfad->bfa);
	/*
	 * Cancel all pending IOs. Happens in bfad_fcs_stop() !
	 */
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	init_completion(&bfad->comp);
	bfa_fcs_stop(&bfad->bfa_fcs);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	wait_for_completion(&bfad->comp);

	bfad_remove_intr(bfad);
	del_timer_sync(&bfad->hal_tmo);
	pci_disable_device(pdev);

	return PCI_ERS_RESULT_NEED_RESET;
}

static void
bfad_pci_os_resume(struct pci_dev *pdev)
{
	unsigned long   flags;
	struct bfad_s *bfad = pci_get_drvdata(pdev);

	dev_printk(KERN_WARNING, &pdev->dev, "resume\n");

	/*
	 * wait until the link is online
	 */
	bfad_os_rport_online_wait(bfad);

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfad_flags_clear(bfad, BFAD_EEH_BUSY);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

}

/*
 * PCI error recovery support, this fearure is only available for kernel
 * 2.6.16 or later. At this point, we only define error_detected and
 * slot_reset.
 */
static struct pci_error_handlers bfad_err_handler = {
	.error_detected = bfad_pci_error_detected,
	.slot_reset = bfad_pci_slot_reset,
	.mmio_enabled = bfad_pci_mmio_enabled,
	.resume = bfad_pci_os_resume,
};
#endif
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
static int
#else
static int      __devinit
#endif
bfad_os_pci_probe(struct pci_dev *pdev, const struct pci_device_id *pid)
{
	return (bfad_pci_probe(pdev, pid));
}

static void
bfad_os_pci_remove(struct pci_dev *pdev)
{
	bfad_pci_remove(pdev);
}

static struct pci_driver bfad_pci_driver = {
	.name = BFAD_DRIVER_NAME,
	.id_table = bfad_id_table,
	.probe = bfad_os_pci_probe,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
	.remove = bfad_os_pci_remove,
#else
	.remove = __devexit_p(bfad_os_pci_remove),
#endif
#ifdef CONFIG_PM
	.suspend = bfad_os_suspend,
	.resume = bfad_os_resume,
#endif

#ifdef SUPPORT_PCI_AER
	/*
	 * PCI error recovery support, this fearure is only available for
	 * kernel 2.6.16 or later.
	 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
	.err_handler = &bfad_err_handler,
#endif
#endif
};

struct pci_driver *bfad_pci_driver_p = &bfad_pci_driver;

/*
 * @}
 */
