/* 
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfad_im_2.6.10.c Linux driver IM module.
 * code for Linux >= 2.6.10
 */

#include "bfad_drv.h"
#include "bfad_im.h"

void
bfad_os_fc_remove_host(struct Scsi_Host *shost)
{
	fc_remove_host(shost);
}

void 
bfad_os_rport_online_wait(struct bfad_s *bfad)
{
	int i;
	int rport_delay = 10;

	for (i = 0; !(bfad->bfad_flags & BFAD_PORT_ONLINE)
		 && i < linkup_delay; i++) {
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
	}

	if (bfad->bfad_flags & BFAD_PORT_ONLINE) {
		rport_delay = rport_delay < linkup_delay ?
				 rport_delay : linkup_delay;
		for (i = 0; !(bfad->bfad_flags & BFAD_RPORT_ONLINE)
			 && i < rport_delay; i++) {
			set_current_state(TASK_UNINTERRUPTIBLE);
			schedule_timeout(HZ);
		}

		if (rport_delay > 0 && (bfad->bfad_flags & BFAD_RPORT_ONLINE)) {
			set_current_state(TASK_UNINTERRUPTIBLE);
			schedule_timeout(rport_delay * HZ);
		}
	}
}

void
bfad_os_scsi_scan(struct bfad_im_port_s *im_port)
{
	return;
}

void
bfad_os_spin_os_lock_irq(struct Scsi_Host *shost)
{
	return;
}

void
bfad_os_spin_os_unlock_irq(struct Scsi_Host *shost)
{
	return;
}

int
bfad_os_get_linkup_delay(struct bfad_s *bfad)
{

	uint8_t         nwwns = 0;
	wwn_t           wwns[BFA_PREBOOT_BOOTLUN_MAX];
	int 		linkup_delay;

	/*
	 * Querying for the boot target port wwns
	 * -- read from boot information in flash.
	 * If nwwns > 0 => boot over SAN and set linkup_delay = 30
	 * else => local boot machine set linkup_delay = 0
	 */

	bfa_iocfc_get_bootwwns(&bfad->bfa, &nwwns, wwns);

	if (nwwns > 0) {
		/* If Boot over SAN set linkup_delay = 30sec */
		linkup_delay = 30;
	} else {
		/* If local boot; no linkup_delay */
		linkup_delay = 0;
	}

	return linkup_delay;
}

