/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfad_im_2.6.9.c Linux driver IM module.
 * code for Linux ver 2.6.9
 */

#include "bfad_drv.h"
#include "bfad_im.h"

BFA_TRC_FILE(LDRV, IM_2_6_9);

BFAD_WORK_HANDLER(bfad_im_port_delete_handler);

/**
 * @brief
 * Work queue handler using scsi_scan_host()
* Context: kernel
 */
BFAD_WORK_HANDLER(bfad_im_port_scsi_scan)
{				/* work is the function arg */
	struct bfad_im_port_s *im_port = container_of(work,
				struct bfad_im_port_s, scsi_scan_work);
	scsi_scan_host(im_port->shost);
}

void
bfad_os_init_work(struct bfad_im_port_s *im_port)
{
	BFAD_INIT_WORK(im_port, scsi_scan_work, bfad_im_port_scsi_scan);
	BFAD_INIT_WORK(im_port, port_delete_work, bfad_im_port_delete_handler);
}

void
bfad_os_delay_workq(struct bfad_im_s *im, struct bfad_im_port_s *im_port)
{
	queue_delayed_work(im->drv_workq, &im_port->scsi_scan_work,
			   SCSI_SCAN_DELAY);
}

void
bfad_os_scsi_scan(struct bfad_im_port_s *im_port)
{
	scsi_scan_host(im_port->shost);
}

void
bfad_os_fc_remove_host(struct Scsi_Host *shost)
{
	return;
}

void
bfad_os_fc_host_init(struct bfad_im_port_s *im_port)
{
	return;
}

void
bfad_os_spin_os_lock_irq(struct Scsi_Host *shost)
{
	spin_lock_irq(shost->host_lock);
        return;
}

void
bfad_os_spin_os_unlock_irq(struct Scsi_Host *shost)
{
	spin_unlock_irq(shost->host_lock);
        return;
}

int
bfad_os_im_itnim_is_online(struct bfad_itnim_s *itnim)
{
	return (itnim->state == ITNIM_STATE_ONLINE);
}

/**
 * @brief
 * Scsi_Host template entry slave_alloc
 */
int
bfad_im_slave_alloc(struct scsi_device *sdev)
{
	struct bfad_im_port_s *im_port = (struct bfad_im_port_s *) sdev->host->hostdata[0];
	struct bfad_itnim_data_s *itnim_data;
	struct bfad_s         *bfad;
	unsigned long   flags;

	if (sdev->id >= MAX_FCP_TARGET)
		return 1;

	bfad = im_port->bfad;

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	itnim_data = &im_port->itnim_data_table[sdev->id];
	if (itnim_data->itnim)
		sdev->hostdata = itnim_data;
	bfa_trc(bfad, 0);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}
