/* 
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFAD_IM_COMPAT_H__
#define __BFAD_IM_COMPAT_H__

#include <bfad_drv.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26))
extern struct device_attribute *bfad_im_host_attrs[];
extern struct device_attribute *bfad_im_vport_attrs[];
#else
extern struct class_device_attribute *bfad_im_host_attrs[];
extern struct class_device_attribute *bfad_im_vport_attrs[];
#endif

/*
 * Set the residual length for the SCSI command
 */
#ifndef scsi_set_resid
#define scsi_set_resid bfad_im_scsi_set_resid
static inline void
bfad_im_scsi_set_resid(struct scsi_cmnd *cmnd, uint32_t resid)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26))
	cmnd->sdb.resid = resid;
#else
	cmnd->resid = resid;
#endif
}
#endif

/*
 * Get to know whether the SG list is to be built or not
 */
#ifndef scsi_sg_count
#define scsi_sg_count bfad_im_scsi_sg_count
static inline int
bfad_im_scsi_sg_count(struct scsi_cmnd *cmnd)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26))
	return (cmnd->sdb.table.nents);
#else
	return (cmnd->use_sg);
#endif
}
#endif

/*
 * Get the scatter/gather buffer
 */
#ifndef scsi_sglist
#define scsi_sglist bfad_im_scsi_sglist
static inline struct scatterlist *
bfad_im_scsi_sglist(struct scsi_cmnd *cmnd)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26))
	return ((struct scatterlist *)cmnd->sdb.table.sgl);
#else
	return ((struct scatterlist *)cmnd->request_buffer);
#endif
}
#endif

static inline void *
bfa_sg_virt(struct scatterlist* sg)
{
#if (defined(__VMKLNX__) && (ESX_VERS > ESX41_VERS))
	return (page_address(sg_page(sg)) + vmklnx_sg_offset(sg));
#else
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24))
	return sg_virt(sg);
#else
        return page_address(sg->page) + sg->offset;
#endif
#endif
}

/*
 * Get the request buffer length, in case the SG list
 * is not required
 */
#ifndef scsi_bufflen
#define scsi_bufflen bfad_im_scsi_bufflen
static inline int
bfad_im_scsi_bufflen(struct scsi_cmnd *cmnd)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26))
	return (cmnd->sdb.length);
#else
	return (cmnd->request_bufflen);
#endif
}
#endif

static inline uint32_t *
bfad_load_fwimg(struct pci_dev *pdev)
{
	return ((uint32_t*)1);
}

#endif
