/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef BFAD_IOCTL_H
#define BFAD_IOCTL_H

#include <linux/ioctl.h>
#include "bfa_os_inc.h"
#include <cs/bfa_cs.h>
#include <defs/bfa_defs.h>
#include <ioctl/bfa_ioctl_iocfc.h>
#include <ioctl/bfa_ioctl_lport.h>
#include <ioctl/bfa_ioctl_rport.h>
#include <ioctl/bfa_ioctl_vport.h>
#include <ioctl/bfa_ioctl_vf.h>
#include <ioctl/bfa_ioctl_fabric.h>
#include <ioctl/bfa_ioctl_flash.h>
#include <ioctl/bfa_ioctl_fcpim.h>
#include <ioctl/bfa_ioctl_boot.h>
#include <ioctl/bfa_ioctl_scsi.h>
#include <ioctl/bfa_ioctl_fcpt.h>
#include <ioctl/bfa_ioctl_itnim.h>
#include <ioctl/bfa_ioctl_auth.h>
#include <ioctl/bfa_ioctl_qos.h>
#include <ioctl/bfa_ioctl_ratelim.h>
#include <ioctl/bfa_ioctl_ipfc.h>
#include <ioctl/bfa_ioctl_aen.h>
#include <ioctl/bfa_ioctl_fcport.h>
#include <ioctl/bfa_ioctl_generic.h>

int bfad_os_ioctl32_register(void);
void bfad_os_ioctl32_unregister(void);
int bfad_cp_to_user(int size, unsigned long arg, uint8_t *iocmd);
void *bfad_cp_from_user(int size, unsigned long arg);
int bfad_ioctl_ioc_get_inst(unsigned long arg);
int bfad_ioctl_ioc_get_version(unsigned long arg);

/* IOCTL cmd code */
#define IOCFC_MAGIC			0xB0
#define LPORT_MAGIC			0xB1
#define RPORT_MAGIC			0xB2
#define VPORT_MAGIC			0xB3
#define VF_MAGIC			0xB4
#define QOS_MAGIC			0xB6
#define RATELIM_MAGIC			0xB7
#define FCPIM_MAGIC			0xB8
#define BOOT_MAGIC			0xB9
#define PASSTHRU_MAGIC			0xBA
#define ITNIM_MAGIC			0xBB
#define AUTH_MAGIC			0xBD
#define FABRIC_MAGIC			0xBE
#define PBIND_MAGIC			0xBF
#define IPFC_MAGIC			0xC0
#define FCPORT_MAGIC			0xC1
#define TRUNK_MAGIC			0xC2
#define FAA_MAGIC			0xC3
#define TARGET_MODE_MAGIC	0xC4

/* IOCFC */
enum {
	BFAD_IOCFC_GET_ATTR = 0x01,
	BFAD_IOCFC_SET_INTR,
};

/* LPORT */
enum {
	BFAD_LPORT_GET_ATTR = 0x01,
	BFAD_LPORT_GET_IOSTATS,
	BFAD_LPORT_GET_STATS,
	BFAD_LPORT_RESET_STATS,
	BFAD_LPORT_GET_RPORTS,
};

/* RPORT */
enum {
	BFAD_RPORT_ADD = 0x01,
	BFAD_RPORT_REMOVE,
	BFAD_RPORT_GET_STATS,
	BFAD_RPORT_RESET_STATS,
	BFAD_RPORT_GET_ATTR,
	BFAD_RPORT_GET_ADDR,
	BFAD_RPORT_SET_SPEED,
};

/* VPORT */
enum {
	BFAD_VPORT_CREATE = 0x01,
	BFAD_VPORT_DEL,
	BFAD_VPORT_START,
	BFAD_VPORT_STOP,
	BFAD_VPORT_GET_ATTR,
	BFAD_VPORT_GET_STATS,
	BFAD_VPORT_RESET_STATS,
};

/* VF */
enum {
	BFAD_VF_ENABLE = 0x01,
	BFAD_VF_DISABLE,
	BFAD_VF_ADD,
	BFAD_VF_DEL,
	BFAD_VF_LIST,
	BFAD_VF_LIST_ALL,
	BFAD_VF_GET_ATTR,
	BFAD_VF_GET_STATS,
	BFAD_VF_RESET_STATS,
};

/* QOS */
enum {
	BFAD_QOS_ENABLE = 0x01,
	BFAD_QOS_DISABLE,
	BFAD_QOS_GET_ATTR,
	BFAD_QOS_GET_VC_ATTR,
	BFAD_QOS_GET_STATS,
	BFAD_QOS_RESET_STATS,
	BFAD_QOS_SET_BW,
};

/* RATELIM */
enum {
	BFAD_RATELIM_ENABLE = 0x01,
	BFAD_RATELIM_DISABLE,
	BFAD_RATELIM_DEF_SPEED,
};

/* FCPIM */
enum {
	BFAD_FCPIM_FAILOVER = 0x01,
	BFAD_FCPIM_QDEPTH,
	BFAD_FCPIM_MODSTATS,
	BFAD_FCPIM_MODSTATSCLR,
	BFAD_FCPIM_DEL_ITN_STATS,
	BFAD_FCPIM_LUNMASK_QUERY,
	BFAD_FCPIM_LUNMASK_ENABLE,
	BFAD_FCPIM_LUNMASK_DISABLE,
	BFAD_FCPIM_LUNMASK_ADD,
	BFAD_FCPIM_LUNMASK_DELETE,
	BFAD_FCPIM_LUNMASK_CLEAR,
	BFAD_FCPIM_THROTTLE_QUERY,
	BFAD_FCPIM_THROTTLE_SET,
	BFAD_FCPIM_IOPF_START,
	BFAD_FCPIM_IOPF_STOP,
	BFAD_FCPIM_IOPF_SHOW,
	BFAD_FCPIM_IOPF_QUERY,
	BFAD_FCPIM_IOPF_RESET,
};

/* BOOT */
enum {
	BFAD_BOOT_CFG = 0x01,
	BFAD_BOOT_QUERY,
	BFAD_PREBOOT_QUERY,
};

/* PASSTHRU */
enum {
	BFAD_SEND_SCSI = 0x01,
	BFAD_SEND_FC,
};

/* ITNIM */
enum {
	BFAD_ITNIM_GET_ATTR = 0x01,
	BFAD_ITNIM_GET_IOSTATS,
	BFAD_ITNIM_RESET_STATS,
	BFAD_ITNIM_GET_ITNSTATS,
	BFAD_ITNIM_GET_IOPROFILE,
	BFAD_ITNIM_TRS_ENABLE,
	BFAD_ITNIM_TRS_DISABLE,
	BFAD_ITNIM_TRS_CLEAR,
	BFAD_ITNIM_TRS_QUERY,
};

/* AUTH */
enum {
	BFAD_AUTH_GET_ATTR = 0x01,
	BFAD_AUTH_GET_STATS,
	BFAD_AUTH_RESET_STATS,
	BFAD_AUTH_REINIT,
	BFAD_AUTH_SET_POLICY,
	BFAD_AUTH_SET_ALGO,
	BFAD_AUTH_SET_GROUP,
	BFAD_AUTH_SET_SECRETSRC,
	BFAD_AUTH_SET_SECRETST,
};

/* FABRIC */
enum {
	BFAD_FABRIC_GET_LPORTS = 0x01,
};

/* PBIND */
enum {
	BFAD_PBIND_SET = 0x01,
};

/* IPFC */
enum {
	BFAD_IPFC_GET_ATTR = 0x01,
	BFAD_IPFC_GET_STATS,
	BFAD_IPFC_RESET_STATS,
	BFAD_IPFC_ENABLE,
	BFAD_IPFC_DISABLE,
};

/* FCPORT */
enum {
	BFAD_FCPORT_GET_STATS = 0x01,
	BFAD_FCPORT_RESET_STATS,
	BFAD_FCPORT_ENABLE,
	BFAD_FCPORT_DISABLE,
};

/* trunk */
enum {
	BFAD_TRUNK_ENABLE = 0x01,
	BFAD_TRUNK_DISABLE,
	BFAD_TRUNK_GET_ATTR,
};

/* FAA */
enum {
	BFAD_FAA_ENABLE = 0x01,
	BFAD_FAA_DISABLE,
	BFAD_FAA_QUERY,
};

/* target mode */
enum {
	BFAD_TARGET_MODE
};

/* IOCFC */
#define IOCMD_IOCFC_GET_ATTR		\
	_IOWR(IOCFC_MAGIC, BFAD_IOCFC_GET_ATTR, bfa_ioctl_iocfc_attr_t)
#define IOCMD_IOCFC_SET_INTR		\
	_IOWR(IOCFC_MAGIC, BFAD_IOCFC_SET_INTR, bfa_ioctl_iocfc_intr_t)

/* LPORT */
#define IOCMD_LPORT_GET_ATTR		\
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_ATTR, bfa_ioctl_lport_attr_t)
#define IOCMD_LPORT_GET_IOSTATS		\
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_IOSTATS, bfa_ioctl_lport_iostats_t)
#define IOCMD_LPORT_GET_STATS		\
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_STATS, bfa_ioctl_lport_stats_t)
#define IOCMD_LPORT_RESET_STATS		\
	_IOWR(LPORT_MAGIC, BFAD_LPORT_RESET_STATS,\
	 bfa_ioctl_lport_reset_stats_t)
#define IOCMD_LPORT_GET_RPORTS		\
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_RPORTS, bfa_ioctl_lport_get_rports_t)

/* RPORT */
#define IOCMD_RPORT_ADD			\
	_IOWR(RPORT_MAGIC, BFAD_RPORT_ADD, bfa_ioctl_rport_t)
#define IOCMD_RPORT_REMOVE		\
	_IOWR(RPORT_MAGIC, BFAD_RPORT_REMOVE, bfa_ioctl_rport_t)
#define IOCMD_RPORT_GET_STATS		\
	_IOWR(RPORT_MAGIC, BFAD_RPORT_GET_STATS, bfa_ioctl_rport_stats_t)
#define IOCMD_RPORT_RESET_STATS		\
	_IOWR(RPORT_MAGIC, BFAD_RPORT_RESET_STATS,\
	 bfa_ioctl_rport_reset_stats_t)
#define IOCMD_RPORT_GET_ATTR		\
	_IOWR(RPORT_MAGIC, BFAD_RPORT_GET_ATTR, bfa_ioctl_rport_attr_t)
#define IOCMD_RPORT_GET_ADDR		\
	_IOWR(RPORT_MAGIC, BFAD_RPORT_GET_ADDR, bfa_ioctl_rport_scsi_addr_t)
#define IOCMD_RPORT_SET_SPEED		\
	_IOWR(RPORT_MAGIC, BFAD_RPORT_SET_SPEED, bfa_ioctl_rport_set_speed_t)

/* VPORT */
#define IOCMD_VPORT_CREATE		\
	_IOWR(VPORT_MAGIC, BFAD_VPORT_CREATE, bfa_ioctl_vport_create_t)
#define IOCMD_VPORT_DEL			\
	_IOWR(VPORT_MAGIC, BFAD_VPORT_DEL, bfa_ioctl_vport_t)
#define IOCMD_VPORT_START		\
	_IOWR(VPORT_MAGIC, BFAD_VPORT_START, bfa_ioctl_vport_t)
#define IOCMD_VPORT_STOP		\
	_IOWR(VPORT_MAGIC, BFAD_VPORT_STOP, bfa_ioctl_vport_t)
#define IOCMD_VPORT_GET_ATTR		\
	_IOWR(VPORT_MAGIC, BFAD_VPORT_GET_ATTR, bfa_ioctl_vport_attr_t)
#define IOCMD_VPORT_GET_STATS		\
	_IOWR(VPORT_MAGIC, BFAD_VPORT_GET_STATS, bfa_ioctl_vport_stats_t)
#define IOCMD_VPORT_RESET_STATS		\
	_IOWR(VPORT_MAGIC, BFAD_VPORT_RESET_STATS,\
	 bfa_ioctl_vport_reset_stats_t)

/* VF */
#define IOCMD_VF_ENABLE			\
	_IOWR(VF_MAGIC, BFAD_VF_ENABLE, bfa_ioctl_vf_t)
#define IOCMD_VF_DISABLE		\
	_IOWR(VF_MAGIC, BFAD_VF_DISABLE, bfa_ioctl_vf_t)
#define IOCMD_VF_ADD			\
	_IOWR(VF_MAGIC, BFAD_VF_ADD, bfa_ioctl_vf_add_t)
#define IOCMD_VF_DEL			\
	_IOWR(VF_MAGIC, BFAD_VF_DEL, bfa_ioctl_vf_t)
#define IOCMD_VF_LIST			\
	_IOWR(VF_MAGIC, BFAD_VF_LIST, bfa_ioctl_vf_list_t)
#define IOCMD_VF_LIST_ALL		\
	_IOWR(VF_MAGIC, BFAD_VF_LIST_ALL, bfa_ioctl_vf_list_t)
#define IOCMD_VF_GET_ATTR		\
	_IOWR(VF_MAGIC, BFAD_VF_GET_ATTR, bfa_ioctl_vf_attr_t)
#define IOCMD_VF_GET_STATS		\
	_IOWR(VF_MAGIC, BFAD_VF_GET_STATS, bfa_ioctl_vf_stats_t)
#define IOCMD_VF_RESET_STATS		\
	_IOWR(VF_MAGIC, BFAD_VF_RESET_STATS, bfa_ioctl_vf_reset_stats_t)

/* QOS */
#define IOCMD_QOS_ENABLE		\
	_IOWR(QOS_MAGIC, BFAD_QOS_ENABLE, bfa_ioctl_gen_t)
#define IOCMD_QOS_DISABLE		\
	_IOWR(QOS_MAGIC, BFAD_QOS_DISABLE, bfa_ioctl_gen_t)
#define IOCMD_QOS_GET_ATTR		\
	_IOWR(QOS_MAGIC, BFAD_QOS_GET_ATTR, bfa_ioctl_qos_attr_t)
#define IOCMD_QOS_GET_VC_ATTR		\
	_IOWR(QOS_MAGIC, BFAD_QOS_GET_VC_ATTR, bfa_ioctl_qos_vc_attr_t)
#define IOCMD_QOS_GET_STATS		\
	_IOWR(QOS_MAGIC, BFAD_QOS_GET_STATS, bfa_ioctl_qos_vc_attr_t)
#define IOCMD_QOS_RESET_STATS		\
	_IOWR(QOS_MAGIC, BFAD_QOS_RESET_STATS, bfa_ioctl_gen_t)
#define IOCMD_QOS_SET_BW		\
	_IOWR(QOS_MAGIC, BFAD_QOS_SET_BW, bfa_ioctl_qos_bw_t)


/* RATELIM */
#define IOCMD_RATELIM_ENABLE	\
	_IOWR(RATELIM_MAGIC, BFAD_RATELIM_ENABLE, bfa_ioctl_gen_t)
#define IOCMD_RATELIM_DISABLE	\
	_IOWR(RATELIM_MAGIC, BFAD_RATELIM_DISABLE, bfa_ioctl_gen_t)
#define IOCMD_RATELIM_DEF_SPEED	\
	_IOWR(RATELIM_MAGIC, BFAD_RATELIM_DEF_SPEED, bfa_ioctl_trl_speed_t)

/* FCPIM */
#define IOCMD_FCPIM_FAILOVER	\
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_FAILOVER, bfa_ioctl_fcpim_t)
#define IOCMD_FCPIM_QDEPTH	\
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_QDEPTH, bfa_ioctl_fcpim_qdepth_t)
#define IOCMD_FCPIM_MODSTATS	\
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_MODSTATS, bfa_ioctl_fcpim_modstats_t)
#define IOCMD_FCPIM_MODSTATSCLR	\
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_MODSTATSCLR,\
	 bfa_ioctl_fcpim_modstatsclr_t)
#define IOCMD_FCPIM_DEL_ITN_STATS	\
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_DEL_ITN_STATS,\
	 bfa_ioctl_fcpim_del_itn_stats_t)
#define IOCMD_FCPIM_LUNMASK_ENABLE  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_LUNMASK_ENABLE, bfa_ioctl_gen_t)
#define IOCMD_FCPIM_LUNMASK_DISABLE  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_LUNMASK_DISABLE, bfa_ioctl_gen_t)
#define IOCMD_FCPIM_LUNMASK_QUERY  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_LUNMASK_QUERY,    \
		bfa_ioctl_fcpim_lunmask_query_t)
#define IOCMD_FCPIM_LUNMASK_ADD    \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_LUNMASK_ADD, bfa_ioctl_fcpim_lunmask_t)
#define IOCMD_FCPIM_LUNMASK_DELETE  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_LUNMASK_DELETE, bfa_ioctl_fcpim_lunmask_t)
#define IOCMD_FCPIM_LUNMASK_CLEAR  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_LUNMASK_CLEAR, bfa_ioctl_gen_t)

#define IOCMD_FCPIM_THROTTLE_QUERY  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_THROTTLE_QUERY, \
		bfa_ioctl_fcpim_throttle_t)
#define IOCMD_FCPIM_THROTTLE_SET  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_THROTTLE_SET, bfa_ioctl_fcpim_throttle_t)
#define IOCMD_FCPIM_IOPF_START  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_IOPF_START, bfa_ioctl_fcpim_iopf_t)
#define IOCMD_FCPIM_IOPF_STOP  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_IOPF_STOP, bfa_ioctl_fcpim_iopf_t)
#define IOCMD_FCPIM_IOPF_SHOW	\
	 _IOWR(FCPIM_MAGIC, BFAD_FCPIM_IOPF_SHOW, bfa_ioctl_fcpim_iopf_show_t)
#define IOCMD_FCPIM_IOPF_QUERY  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_IOPF_QUERY, bfa_ioctl_fcpim_iopf_query_t)
#define IOCMD_FCPIM_IOPF_RESET  \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_IOPF_RESET, bfa_ioctl_fcpim_iopf_t)

/* BOOT */
#define IOCMD_BOOT_CFG	\
	_IOWR(BOOT_MAGIC, BFAD_BOOT_CFG, bfa_ioctl_boot_t)
#define IOCMD_BOOT_QUERY	\
	_IOWR(BOOT_MAGIC, BFAD_BOOT_QUERY, bfa_ioctl_boot_t)
#define IOCMD_PREBOOT_QUERY	\
	_IOWR(BOOT_MAGIC, BFAD_PREBOOT_QUERY, bfa_ioctl_preboot_t)

/* PASSTHRU */
#define IOCMD_SEND_SCSI		\
	_IOWR(PASSTHRU_MAGIC, BFAD_SEND_SCSI, bfa_ioctl_scsi_t)
#define IOCMD_SEND_FC		\
	_IOWR(PASSTHRU_MAGIC, BFAD_SEND_FC, bfa_ioctl_fcpt_t)

/* ITNIM */
#define IOCMD_ITNIM_GET_ATTR	\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_GET_ATTR, bfa_ioctl_itnim_attr_t)
#define IOCMD_ITNIM_GET_IOSTATS	\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_GET_IOSTATS, bfa_ioctl_itnim_iostats_t)
#define IOCMD_ITNIM_RESET_STATS	\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_RESET_STATS,\
	 bfa_ioctl_itnim_reset_stats_t)
#define IOCMD_ITNIM_GET_ITNSTATS	\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_GET_ITNSTATS,\
	bfa_ioctl_itnim_itnstats_t)
#define IOCMD_ITNIM_TRS_ENABLE	\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_TRS_ENABLE, bfa_ioctl_itnim_attr_t)
#define IOCMD_ITNIM_TRS_DISABLE	\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_TRS_DISABLE, bfa_ioctl_itnim_attr_t)
#define IOCMD_ITNIM_TRS_CLEAR		\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_TRS_CLEAR, bfa_ioctl_gen_t)
#define IOCMD_ITNIM_TRS_QUERY		\
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_TRS_QUERY, bfa_ioctl_itnim_trs_cfg_t)

/* AUTH */
#define IOCMD_AUTH_GET_ATTR		\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_GET_ATTR, bfa_ioctl_auth_attr_t)
#define IOCMD_AUTH_GET_STATS		\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_GET_STATS, bfa_ioctl_auth_stats_t)
#define IOCMD_AUTH_RESET_STATS		\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_RESET_STATS, bfa_ioctl_gen_t)
#define IOCMD_AUTH_REINIT		\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_REINIT, bfa_ioctl_gen_t)
#define IOCMD_AUTH_SET_POLICY		\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_SET_POLICY, bfa_ioctl_auth_policy_t)
#define IOCMD_AUTH_SET_ALGO		\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_SET_ALGO, bfa_ioctl_auth_algo_t)
#define IOCMD_AUTH_SET_GROUP		\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_SET_GROUP, bfa_ioctl_auth_group_t)
#define IOCMD_AUTH_SET_SECRETSRC	\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_SET_SECRETSRC,\
	 bfa_ioctl_auth_secretsource_t)
#define IOCMD_AUTH_SET_SECRETSTR	\
	_IOWR(AUTH_MAGIC, BFAD_AUTH_SET_SECRETST, bfa_ioctl_auth_secretstring_t)

/* FABRIC */
#define IOCMD_FABRIC_GET_LPORTS	\
	_IOWR(FABRIC_MAGIC, BFAD_FABRIC_GET_LPORTS,\
	bfa_ioctl_fabric_get_lports_t)

/* PBIND */
#define IOCMD_PBIND_SET		\
	_IOWR(PBIND_MAGIC, BFAD_PBIND_SET, bfa_ioctl_pbind_cap_t)

/* IPFC */
#define IOCMD_IPFC_GET_ATTR	\
	_IOWR(IPFC_MAGIC, BFAD_IPFC_GET_ATTR, bfa_ioctl_ipfc_attr_t)
#define IOCMD_IPFC_GET_STATS	\
	_IOWR(IPFC_MAGIC, BFAD_IPFC_GET_STATS, bfa_ioctl_ipfc_stats_t)
#define IOCMD_IPFC_RESET_STATS	\
	_IOWR(IPFC_MAGIC, BFAD_IPFC_RESET_STATS, bfa_ioctl_gen_t)
#define IOCMD_IPFC_ENABLE	\
	_IOWR(IPFC_MAGIC, BFAD_IPFC_ENABLE, bfa_ioctl_gen_t)
#define IOCMD_IPFC_DISABLE	\
	_IOWR(IPFC_MAGIC, BFAD_IPFC_DISABLE, bfa_ioctl_gen_t)

/* FC PORT */
#define IOCMD_FCPORT_GET_STATS	\
	_IOWR(FCPORT_MAGIC, BFAD_FCPORT_GET_STATS, bfa_ioctl_fcport_stats_t)
#define IOCMD_FCPORT_RESET_STATS  \
	 _IOWR(FCPORT_MAGIC, BFAD_FCPORT_RESET_STATS, bfa_ioctl_gen_t)
#define IOCMD_FCPORT_ENABLE  \
	 _IOWR(FCPORT_MAGIC, BFAD_FCPORT_ENABLE, bfa_ioctl_gen_t)
#define IOCMD_FCPORT_DISABLE  \
	 _IOWR(FCPORT_MAGIC, BFAD_FCPORT_DISABLE, bfa_ioctl_gen_t)

/* TRUNK */
#define IOCMD_TRUNK_ENABLE      \
	_IOWR(TRUNK_MAGIC, BFAD_TRUNK_ENABLE, bfa_ioctl_gen_t)
#define IOCMD_TRUNK_DISABLE     \
	_IOWR(TRUNK_MAGIC, BFAD_TRUNK_DISABLE, bfa_ioctl_gen_t)
#define IOCMD_TRUNK_GET_ATTR	\
	_IOWR(TRUNK_MAGIC, BFAD_TRUNK_GET_ATTR, bfa_ioctl_trunk_attr_t)

/* FAA */
#define IOCMD_FAA_ENABLE _IOWR(FAA_MAGIC, BFAD_FAA_ENABLE, \
				      bfa_ioctl_gen_t)
#define IOCMD_FAA_DISABLE _IOWR(FAA_MAGIC, BFAD_FAA_DISABLE, \
				      bfa_ioctl_gen_t)
#define IOCMD_FAA_QUERY _IOWR(FAA_MAGIC, BFAD_FAA_QUERY, \
				      bfa_ioctl_faa_attr_t)

/* TARGET MODE */
#define IOCMD_TARGET_MODE		\
	_IOWR(TARGET_MODE_MAGIC, BFAD_TARGET_MODE, bfa_tgt_mode_ioctl_t)

#endif /* BFAD_IOCTL_H */
