/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#include <linux/module.h>

#include "bfad_ioctl_cna.h"
#include "bfad_ioctl.h"
#include "bfad_drv.h"

#include "bfad_aen.h"
#include "ioctl/bfa_ioctl_pbind.h"
#ifdef CONFIG_COMPAT
#include <linux/ioctl32.h>

static int bfad_ioctl32_handler(unsigned int fd, unsigned int cmd,
	unsigned long arg, struct file *filep);
extern int bfad_ioctl(struct inode *inode, struct file *file,
	unsigned int cmd, unsigned long arg);

unsigned int ioctl_codes_table[] = {
	IOCMD_IOCFC_GET_ATTR,
	IOCMD_IOCFC_SET_INTR,
	IOCMD_LPORT_GET_ATTR,
	IOCMD_LPORT_GET_STATS,
	IOCMD_LPORT_GET_IOSTATS,
	IOCMD_LPORT_RESET_STATS,
	IOCMD_LPORT_GET_RPORTS,
	IOCMD_RPORT_ADD,
	IOCMD_RPORT_REMOVE,
	IOCMD_RPORT_GET_STATS,
	IOCMD_RPORT_RESET_STATS,
	IOCMD_RPORT_GET_ATTR,
	IOCMD_RPORT_GET_ADDR,
	IOCMD_RPORT_SET_SPEED,


	IOCMD_VPORT_CREATE,
	IOCMD_VPORT_DEL,
	IOCMD_VPORT_START,
	IOCMD_VPORT_STOP,
	IOCMD_VPORT_GET_ATTR,
	IOCMD_VPORT_GET_STATS,
	IOCMD_VPORT_RESET_STATS,

	IOCMD_VF_ENABLE,
	IOCMD_VF_DISABLE,
	IOCMD_VF_ADD,
	IOCMD_VF_DEL,
	IOCMD_VF_LIST,
	IOCMD_VF_LIST_ALL,
	IOCMD_VF_GET_ATTR,
	IOCMD_VF_GET_STATS,
	IOCMD_VF_RESET_STATS,

	IOCMD_FLASH_GET_ATTR,
	IOCMD_FLASH_ERASE_PART,
	IOCMD_FLASH_UPDATE_PART,
	IOCMD_FLASH_READ_PART,
	IOCMD_FLASH_MFG_INFO,
	IOCMD_FLASH_ENABLE_OPTROM,
	IOCMD_FLASH_DISABLE_OPTROM,
	IOCMD_FLASH_GET_CFG_BOOTVER,

	IOCMD_QOS_ENABLE,
	IOCMD_QOS_DISABLE,
	IOCMD_QOS_GET_ATTR,
	IOCMD_QOS_GET_VC_ATTR,
	IOCMD_QOS_GET_STATS,
	IOCMD_QOS_RESET_STATS,
	IOCMD_QOS_SET_BW,

	IOCMD_RATELIM_ENABLE,
	IOCMD_RATELIM_DISABLE,
	IOCMD_RATELIM_DEF_SPEED,

	IOCMD_FCPIM_FAILOVER,
	IOCMD_FCPIM_QDEPTH,
	IOCMD_FCPIM_MODSTATS,
	IOCMD_FCPIM_MODSTATSCLR,
	IOCMD_FCPIM_DEL_ITN_STATS,
	IOCMD_FCPIM_LUNMASK_ADD,
	IOCMD_FCPIM_LUNMASK_DELETE,
	IOCMD_FCPIM_LUNMASK_ENABLE,
	IOCMD_FCPIM_LUNMASK_DISABLE,
	IOCMD_FCPIM_LUNMASK_CLEAR,
	IOCMD_FCPIM_LUNMASK_QUERY,

	IOCMD_FCPIM_THROTTLE_QUERY,
	IOCMD_FCPIM_THROTTLE_SET,

	IOCMD_FCPIM_IOPF_START,
	IOCMD_FCPIM_IOPF_STOP,
	IOCMD_FCPIM_IOPF_SHOW,
	IOCMD_FCPIM_IOPF_QUERY,
	IOCMD_FCPIM_IOPF_RESET,

	IOCMD_BOOT_CFG,
	IOCMD_BOOT_QUERY,

	IOCMD_SEND_SCSI,
	IOCMD_SEND_FC,

	IOCMD_ITNIM_GET_ATTR,
	IOCMD_ITNIM_GET_IOSTATS,
	IOCMD_ITNIM_RESET_STATS,
	IOCMD_ITNIM_GET_ITNSTATS,
	IOCMD_ITNIM_TRS_ENABLE,
	IOCMD_ITNIM_TRS_DISABLE,
	IOCMD_ITNIM_TRS_CLEAR,
	IOCMD_ITNIM_TRS_QUERY,

	IOCMD_AUTH_GET_ATTR,
	IOCMD_AUTH_GET_STATS,
	IOCMD_AUTH_RESET_STATS,
	IOCMD_AUTH_REINIT,
	IOCMD_AUTH_SET_POLICY,
	IOCMD_AUTH_SET_ALGO,
	IOCMD_AUTH_SET_GROUP,
	IOCMD_AUTH_SET_SECRETSRC,
	IOCMD_AUTH_SET_SECRETSTR,

	IOCMD_FABRIC_GET_LPORTS,

	IOCMD_PBIND_SET,

	IOCMD_IPFC_GET_ATTR,
	IOCMD_IPFC_GET_STATS,
	IOCMD_IPFC_RESET_STATS,
	IOCMD_IPFC_ENABLE,
	IOCMD_IPFC_DISABLE,

	IOCMD_FCPORT_GET_STATS,
	IOCMD_FCPORT_RESET_STATS,

	/* CNA IOCTLs */
	IOCMD_IOC_GET_VERSION,
	IOCMD_IOC_GET_INSTS,
	IOCMD_IOC_ENABLE,
	IOCMD_IOC_DISABLE,
	IOCMD_IOC_GET_ATTR,
	IOCMD_IOC_GET_INFO,
	IOCMD_IOC_GET_STATS,
	IOCMD_IOC_RESET_STATS,
	IOCMD_IOC_SET_ADAPTER_NAME,
	IOCMD_IOC_SET_PORT_NAME,
	IOCMD_IOC_FW_SIG_INV,

	/* DIAG */
	IOCMD_DIAG_REGRD,
	IOCMD_DIAG_MEMRD,
	IOCMD_DIAG_REGWR,
	IOCMD_DIAG_TEMP,
	IOCMD_DIAG_POST,
	IOCMD_DIAG_MEMTEST,
	IOCMD_DIAG_LOOPBACK,
	IOCMD_DIAG_FWPING,
	IOCMD_DIAG_QUEUETEST,
	IOCMD_DIAG_POM,
	IOCMD_DIAG_SFP,
	IOCMD_DIAG_LED,
	IOCMD_DIAG_BEACON_LPORT,
	IOCMD_DIAG_LL_LOOPBACK,
	IOCMD_DIAG_LB_STAT,
	IOCMD_DIAG_DPORT_ENABLE,
	IOCMD_DIAG_DPORT_DISABLE,
	IOCMD_DIAG_DPORT_SHOW,
	IOCMD_DIAG_DPORT_START,
	/* DEBUG */
	IOCMD_DEBUG_DRV_TRACE,
	IOCMD_DEBUG_DRV_STATE,
	IOCMD_DEBUG_FW_TRACE,
	IOCMD_DEBUG_FW_STATE,
	IOCMD_DEBUG_FW_STATE_CLR,
	IOCMD_DEBUG_FW_CORE,
	IOCMD_DEBUG_PORTLOG,
	IOCMD_DEBUG_PORTLOG_CLR,
	IOCMD_DEBUG_PORTLOG_CTL,
	IOCMD_DEBUG_START_DTRC,
	IOCMD_DEBUG_STOP_DTRC,

	/* LOG */
	IOCMD_LOG_SET_LEVEL,
	IOCMD_LOG_GET_LEVEL,

	/* AEN */
	IOCMD_AEN_GET,
	IOCMD_AEN_UPDATE,

	/* PORT */
	IOCMD_PORT_ENABLE,
	IOCMD_PORT_DISABLE,
	IOCMD_PORT_GET_ATTR,
	IOCMD_PORT_GET_STATS,
	// IOCMD_PORT_GET_FC4STATS,
	// IOCMD_PORT_GET_FCPMAP,
	IOCMD_PORT_RESET_STATS,
	IOCMD_PORT_CFG_TOPO,
	IOCMD_PORT_CFG_SPEED,
	IOCMD_PORT_CFG_ALPA,
	IOCMD_PORT_CLR_ALPA,
	IOCMD_PORT_CFG_MAXFRSZ,
	IOCMD_PORT_CFG_RNID,
	IOCMD_PORT_GET_RNID,
	IOCMD_PORT_BBCR_ENABLE,
	IOCMD_PORT_BBCR_DISABLE,
	IOCMD_PORT_BBCR_GET_ATTR,

	/* CEE */
	IOCMD_CEE_GET_ATTR,
	IOCMD_CEE_GET_STATS,
	IOCMD_CEE_RESET_STATS,


};

static int reg_ioctl_cmd;

static int
bfad_ioctl32_handler(unsigned int fd, unsigned int cmd,
	unsigned long arg, struct file *filep)
{
	struct inode *inode = filep->f_dentry->d_inode;
	int ret = 0;

	if (inode == NULL) {
		return -EROFS;
	}

	ret = bfad_ioctl(inode, filep, cmd, arg);

	return ret;
}
#endif

int
bfad_os_ioctl32_register(void)
{
#ifdef CONFIG_COMPAT
	int ret = 0, count = 0;

	/*
	 * Mark that none of the IOCTLs are registered
	 */
	reg_ioctl_cmd = -1;

	for (count = 0;
	     count < (sizeof(ioctl_codes_table)/sizeof(ioctl_codes_table[0]));
	     count++) {
		ret = register_ioctl32_conversion(ioctl_codes_table[count],
				bfad_ioctl32_handler);
		if (ret != 0) {
			printk(KERN_WARNING "IOCTL32 conversion registration"
				" failed \n");
			/*
			 * Mark the IOCTL32 registration failure point
			 */
			reg_ioctl_cmd = ioctl_codes_table[count];
			return -1;
		}
	}

	/*
	 * Indicate that all the IOCTL32 registration succeeded
	 * by clearing the flag.
	 */
	reg_ioctl_cmd = 0;

#endif

	return 0;
}


void
bfad_os_ioctl32_unregister(void)
{
#ifdef CONFIG_COMPAT
	int count = 0;

	if (reg_ioctl_cmd == -1) {
		/*
		 * Nothing to unregister for IOCTL32
		 */
		return;
	}

	for (count = 0;
	     count < (sizeof(ioctl_codes_table)/sizeof(ioctl_codes_table[0]));
	     count++) {
		/**
		 * If we hit the below condition, it means that there was
		 * a failure in registering the IOCTL32. So, don't unregister.
		 */
		if (ioctl_codes_table[count] == reg_ioctl_cmd) {
			/*
			 * When unregistration is done, mark the flag to
			 * indicate the same.
			 */
			reg_ioctl_cmd = -1;
			return;
		}
		unregister_ioctl32_conversion(ioctl_codes_table[count]);
	}

	/*
	 * When unregistration is done, mark the flag to
	 * indicate the same.
	 */
	reg_ioctl_cmd = -1;

	return;
#endif
}
