/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfad_os.c Linux driver OS specific calls.
 */

#include "bfa_os_inc.h"
#include "bfad_drv.h"

void
bfa_os_gettimeofday(struct bfa_timeval_s *tv)
{
	struct timeval  tmp_tv;

	do_gettimeofday(&tmp_tv);
	tv->tv_sec = (uint32_t) tmp_tv.tv_sec;
	tv->tv_usec = (uint32_t) tmp_tv.tv_usec;
}

#ifdef __VMKERNEL_MODULE__
void
bfa_os_printf(struct bfa_log_mod_s *log_mod, uint32_t msg_id,
			va_list params,
			const char *fmt, ...)
{
	va_list ap;
#define BFA_STRING_256  256
	char tmp[BFA_STRING_256];

	va_start(ap, fmt);
	vsnprintf(tmp, sizeof(tmp), fmt, ap);
	va_end(ap);

	printk(tmp);
}
#else
void
bfa_os_printf(struct bfa_log_mod_s *log_mod, uint32_t msg_id,
			va_list params,
			const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vprintk(fmt, ap);
	va_end(ap);
}
#endif

uint32_t
bfa_os_get_instance_id(struct bfad_s *bfad)
{
	return (bfad->inst_no);
}

/**
 * @}
 */
