/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#include <bfa/bfa_modules.h>

/**
 * BFA module list terminated by NULL
 */
struct bfa_module_s *hal_mods[] = {
	&hal_mod_fcdiag,
	&hal_mod_sgpg,
	&hal_mod_fcport,
	&hal_mod_fcxp,
	&hal_mod_lps,
	&hal_mod_uf,
	&hal_mod_rport,
	&hal_mod_fcp,
	&hal_mod_dconf,
#ifdef BFA_CFG_PBIND
	&hal_mod_pbind,
#endif
	NULL
};

/**
 * Message handlers for various modules.
 */
bfa_isr_func_t  bfa_isrs[BFI_MC_MAX] = {
	bfa_isr_unhandled,	/* NONE */
	bfa_isr_unhandled,	/* BFI_MC_IOC */
	bfa_fcdiag_intr,	/* BFI_MC_DIAG */
	bfa_isr_unhandled,	/* BFI_MC_FLASH */
	bfa_isr_unhandled,	/* BFI_MC_CEE */
	bfa_fcport_isr,		/* BFI_MC_FCPORT */
	bfa_isr_unhandled,	/* BFI_MC_IOCFC */
	bfa_isr_unhandled,	/* BFI_MC_LL */
	bfa_uf_isr,		/* BFI_MC_UF */
	bfa_fcxp_isr,		/* BFI_MC_FCXP */
	bfa_lps_isr,		/* BFI_MC_LPS */
	bfa_rport_isr,		/* BFI_MC_RPORT */
	bfa_itn_isr,		/* BFI_MC_ITN */
	bfa_isr_unhandled,	/* BFI_MC_IOIM_READ */
	bfa_isr_unhandled,	/* BFI_MC_IOIM_WRITE */
	bfa_isr_unhandled,	/* BFI_MC_IOIM_IO */
	bfa_ioim_isr,		/* BFI_MC_IOIM */
	bfa_ioim_good_comp_isr,	/* BFI_MC_IOIM_IOCOM */
	bfa_tskim_isr,		/* BFI_MC_TSKIM */
	bfa_isr_unhandled,	/* BFI_MC_SBOOT */
	bfa_isr_unhandled,	/* BFI_MC_IPFC */
	bfa_isr_unhandled,	/* BFI_MC_PORT */
	bfa_isr_unhandled,	/* BFI_MC_TIO_DATA_XFERED */
	bfa_isr_unhandled,	/* BFI_MC_TIO_IO */
	bfa_isr_unhandled,	/* BFI_MC_TIO */
	bfa_isr_unhandled,	/* --------- */
	bfa_isr_unhandled,	/* --------- */
	bfa_isr_unhandled,	/* --------- */
	bfa_isr_unhandled,	/* --------- */
	bfa_isr_unhandled,	/* --------- */
	bfa_isr_unhandled,	/* --------- */
	bfa_isr_unhandled,	/* --------- */
};


/**
 * Message handlers for mailbox command classes
 */
bfa_ioc_mbox_mcfunc_t  bfa_mbox_isrs[BFI_MC_MAX] = {
	NULL,
	NULL,			/* BFI_MC_IOC   */
	NULL,			/* BFI_MC_DIAG  */
	NULL,			/* BFI_MC_FLASH */
	NULL,			/* BFI_MC_CEE   */
	NULL,			/* BFI_MC_PORT  */
	bfa_iocfc_isr,		/* BFI_MC_IOCFC */
	NULL,
};

/**
 *
 */

static void
bfa_com_port_attach(struct bfa_s *bfa)
{
	struct bfa_port_s	*port = &bfa->modules.port;
	struct bfa_mem_dma_s	*port_dma = BFA_MEM_PORT_DMA(bfa);

	bfa_port_attach(port, &bfa->ioc, bfa, bfa->trcmod, bfa->logm);
	bfa_port_mem_claim(port, port_dma->kva_curp, port_dma->dma_curp);
}

static void
bfa_com_cee_attach(struct bfa_s *bfa)
{
	struct bfa_cee_s  *cee = &bfa->modules.cee;
	struct bfa_mem_dma_s	*cee_dma = BFA_MEM_CEE_DMA(bfa);

	/**
	 * CEE module attach.
	 */
	bfa_cee_log_trc_set(cee, bfa->trcmod, bfa->logm);
	bfa_cee_attach(cee, &bfa->ioc, bfa);
	bfa_cee_mem_claim(cee, cee_dma->kva_curp, cee_dma->dma_curp);
}

static void
bfa_com_diag_attach(struct bfa_s *bfa)
{
	bfa_diag_mod_t	*diag = BFA_DIAG_MOD(bfa);
	struct bfa_mem_dma_s	*diag_dma = BFA_MEM_DIAG_DMA(bfa);

	/**
	 * Diag module attach.
	 */
	bfa_diag_attach(diag, &bfa->ioc, bfa, bfa_fcport_beacon, bfa->trcmod);
	bfa_diag_memclaim(diag, diag_dma->kva_curp, diag_dma->dma_curp);
}

/**
 * SFP command attach
 */
static void
bfa_com_sfp_attach(struct bfa_s *bfa)
{
	bfa_sfp_t	*sfp = BFA_SFP_MOD(bfa);
	struct bfa_mem_dma_s	*sfp_dma = BFA_MEM_SFP_DMA(bfa);

	bfa_sfp_attach(sfp, &bfa->ioc, bfa, bfa->trcmod);
	bfa_sfp_memclaim(sfp, sfp_dma->kva_curp, sfp_dma->dma_curp);
}

/**
 * Flash command attach
 */
static void
bfa_com_flash_attach(struct bfa_s *bfa, bfa_boolean_t mincfg)
{
	bfa_flash_t	*flash = BFA_FLASH(bfa);
	struct bfa_mem_dma_s	*flash_dma = BFA_MEM_FLASH_DMA(bfa);

	bfa_flash_attach(flash, &bfa->ioc, bfa, bfa->trcmod, mincfg);
	bfa_flash_memclaim(flash, flash_dma->kva_curp,
			   flash_dma->dma_curp, mincfg);
}

/**
 * Phy command attach
 */
static void
bfa_com_phy_attach(struct bfa_s *bfa, bfa_boolean_t mincfg)
{
	bfa_phy_t	*phy = BFA_PHY(bfa);
	struct bfa_mem_dma_s	*phy_dma = BFA_MEM_PHY_DMA(bfa);

	bfa_phy_attach(phy, &bfa->ioc, bfa, bfa->trcmod, mincfg);
	bfa_phy_memclaim(phy, phy_dma->kva_curp, phy_dma->dma_curp, mincfg);
}

/**
 * fru command attach
 */
static void
bfa_com_fru_attach(struct bfa_s *bfa, bfa_boolean_t mincfg)
{
	bfa_fru_t	*fru = BFA_FRU(bfa);
	struct bfa_mem_dma_s	*fru_dma = BFA_MEM_FRU_DMA(bfa);

	bfa_fru_attach(fru, &bfa->ioc, bfa, bfa->trcmod, mincfg);
	bfa_fru_memclaim(fru, fru_dma->kva_curp, fru_dma->dma_curp, mincfg);
}

/**
 * ablk module attach
 */
static void
bfa_com_ablk_attach(struct bfa_s *bfa)
{
	struct bfa_ablk_s	*ablk = &bfa->modules.ablk;
	struct bfa_mem_dma_s	*ablk_dma = BFA_MEM_ABLK_DMA(bfa);

	bfa_ablk_attach(ablk, &bfa->ioc);
	bfa_ablk_memclaim(ablk, ablk_dma->kva_curp, ablk_dma->dma_curp);
}

/**
 * Common modules DMA memory requirement.
 */
void
bfa_com_meminfo(bfa_boolean_t mincfg, struct bfa_meminfo_s *meminfo,
		struct bfa_s *bfa)
{
	struct bfa_mem_dma_s *flash_dma = bfa ? BFA_MEM_FLASH_DMA(bfa) : NULL;
	struct bfa_mem_dma_s *phy_dma = bfa ? BFA_MEM_PHY_DMA(bfa) : NULL;
	struct bfa_mem_dma_s *fru_dma = bfa ? BFA_MEM_FRU_DMA(bfa) : NULL;
	struct bfa_mem_dma_s *diag_dma = bfa ? BFA_MEM_DIAG_DMA(bfa) : NULL;
	struct bfa_mem_dma_s *sfp_dma = bfa ? BFA_MEM_SFP_DMA(bfa) : NULL;
	struct bfa_mem_dma_s *cee_dma = bfa ? BFA_MEM_CEE_DMA(bfa) : NULL;
	struct bfa_mem_dma_s *port_dma = bfa ? BFA_MEM_PORT_DMA(bfa) : NULL;
	struct bfa_mem_dma_s *ablk_dma = bfa ? BFA_MEM_ABLK_DMA(bfa) : NULL;

	/* dma info setup */
	bfa_mem_dma_setup(meminfo, flash_dma, bfa_flash_meminfo(mincfg));
	bfa_mem_dma_setup(meminfo, phy_dma, bfa_phy_meminfo(mincfg));
	bfa_mem_dma_setup(meminfo, fru_dma, bfa_fru_meminfo(mincfg));
	bfa_mem_dma_setup(meminfo, diag_dma, bfa_diag_meminfo());
	bfa_mem_dma_setup(meminfo, sfp_dma, bfa_sfp_meminfo());
	bfa_mem_dma_setup(meminfo, cee_dma, bfa_cee_meminfo());
	bfa_mem_dma_setup(meminfo, port_dma, bfa_port_meminfo());
	bfa_mem_dma_setup(meminfo, ablk_dma, bfa_ablk_meminfo());
}

void
bfa_com_attach(struct bfa_s *bfa, bfa_boolean_t mincfg)
{
	bfa_com_flash_attach(bfa, mincfg);
	bfa_com_phy_attach(bfa, mincfg);
	bfa_com_fru_attach(bfa, mincfg);
	bfa_com_diag_attach(bfa);
	bfa_com_sfp_attach(bfa);
	bfa_com_cee_attach(bfa);
	bfa_com_port_attach(bfa);
	bfa_com_ablk_attach(bfa);
}
