/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCP_H__
#define __BFA_FCP_H__

#include <bfa/bfa_fcpim.h>

#define BFA_IO_MAX	BFI_IO_MAX

#define BFA_FWTIO_MIN	8
#define BFA_FWTIO_MAX	2000

typedef struct bfa_iotag_s {
	struct bfa_q_s	qe;	/*!< queue element	*/
	uint16_t	tag;	/*!< FW IO tag		*/
}bfa_iotag_t;

typedef struct bfa_itn_s {
	bfa_isr_func_t isr;
}bfa_itn_t;

/* Max FCP dma segs required */
#define BFA_FCP_DMA_SEGS	BFI_IOIM_SNSBUF_SEGS

struct bfa_fcp_mod_s {
	struct bfa_s		*bfa;
	struct bfa_q_s		iotag_ioim_free_q;   /*!< free IO resources  */
	struct bfa_q_s		iotag_tio_free_q;   /*!< free IO resources  */
	struct bfa_q_s		iotag_unused_q;	     /* unused IO resources */
	struct bfa_iotag_s	*iotag_arr;
	struct bfa_itn_s	*itn_arr;
	int			max_ioim_reqs;
	int			num_ioim_reqs;
	int			num_fwtio_reqs;
	int			num_itns;
	struct bfa_dma_s	snsbase[BFA_FCP_DMA_SEGS];
	struct bfa_fcpim_s	fcpim;
	void			*fcptm;
	struct bfa_mem_dma_s	dma_seg[BFA_FCP_DMA_SEGS];
	struct bfa_mem_kva_s	kva_seg;
	int			throttle_update_required;
};

void bfa_itn_create(struct bfa_s *bfa, struct bfa_rport_s *rport,
		void (*isr)(struct bfa_s *bfa, struct bfi_msg_s *m));
void bfa_itn_isr(struct bfa_s *bfa, struct bfi_msg_s *m);

bfa_status_t bfa_fcp_alloc_iotag(struct bfa_s *bfa,
		itn_role_t tole, uint16_t *tag);
void bfa_fcp_release_iotag(struct bfa_s *bfa, uint16_t tag);
void bfa_fcp_res_recfg(struct bfa_s *bfa, uint16_t num_ioim_fw,
			uint16_t max_ioim_fw);

#define BFA_FCP_MOD(_hal) (&(_hal)->modules.fcp_mod)
#define BFA_MEM_FCP_KVA(__bfa) (&(BFA_FCP_MOD(__bfa)->kva_seg))
#define BFA_IOTAG_FROM_TAG(_fcp, _tag)	\
	(&fcp->iotag_arr[(_tag & BFA_IOIM_IOTAG_MASK)])
#define BFA_ITN_FROM_TAG(_fcp, _tag) \
	((_fcp)->itn_arr + ((_tag) & ((_fcp)->num_itns -1)))
#define BFA_SNSINFO_FROM_TAG(_fcp, _tag) \
	bfa_mem_get_dmabuf_kva(_fcp, (_tag & BFA_IOIM_IOTAG_MASK),	\
	BFI_IOIM_SNSLEN)

#endif
