/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCPIM_H__
#define __BFA_FCPIM_H__

#include <bfa/bfa.h>
#include <bfa/bfa_svc.h>
#include <bfi/bfi_ms.h>
#include <bfi/bfi_dconf.h>
#include <defs/bfa_defs_svc.h>
#include <cs/bfa_cs.h>


#define BFA_ITNIM_MIN   32
#define BFA_ITNIM_MAX   1024

#define BFA_IOIM_MIN	8
#define BFA_IOIM_MAX	2000

#define BFA_TSKIM_MIN   4
#define BFA_TSKIM_MAX   512
#define BFA_FCPIM_PATHTOV_DEF	(30 * 1000)	/* in millisecs */
#define BFA_FCPIM_PATHTOV_MAX	(90 * 1000)	/* in millisecs */

#define BFA_IOIM_RETRY_TAG_OFFSET 11
#define BFA_IOIM_IOTAG_MASK 0x07ff /* 2K IOs */
#define BFA_IOIM_RETRY_MAX 7

/* 5 Buckets are from 512 bytes to 2MB IO sz */
static inline uint32_t
bfa_ioim_get_index(uint32_t n) {
	int pos = 0;
	if (n < 1 << 9) return 0;
	if (n >= (1UL)<<16) { return  BFA_IOBUCKET_MAX - 1; }
	/* for > 64k (16 bits)IO sz return 4 */
	n >>= 8; /* lower 8 bits are gone */
	if (n >= 1 << 5) { n >>= 5; pos += 3; }
	if (n >= 1 << 4) { n >>= 4; pos += 2; }
	if (n >= 1 << 1) { pos += 1; }
	/* for 512 byet or below return  0 */
	/* for more than 512 (9 bits) return 1 */
	/* for more than 4k (13 bits) return 2 */

	return (pos);
}

/*
 * forward declarations
 */
struct bfa_ioim_s;
struct bfa_tskim_s;
struct bfad_ioim_s;
typedef struct bfad_ioim_s bfad_ioim_t;
struct bfad_tskim_s;
typedef struct bfad_tskim_s bfad_tskim_t;

typedef void    (*bfa_fcpim_profile_t) (struct bfa_ioim_s *ioim);
typedef bfa_boolean_t (*bfa_ioim_lm_proc_rsp_data_t) (struct bfa_ioim_s *ioim);

struct bfa_fcpim_iopf_qe_s
{
	struct bfa_q_s 		qe;		/*!< queue elememt	*/
	struct bfa_fcpim_iopf_s iopf;
};
typedef struct bfa_fcpim_iopf_qe_s bfa_fcpim_iopf_qe_t;


struct bfa_fcpim_s {
	struct bfa_s 	*bfa;
	struct bfa_fcp_mod_s	*fcp;
	struct bfa_itnim_s 	*itnim_arr;
	struct bfa_ioim_s 	*ioim_arr;
	struct bfa_ioim_sp_s	*ioim_sp_arr;
	struct bfa_tskim_s 	*tskim_arr;
	struct bfa_fcpim_iopf_qe_s *iopf_arr; /*!< for IO profile */
	int			num_itnims;
	int			num_tskim_reqs;
	uint32_t		path_tov;
	uint16_t		q_depth;
	uint8_t			reqq;		/*!< Request queue to be used */
	uint8_t			lun_masking_pending;
	struct bfa_q_s 		itnim_q;	/*!< queue of active itnim */
	struct bfa_q_s 		ioim_resfree_q; /*!< IOs waiting for f/w */
	struct bfa_q_s 		ioim_comp_q;	/*!< IO global comp Q	*/
	struct bfa_q_s 		tskim_free_q;
	struct bfa_q_s		tskim_unused_q;	/* Unused tskim Q */
	struct bfa_q_s		iopf_free_q; /* bfa_fcpim_iopf_qe_t entry */
	struct bfa_q_s		iopf_active_q; /* bfa_fcpim_iopf_qe_t  */
	uint32_t		ios_active;	/*!< current active IOs	*/
	uint32_t		delay_comp;
	struct bfa_fcpim_del_itn_stats_s del_itn_stats;
	bfa_boolean_t		ioredirect;
	bfa_fcpim_profile_t     profile_comp;
	bfa_fcpim_profile_t     profile_start;
};
typedef struct bfa_fcpim_s bfa_fcpim_t;

/**
 * BFA IO (initiator mode)
 */
struct bfa_ioim_s {
	struct bfa_q_s 		qe;		/*!< queue elememt	*/
	bfa_sm_t		sm; 		/*!< BFA ioim state machine   */
	struct bfa_s 		*bfa;		/*!< BFA module	*/
	struct bfa_fcpim_s	*fcpim;		/*!< parent fcpim module */
	struct bfa_itnim_s 	*itnim;		/*!< i-t-n nexus for this IO  */
	struct bfad_ioim_s 	*dio;		/*!< driver IO handle	*/
	uint16_t		iotag;		/*!< FWI IO tag	*/
	uint16_t		abort_tag;	/*!< unqiue abort request tag */
	uint16_t		nsges;		/*!< number of SG elements */
	uint16_t		nsgpgs;		/*!< number of SG pages	*/
	struct bfa_sgpg_s	*sgpg;		/*!< first SG page	*/
	struct bfa_q_s 		sgpg_q;		/*!< allocated SG pages	*/
	struct bfa_cb_qe_s	hcb_qe;		/*!< bfa callback qelem	*/
	bfa_cb_cbfn_t		io_cbfn;	/*!< IO completion handler */
	struct bfa_ioim_sp_s *iosp;		/*!< slow-path IO handling */
	struct bfa_fcpim_iopf_s *iopf;
	uint8_t 		reqq;		/*!< Request queue for I/O */
	uint8_t 		mode;		/*!< IO is passthrough or not */
	uint64_t		latency;	/*!< Time for completion */
	uint64_t start_time;			/*!< IO's Profile start val */
	bfa_ioim_lm_proc_rsp_data_t	proc_rsp_data;	/*!< RSP data adjust */
};
typedef struct bfa_ioim_s bfa_ioim_t;


struct bfa_ioim_sp_s {
	struct bfi_msg_s 	comp_rspmsg;	/*!< IO comp f/w response */
	struct bfa_sgpg_wqe_s sgpg_wqe;	/*!< waitq elem for sgpg	*/
	struct bfa_reqq_wait_s reqq_wait;	/*!< to wait for room in reqq */
	bfa_boolean_t		abort_explicit;	/*!< aborted by OS	*/
	struct bfa_tskim_s	*tskim;		/*!< Relevant TM cmd	*/
};

/**
 * BFA Task management command (initiator mode)
 */
struct bfa_tskim_s {
	struct bfa_q_s	qe;
	bfa_sm_t		sm;
	struct bfa_s	*bfa;	/*!< BFA module  */
	struct bfa_fcpim_s	*fcpim;	/*!< parent fcpim module	*/
	struct bfa_itnim_s	*itnim;	/*!< i-t-n nexus for this IO  */
	struct bfad_tskim_s	*dtsk;   /*!< driver task mgmt cmnd	*/
	bfa_boolean_t	notify;	/*!< notify itnim on TM comp  */
	lun_t	lun;	/*!< lun if applicable	*/
	fcp_tm_cmnd_t	tm_cmnd;	/*!< task management command  */
	uint16_t	tsk_tag;	/*!< FWI IO tag	*/
	uint8_t	tsecs;	/*!< timeout in seconds	*/
	struct bfa_reqq_wait_s  reqq_wait;   /*!< to wait for room in reqq */
	struct bfa_q_s	io_q;	/*!< queue of affected IOs	*/
	struct bfa_wc_s	wc;	/*!< waiting counter	*/
	struct bfa_cb_qe_s	hcb_qe;	/*!< bfa callback qelem	*/
	enum bfi_tskim_status   tsk_status;  /*!< TM status	*/
};
typedef struct bfa_tskim_s bfa_tskim_t;


/**
 * BFA i-t-n (initiator mode)
 */
struct bfa_itnim_s {
	struct bfa_q_s	qe;		/*!< queue element	*/
	bfa_sm_t	  sm;		/*!< i-t-n im BFA state machine  */
	struct bfa_s	*bfa;		/*!< bfa instance	*/
	struct bfa_rport_s *rport;	/*!< bfa rport	*/
	void	*ditn;		/*!< driver i-t-n structure	*/
	struct bfi_mhdr_s	mhdr;	/*!< pre-built mhdr	*/
	uint8_t	msg_no;		/*!< itnim/rport firmware handle */
	uint8_t	reqq;		/*!< CQ for requests	*/
	struct bfa_cb_qe_s	hcb_qe;	/*!< bfa callback qelem	*/
	struct bfa_q_s pending_q;	/*!< queue of pending IO requests */
	struct bfa_q_s io_q;		/*!< queue of active IO requests */
	struct bfa_q_s io_cleanup_q;	/*!< IO being cleaned up	*/
	struct bfa_q_s tsk_q;		/*!< queue of active TM commands */
	struct bfa_q_s  delay_comp_q; /*!< queue of failed inflight cmds */
	bfa_boolean_t   seq_rec;	/*!< SQER supported	*/
	bfa_boolean_t   is_online;	/*!< itnim is ONLINE for IO	*/
	bfa_boolean_t   iotov_active;	/*!< IO TOV timer is active	 */
	bfa_boolean_t   lun_profile_active;	/*!< LUN profile active  */
	struct bfa_wc_s	wc;	/*!< waiting counter	*/
	struct bfa_timer_s timer;	/*!< pending IO TOV		 */
	struct bfa_reqq_wait_s reqq_wait; /*!< to wait for room in reqq */
	struct bfa_fcpim_s *fcpim;	/*!< fcpim module	*/
	struct bfa_itnim_iostats_s	stats;
	struct bfa_itnim_latency_s  iolat;
	bfa_boolean_t	trs_blocked; /*!< Target Reset disabled */
};
typedef struct bfa_itnim_s bfa_itnim_t;


#define bfa_itnim_is_online(_itnim) (_itnim)->is_online
#define BFA_FCPIM(_hal) (&(_hal)->modules.fcp_mod.fcpim)
#define BFA_IOIM_TAG_2_ID(_iotag) ((_iotag) & BFA_IOIM_IOTAG_MASK)
#define BFA_IOIM_FROM_TAG(_fcpim, _iotag)	\
	(&fcpim->ioim_arr[(_iotag & BFA_IOIM_IOTAG_MASK)])
#define BFA_TSKIM_FROM_TAG(_fcpim, _tmtag)	\
	(&fcpim->tskim_arr[_tmtag & (fcpim->num_tskim_reqs - 1)])

#define BFA_TM_CMD_IS_TRS(_cmnd) (_cmnd == FCP_TM_TARGET_RESET)
#define BFA_ITNIM_IS_TRS_DISABLED(_itnim) (_itnim->trs_blocked)
#define BFA_ITNIM_GET_LPWWN(_itnim)	\
	(BFA_LPS_FROM_TAG(BFA_LPS_MOD(_itnim->bfa),	\
			_itnim->rport->rport_info.lp_tag))->pwwn

#define bfa_ioim_update_iotag(__ioim) do {				\
	uint16_t k = (__ioim)->iotag >> BFA_IOIM_RETRY_TAG_OFFSET;	\
	k++; (__ioim)->iotag &= BFA_IOIM_IOTAG_MASK;		\
	(__ioim)->iotag |= k << BFA_IOIM_RETRY_TAG_OFFSET;		\
} while (0)

#define BFA_IOPF_QE_FROM_ID(_fcpim, _iopfid)		\
	(&(fcpim->iopf_arr[(_iopfid)]))
#define BFA_IOIM_TO_LPS(__ioim)         		\
	BFA_LPS_FROM_TAG(BFA_LPS_MOD(__ioim->bfa),	\
	 __ioim->itnim->rport->rport_info.lp_tag)

enum ioim_mode {
	BFA_IOIM_WITH_SG = 1,
	BFA_IOIM_NO_SG = 2,
};

static inline bfa_boolean_t
bfa_ioim_maxretry_reached(struct bfa_ioim_s *ioim)
{
	uint16_t k = ioim->iotag >> BFA_IOIM_RETRY_TAG_OFFSET;
	if (k < BFA_IOIM_RETRY_MAX)
		return BFA_FALSE;
	return BFA_TRUE;
}

/*
 * function prototypes
 */
void	bfa_ioim_attach(struct bfa_fcpim_s *fcpim);
void	bfa_ioim_detach(struct bfa_fcpim_s *fcpim);
void	bfa_ioim_isr(struct bfa_s *bfa, struct bfi_msg_s *msg);
void	bfa_ioim_good_comp_isr(struct bfa_s *bfa,
					struct bfi_msg_s *msg);
void	bfa_ioim_cleanup(struct bfa_ioim_s *ioim);
void	bfa_ioim_cleanup_tm(struct bfa_ioim_s *ioim,
					struct bfa_tskim_s *tskim);
void	bfa_ioim_iocdisable(struct bfa_ioim_s *ioim);
void	bfa_ioim_tov(struct bfa_ioim_s *ioim);

void	bfa_tskim_attach(struct bfa_fcpim_s *fcpim);
void	bfa_tskim_detach(struct bfa_fcpim_s *fcpim);
void	bfa_tskim_isr(struct bfa_s *bfa, struct bfi_msg_s *msg);
void	bfa_tskim_iodone(struct bfa_tskim_s *tskim);
void	bfa_tskim_iocdisable(struct bfa_tskim_s *tskim);
void	bfa_tskim_cleanup(struct bfa_tskim_s *tskim);
void	bfa_tskim_res_recfg(struct bfa_s *bfa, uint16_t num_tskim_fw);

void	bfa_itnim_meminfo(struct bfa_iocfc_cfg_s *cfg, uint32_t *km_len);
void	bfa_itnim_attach(struct bfa_fcpim_s *fcpim);
void	bfa_itnim_detach(struct bfa_fcpim_s *fcpim);
void	bfa_itnim_iocdisable(struct bfa_itnim_s *itnim);
void	bfa_itnim_isr(struct bfa_s *bfa, struct bfi_msg_s *msg);
void	bfa_itnim_iodone(struct bfa_itnim_s *itnim);
void	bfa_itnim_tskdone(struct bfa_itnim_s *itnim);
bfa_boolean_t   bfa_itnim_hold_io(struct bfa_itnim_s *itnim);

/*
 * bfa fcpim module API functions
 */
bfa_status_t bfa_fcpim_trs_enable(struct bfa_s *bfa,
		wwn_t lpwwn, wwn_t rpwwn);
bfa_status_t bfa_fcpim_trs_disable(struct bfa_s *bfa,
		wwn_t lpwwn, wwn_t rpwwn);
bfa_status_t bfa_fcpim_trs_clear(struct bfa_s *bfa);
bfa_itnim_t *
bfa_fcpim_get_itnim_by_wwn(struct bfa_fcpim_s *fcpim,
				wwn_t lpwwn, wwn_t rpwwn);
void bfa_fcpim_meminfo(struct bfa_iocfc_cfg_s *cfg,	uint32_t *km_len);
void bfa_fcpim_attach(struct bfa_fcp_mod_s *fcp, void *bfad,
		struct bfa_iocfc_cfg_s *cfg,
		struct bfa_pcidev_s *pcidev);
void bfa_fcpim_detach(struct bfa_fcp_mod_s *fcp);
void bfa_fcpim_start(struct bfa_fcp_mod_s *fcp);
void bfa_fcpim_stop(struct bfa_fcp_mod_s *fcp);
void bfa_fcpim_iocdisable(struct bfa_fcp_mod_s *fcp);

void		bfa_fcpim_path_tov_set(struct bfa_s *bfa, uint16_t path_tov);
uint16_t	bfa_fcpim_path_tov_get(struct bfa_s *bfa);
void		bfa_fcpim_qdepth_set(struct bfa_s *bfa, uint16_t q_depth);
uint16_t	bfa_fcpim_qdepth_get(struct bfa_s *bfa);
bfa_status_t bfa_fcpim_get_modstats(struct bfa_s *bfa,
	 struct bfa_itnim_iostats_s *modstats);
bfa_status_t bfa_fcpim_port_iostats(struct bfa_s *bfa,
		struct bfa_itnim_iostats_s *stats, uint8_t lp_tag);
bfa_status_t bfa_fcpim_get_del_itn_stats(struct bfa_s *bfa,
	 struct bfa_fcpim_del_itn_stats_s *modstats);
bfa_status_t bfa_fcpim_port_clear_iostats(struct bfa_s *bfa, uint8_t lp_tag);
void bfa_fcpim_add_stats(struct bfa_itnim_iostats_s *fcpim_stats,
		struct bfa_itnim_iostats_s *itnim_stats);
bfa_status_t bfa_fcpim_clr_modstats(struct bfa_s *bfa);
void		bfa_fcpim_set_ioredirect(struct bfa_s *bfa,
				bfa_boolean_t state);
void		bfa_fcpim_update_ioredirect(struct bfa_s *bfa);
void		bfa_cb_ioredirect_state_change(void *hcb_bfad,
				bfa_boolean_t ioredirect);
#define bfa_fcpim_ioredirect_enabled(__bfa)				\
	((struct bfa_fcpim_s *)(BFA_FCPIM(__bfa)))->ioredirect

#define bfa_fcpim_get_next_reqq(__bfa, __qid)				\
{									\
	struct bfa_fcpim_s *__fcpim = BFA_FCPIM(__bfa);		\
	__fcpim->reqq++;						\
	__fcpim->reqq &= (BFI_IOC_MAX_CQS -1);				\
	*(__qid) = __fcpim->reqq;					\
}

#define bfa_iocfc_map_msg_to_qid(__msg, __qid)				\
	*(__qid) = (uint8_t)((__msg)& (BFI_IOC_MAX_CQS - 1));
/*
 * bfa itnim API functions
 */
struct bfa_itnim_s *bfa_itnim_create(struct bfa_s *bfa,
					struct bfa_rport_s *rport, void *itnim);
void		bfa_itnim_delete(struct bfa_itnim_s *itnim);
void		bfa_itnim_online(struct bfa_itnim_s *itnim,
				 bfa_boolean_t seq_rec);
void		bfa_itnim_offline(struct bfa_itnim_s *itnim);
void		bfa_itnim_get_stats(struct bfa_itnim_s *itnim,
			struct bfa_itnim_iostats_s *stats);
void		bfa_itnim_clear_stats(struct bfa_itnim_s *itnim);
#define bfa_itnim_get_reqq(__ioim) ((struct bfa_ioim_s *)__ioim)->itnim->reqq

/**
 * @brief
 * 		BFA completion callback for bfa_itnim_online().
 *
 * @param[in]		itnim		FCS or driver itnim instance
 *
 * return None
 */
void	bfa_cb_itnim_online(void *itnim);

/**
 * @brief
 * 		BFA completion callback for bfa_itnim_offline().
 *
 * @param[in]		itnim		FCS or driver itnim instance
 *
 * return None
 */
void	bfa_cb_itnim_offline(void *itnim);
void	bfa_cb_itnim_tov_begin(void *itnim);
void	bfa_cb_itnim_tov(void *itnim);

/**
 * @brief
 * 		BFA notification to FCS/driver for second level error recovery.
 *
 * Atleast one I/O request has timedout and target is unresponsive to
 * repeated abort requests. Second level error recovery should be initiated
 * by starting implicit logout and recovery procedures.
 *
 * @param[in]		itnim		FCS or driver itnim instance
 *
 * return None
 */
void	bfa_cb_itnim_sler(void *itnim);

/*
 * bfa ioim API functions
 */
struct bfa_ioim_s	*bfa_ioim_alloc(struct bfa_s *bfa,
					struct bfad_ioim_s *dio,
					struct bfa_itnim_s *itnim,
					uint16_t nsgles);

void		bfa_ioim_free(struct bfa_ioim_s *ioim);
void	bfa_ioim_start(struct bfa_ioim_s *ioim, uint8_t mode);
bfa_status_t	bfa_ioim_abort(struct bfa_ioim_s *ioim);
void 		bfa_ioim_delayed_comp(struct bfa_ioim_s *ioim,
				      bfa_boolean_t iotov);


/**
 * @brief
 * 		I/O completion notification.
 *
 * @param[in]		dio			driver IO structure
 * @param[in]		io_status		IO completion status
 * @param[in]		scsi_status		SCSI status returned by target
 * @param[in]		sns_len			SCSI sense length, 0 if none
 * @param[in]		sns_info		SCSI sense data, if any
 * @param[in]		residue			Residual length
 *
 * @return None
 */
void	bfa_cb_ioim_done(void *bfad, struct bfad_ioim_s *dio,
				  enum bfi_ioim_status io_status,
				  uint8_t scsi_status, int sns_len,
				  uint8_t *sns_info, int32_t residue);

/**
 * @brief
 * 		I/O good completion notification.
 *
 * @param[in]		dio			driver IO structure
 *
 * @return None
 */
void	bfa_cb_ioim_good_comp(void *bfad, struct bfad_ioim_s *dio);

/**
 * @brief
 * 		I/O abort completion notification
 *
 * @param[in]		dio			driver IO that was aborted
 *
 * @return None
 */
void	bfa_cb_ioim_abort(void *bfad, struct bfad_ioim_s *dio);
void		bfa_cb_ioim_resfree(void *hcb_bfad);

void 			bfa_cb_ioim_resfree(void *hcb_bfad);

/*
 * bfa tskim API functions
 */
struct bfa_tskim_s	*bfa_tskim_alloc(struct bfa_s *bfa,
					struct bfad_tskim_s *dtsk);
void		bfa_tskim_free(struct bfa_tskim_s *tskim);
bfa_status_t	bfa_tskim_start(struct bfa_tskim_s *tskim,
				struct bfa_itnim_s *itnim, lun_t lun,
				fcp_tm_cmnd_t tm, uint8_t t_secs);
void		bfa_cb_tskim_done(void *bfad, struct bfad_tskim_s *dtsk,
				  enum bfi_tskim_status tsk_status);

void bfa_fcpim_lunmask_rp_update(bfa_t *bfa, wwn_t lp_wwn, wwn_t rp_wwn,
		 uint16_t rp_tag, uint8_t lp_tag);
bfa_status_t bfa_fcpim_lunmask_update(struct bfa_s *bfa, uint32_t on_off);
bfa_status_t bfa_fcpim_lunmask_query(struct bfa_s *bfa, void *buf);

bfa_status_t bfa_fcpim_lunmask_delete(struct bfa_s *bfa, uint16_t vf_id,
	wwn_t *pwwn, wwn_t rpwwn, lun_t lun);

bfa_status_t bfa_fcpim_lunmask_add(struct bfa_s *bfa, uint16_t vf_id,
	wwn_t *pwwn, wwn_t rpwwn, lun_t lun);
bfa_status_t bfa_fcpim_lunmask_clear(struct bfa_s *bfa);

bfa_status_t bfa_fcpim_iopf_start(struct bfa_s *bfa, uint16_t vf_id,
	wwn_t lpwwn, wwn_t rpwwn, int flow_count,
	lun_t lunstart[BFA_IOPF_MAX], uint8_t lun_status[BFA_IOPF_MAX],
	uint32_t time);
bfa_status_t bfa_fcpim_iopf_stop(struct bfa_s *bfa, uint16_t vf_id,
	wwn_t lpwwn, wwn_t rpwwn, int flow_count, lun_t lunstart[BFA_IOPF_MAX]);
bfa_status_t bfa_fcpim_iopf_query(struct bfa_s *bfa, uint32_t *flow_count,
	 bfa_fcpim_iopf_cfg_t *iopf);
bfa_status_t bfa_fcpim_iopf_show(struct bfa_s *bfa, uint16_t vf_id,
	 wwn_t pwwn, wwn_t rpwwn, lun_t lun,
	 enum bfa_fcpim_iopf_data_type itn_lun, bfa_fcpim_iopf_t *iopf);
bfa_status_t bfa_fcpim_iopf_reset(struct bfa_s *bfa, uint16_t vf_id,
		wwn_t lpwwn, wwn_t rpwwn, uint32_t time,  int flow_count,
		lun_t lun[BFA_IOPF_MAX]);

uint16_t bfa_fcpim_get_throttle_cfg(struct bfa_s *bfa, uint16_t drv_cfg_param);
uint16_t bfa_fcpim_read_throttle(struct bfa_s *bfa);
bfa_status_t bfa_fcpim_write_throttle(struct bfa_s *bfa, uint16_t value);
bfa_status_t bfa_fcpim_throttle_set(struct bfa_s *bfa, uint16_t value);
bfa_status_t bfa_fcpim_throttle_get(struct bfa_s *bfa, void *buf);

#endif /* __BFA_FCPIM_H__ */
