/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_modules.h BFA modules
 */

#ifndef __BFA_MODULES_H__
#define __BFA_MODULES_H__

#include <cs/bfa_cs.h>
#include <bfa/bfa.h>
#include <bfa/bfa_svc.h>
#include <bfa/bfa_ipfc.h>
#include <bfa/bfa_fcp.h>
#include <bfa/bfa_pbind.h>
#include <cna/bfa_port.h>
#include <cna/bfa_sfp.h>
#include <cna/bfa_flash.h>
#include <cna/bfa_phy.h>
#include <cna/bfa_fru.h>
#include <bfa/bfa_tmops.h>
#include <bfa/bfa_dconf.h>

#ifdef BFA_BIOS_BUILD
#include <bfa/bfa_modules_bios.h>
#else
struct bfa_modules_s {
	struct bfa_fcport_s	fcport;		/*!< fc port module	      */
	struct bfa_fcxp_mod_s	fcxp_mod;	/*!< fcxp module	      */
	struct bfa_lps_mod_s	lps_mod;	/*!< fcxp module	      */
	struct bfa_uf_mod_s	uf_mod;		/*!< unsolicited frame module */
	struct bfa_rport_mod_s	rport_mod;	/*!< remote port module	      */
	struct bfa_fcp_mod_s	fcp_mod;	/*!< FCP IM-TM common module  */
	struct bfa_dconf_mod_s dconf_mod;	/*!< DCONF common module   */
	struct bfa_sgpg_mod_s	sgpg_mod;	/*!< SG page module	      */
	bfa_diag_mod_t		diag_mod;	/*!< diagnostics module	      */
	bfa_fcdiag_t		fcdiag;		/*!< FC diagnostics	      */
	bfa_sfp_t		sfp;		/*!< SFP module		      */
	bfa_flash_t		flash;		/*!< flash module	      */
	bfa_phy_t		phy;		/*!< phy module		      */
	bfa_ipfc_mod_t		ipfc_mod;	/*!< BFA IPFC receive	      */
	struct bfa_cee_s	cee;		/*!< CEE Module		      */
	struct bfa_port_s	port;		/*!< Physical port module     */
	bfa_pbind_mod_t		pbind_mod;	/*!< PBind module	      */
	struct bfa_ablk_s	ablk;		/*!< ASIC block config module */
	bfa_fru_t		fru;		/*!< fru module		      */
};
#endif

/*
 * !!! Only append to the enums defined here to avoid any versioning
 * !!! needed between trace utility and driver version
 */
enum {
	BFA_TRC_HAL_CORE	= 1,
	BFA_TRC_HAL_FCXP	= 2,
	BFA_TRC_HAL_FCPIM	= 3,
	BFA_TRC_HAL_FLASH	= 4,
	BFA_TRC_HAL_DEBUG	= 5,
	BFA_TRC_HAL_WWN		= 6,
	BFA_TRC_HAL_FLASH_RAW	= 7,
	BFA_TRC_HAL_SBOOT	= 8,
	BFA_TRC_HAL_SBOOT_IO	= 9,
	BFA_TRC_HAL_SBOOT_INTR	= 10,
	BFA_TRC_HAL_SBTEST	= 11,
	BFA_TRC_HAL_IPFC	= 12,
	BFA_TRC_HAL_FCPTM	= 13,
	BFA_TRC_HAL_TIO		= 14,
	BFA_TRC_HAL_TTSK	= 15,
	BFA_TRC_HAL_TIN		= 16,
	BFA_TRC_HAL_FCDIAG	= 17,
	BFA_TRC_HAL_PBIND	= 18,
	BFA_TRC_HAL_IOCFC_CT	= 19,
	BFA_TRC_HAL_IOCFC_CB	= 20,
	BFA_TRC_HAL_IOCFC_Q	= 21,
	BFA_TRC_HAL_TIOSP	= 22,
	BFA_TRC_HAL_FCP		= 23,
	BFA_TRC_HAL_DCONF	= 24,
};


/**
 * @brief
 * Macro to define a new BFA module
 */
#define BFA_MODULE(__mod) 						\
	static void bfa_ ## __mod ## _meminfo(				\
			struct bfa_iocfc_cfg_s *cfg,			\
			struct bfa_meminfo_s *meminfo,			\
			struct bfa_s *bfa);				\
	static void bfa_ ## __mod ## _attach(struct bfa_s *bfa,		\
			void *bfad, struct bfa_iocfc_cfg_s *cfg, 	\
			struct bfa_pcidev_s *pcidev);			\
	static void bfa_ ## __mod ## _detach(struct bfa_s *bfa);	\
	static void bfa_ ## __mod ## _start(struct bfa_s *bfa);		\
	static void bfa_ ## __mod ## _stop(struct bfa_s *bfa);		\
	static void bfa_ ## __mod ## _iocdisable(struct bfa_s *bfa);	\
									\
	extern struct bfa_module_s hal_mod_ ## __mod;			\
	struct bfa_module_s hal_mod_ ## __mod = {			\
		bfa_ ## __mod ## _meminfo,				\
		bfa_ ## __mod ## _attach,				\
		bfa_ ## __mod ## _detach,				\
		bfa_ ## __mod ## _start,				\
		bfa_ ## __mod ## _stop,					\
		bfa_ ## __mod ## _iocdisable,				\
	}

#define BFA_CACHELINE_SZ	(256)

/**
 * Structure used to interact between different BFA sub modules
 *
 * Each sub module needs to implement only the entry points relevant to it (and
 * can leave entry points as NULL)
 */
struct bfa_module_s {
	void (*meminfo) (struct bfa_iocfc_cfg_s *cfg,
			 struct bfa_meminfo_s *meminfo,
			 struct bfa_s *bfa);
	void (*attach) (struct bfa_s *bfa, void *bfad,
			struct bfa_iocfc_cfg_s *cfg,
			struct bfa_pcidev_s *pcidev);
	void (*detach) (struct bfa_s *bfa);
	void (*start) (struct bfa_s *bfa);
	void (*stop) (struct bfa_s *bfa);
	void (*iocdisable) (struct bfa_s *bfa);
};

extern struct bfa_module_s *hal_mods[];

struct bfa_s {
	void			*bfad;		/*!< BFA driver instance      */
	struct bfa_aen_s	*aen;		/*!< AEN module		      */
	struct bfa_plog_s	*plog;		/*!< portlog buffer	      */
	struct bfa_log_mod_s	*logm;		/*!< driver logging modulen   */
	struct bfa_trc_mod_s	*trcmod;	/*!< driver tracing	      */
	struct bfa_ioc_s	ioc;		/*!< IOC module		      */
	struct bfa_iocfc_s	iocfc;		/*!< IOCFC module	      */
	struct bfa_timer_mod_s	timer_mod;	/*!< timer module	      */
	struct bfa_modules_s	modules;	/*!< BFA modules	      */
	struct bfa_q_s		comp_q;		/*!< pending completions      */
	bfa_boolean_t		queue_process;	/*!< queue processing enabled */
	struct bfa_q_s		reqq_waitq[BFI_IOC_MAX_CQS];
	bfa_boolean_t		fcs;		/*!< FCS is attached to BFA   */
	struct bfa_msix_s	msix;
	struct bfa_tm_ops_s	*tm_ops;
	bfa_boolean_t		intr_enabled;	/*!< Status of interrupts     */
};

extern bfa_isr_func_t bfa_isrs[BFI_MC_MAX];
extern bfa_ioc_mbox_mcfunc_t  bfa_mbox_isrs[];
extern bfa_boolean_t bfa_auto_recover;
extern struct bfa_module_s hal_mod_flash;
extern struct bfa_module_s hal_mod_fcdiag;
extern struct bfa_module_s hal_mod_sgpg;
extern struct bfa_module_s hal_mod_fcport;
extern struct bfa_module_s hal_mod_fcxp;
extern struct bfa_module_s hal_mod_lps;
extern struct bfa_module_s hal_mod_uf;
extern struct bfa_module_s hal_mod_rport;
extern struct bfa_module_s hal_mod_fcp;
extern struct bfa_module_s hal_mod_pbind;
extern struct bfa_module_s hal_mod_dconf;


#endif /* __BFA_MODULES_H__ */
