/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file hal_pbind.h BFA pBind module
 */

#ifndef __BFA_PBIND_H__
#define __BFA_PBIND_H__


#include <defs/bfa_defs.h>
#include <defs/bfa_defs_pbind.h>
#include <bfi/bfi_pbind.h>
#include <bfa_os_inc.h>


#define BFA_PBIND_SIGNATURE 		0xbeedbaba
#define BFA_PBIND_SIGNATURE_LEN 	8


typedef void	(*bfa_pbind_cb_t) (void *pbind, bfa_status_t status);
/**
 * Target persistent binding entry in memory.
 */
typedef struct {
	bfa_q_t		hqe;		/*!< queue entry element */
	bfa_pbind_map_t map;		/*!< mapping information */
} bfa_pbind_entry_t;

/**
 * Target persistent binding module data structure
 */
typedef struct bfa_pbind_mod_s {
	bfa_sm_t		sm;
	bfa_t	*bfa;	/*!< BFA HAL instance */
	void	*bfad;	/*!< BFA driver instance */
	bfa_trc_mod_t	*trcmod;
	bfa_q_t	pbind_free_q;
	bfa_q_t	pbind_alloc_q;
	bfa_q_t	pbind_aging_q;
	bfa_q_t			pbind_rsvd_q;
	uint32_t		num_pbind_entries;
	bfa_boolean_t	pbstore_valid;
	bfa_boolean_t	pbind_enabled;
	bfa_pbind_entry_t	*pb_list;
	bfi_pbind_t		*pb_flash_list;
	uint8_t			instance;
	bfa_pbind_cb_t	drv_cb;
	struct bfa_mem_kva_s	kva_seg;
} bfa_pbind_mod_t;

#define BFA_INVALID_OS_MAP	(uint16_t)-1
#define BFA_PBIND_MOD(__bfa)	(&(__bfa)->modules.pbind_mod)
#define BFA_MEM_PBIND_KVA(__bfa) (&(BFA_PBIND_MOD(__bfa)->kva_seg))

#define bfa_pbind_entry_init(pbent)	do { 				\
	(pbent)->map.lpwwn = (wwn_t)0;					\
	(pbent)->map.rpwwn = (wwn_t)0;					\
	(pbent)->map.os_tid = BFA_INVALID_OS_MAP;		\
	bfa_q_qe_init(pbent);	\
		} while (0)

#define bfa_pbind_entry_copyto_mem(mem, flash, i) do {	\
		(mem)->pb_list[i].map.rpwwn = (flash)->map[i].rpwwn; \
		(mem)->pb_list[i].map.lpwwn = (flash)->map[i].lpwwn; \
		(mem)->pb_list[i].map.os_tid = \
		BFA_PBIND_MAKE_OSTID((flash)->map[i].bus, \
			(flash)->map[i].target);	\
		} while (0)

#define bfa_pbind_entry_copyto_flash(flash, mem, i) do 	{	\
		(flash)->map[i].rpwwn = (mem)->pb_list[i].map.rpwwn; \
		(flash)->map[i].lpwwn = (mem)->pb_list[i].map.lpwwn; \
		(flash)->map[i].bus = BFA_PBIND_GET_OSBUS(( \
			mem)->pb_list[i].map.os_tid);			\
		(flash)->map[i].target = BFA_PBIND_GET_OSTARGET( \
		(mem)->pb_list[i].map.os_tid);	\
	} while (0)

bfa_boolean_t bfa_pbind_enabled(bfa_t *bfa);
void bfa_pbind_enable(bfa_t *bfa);
void bfa_pbind_disable(bfa_t *bfa);
uint16_t bfa_pbind_map(bfa_t *bfa, wwn_t lpwwn, wwn_t rpwwn,
		bfa_boolean_t synth_fc);
void bfa_pbind_unmap(bfa_t *bfa, wwn_t lpwwn, wwn_t rpwwn);
void bfa_pbind_list_map(bfa_t *bfa, wwn_t lpwwn,
					bfa_pbind_map_t *map, uint32_t *nmaps);
void bfa_pbind_get_map_idx(bfa_t *bfa, wwn_t lpwwn,
	bfa_pbind_map_t *map, uint32_t idx);
void bfa_pbind_get_map_count(bfa_t *bfa, wwn_t lpwwn, uint32_t *count);
void bfa_pbind_table_init(bfa_t *bfa);
wwn_t
bfa_pbind_get_lpwwn(bfa_t *bfa, uint16_t os_tid, wwn_t pp_pwwn);
wwn_t
bfa_pbind_get_get_rpwwn(bfa_t *bfa, uint16_t os_tid, wwn_t pp_pwwn);
bfa_status_t
bfa_pbind_set_map(bfa_t *bfa, wwn_t lpwwn, wwn_t rpwwn,
				  uint8_t bus, uint8_t target);
bfa_status_t
bfa_pbind_clear_map(bfa_t *bfa, wwn_t lpwwn, wwn_t rpwwn);
uint8_t bfa_cb_pbind_get_bus_offset(void *bfad);
uint16_t bfa_cb_pbind_map_ostid(void *bfad, uint16_t ostid);
#endif /* __BFA_PBIND_H__ */
