/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_SBOOT_H__
#define __BFA_SBOOT_H__

#include <defs/bfa_defs.h>
#include <defs/bfa_defs_boot.h>
#include <bfa_os_inc.h>
#include <bfa/bfa.h>
#include <bfi/bfi_ms.h>

typedef struct {
	bfa_sboot_config_t	*sbcfg;
	bfi_sboot_rport_t	*rplist;
	bfi_sboot_lun_t	*lunlist;
	struct bfa_dma_s	sbcfg_dma;
	struct bfa_dma_s	rplist_dma;
	struct bfa_dma_s	lunlist_dma;
	uint16_t		num_sboot_tgts;
	uint16_t		num_sboot_luns;
} bfa_sboot_t;

/*
 * protected functions
 */
void	bfa_sboot_isr(struct bfa_s *bfa, struct bfi_msg_s *msg);


bfa_status_t    bfa_sboot_init(struct bfa_s *bfa);
void			bfa_sboot_set_speed(struct bfa_s *bfa,
					enum bfa_port_speed speed);
bfa_status_t    bfa_sboot_openblun(struct bfa_s *bfa, bfa_boolean_t autodisc,
				       wwn_t pwwn, lun_t lun);
bfa_status_t    bfa_sboot_discover_rports(bfa_t *bfa, uint64_t addr,
					   uint32_t nrports);
bfa_status_t	bfa_sboot_discover_luns(bfa_t *bfa, wwn_t pwwn,
				  uint32_t fc_pid, uint64_t addr,
				  uint32_t nluns);

void bfa_sboot_init_comp(struct bfa_s *bfa, bfi_sboot_init_rsp_t *msg);
void bfa_sboot_discover_rports_comp(struct bfa_s *bfa,
					bfi_sboot_disc_rports_rsp_t *msg);
void bfa_sboot_discover_luns_comp(struct bfa_s *bfa,
				 bfi_sboot_disc_luns_rsp_t *msg);
void bfa_sboot_openblun_comp(struct bfa_s *bfa, bfi_sboot_openblun_rsp_t *msg);
void bfa_sboot_iodone_comp(struct bfa_s *bfa, bfi_sboot_iorw_rsp_t *msg);

bfa_status_t    bfa_sboot_io_read(struct bfa_s *bfa, bfa_os_addr_t bufaddr,
				      uint32_t blkoff, uint32_t nblks);
bfa_status_t    bfa_sboot_io_write(struct bfa_s *bfa, bfa_os_addr_t bufaddr,
				       uint32_t blkoff, uint32_t nblks);

bfi_sboot_rport_t *bfa_sboot_get_rplist(struct bfa_s *bfa);
bfi_sboot_lun_t *bfa_sboot_get_lunlist(struct bfa_s *bfa);

/**
 * @brief
 * 	BFA san boot initializatoin callback.
 *
 * @param[in]		bfad		driver instance
 * @param[in]		status		Initialization status.
 *
 * @return None
 */
void		bfa_cb_sboot_init(void *bfad, bfa_status_t status);

/**
 * @brief
 * 	BFA san boot open boot lun callback
 *
 * @param[in]		bfad		driver instance
 * @param[in]		rpid		remote target fc id
 * @param[in]		wwn			remote target wwn
 * @param[in]		lun			boot lun
 * @param[in]		status		config save status.
 *
 * @return None
 */
void		bfa_cb_sboot_openblun(void *bfad, uint32_t rpid,  wwn_t wwn,
					lun_t lun, bfa_status_t status);

/**
 * @brief
 * 	BFA san boot rport discovery callback. The rport array that
 * 	is provided at initialization time is populated with the discovered
 * 	targets.
 *
 * @param[in]		bfad		Driver instance.
 * @param[in]		nrports		Number of rports discovered.
 * @param[in]		status		Remote target port discovery status.
 *
 * @return None
 */
void		bfa_cb_sboot_discover_rports(void *bfad, uint32_t nrports,
					      bfa_status_t status);

/**
 * @brief
 * 	BFA san boot lun discovery callback. The lun array that is part of
 * 	san boot configuration area provided at initialization time is
 * 	populated with the discovered LUN data.
 *
 * @param[in]		bfad		Driver instance.
 * @param[in]		nluns		Number of LUNs discovered.
 * @param[in]		status		LUN discovery status.
 *
 * @return None
 */
void		bfa_cb_sboot_discover_luns(void *bfad, uint32_t nluns,
					    bfa_status_t status);

/**
 * @brief
 * 	BFA san boot IO completion callback.
 *
 * @param[in]		bfad		driver instance
 * @param[in]		status		IO completion status.
 *
 * @return None
 */
void		bfa_cb_sboot_iodone(void *bfad, bfa_status_t status);

/**
 * @brief
 * 	BFA san boot offline event callback.
 *
 * @param[in]		bfad		driver instance
 * @param[in]		rsn			reason for offline event
 *
 * @return None
 */
void		bfa_cb_sboot_offline(void *bfad, uint8_t rsn);

#endif /* __BFA_SBOOT_H__ */
