/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFI_MS_H__
#define __BFI_MS_H__

#include <bfi/bfi.h>
#include <defs/bfa_defs_svc.h>
#include <defs/bfa_defs_boot.h>

#pragma pack(1)

enum bfi_iocfc_h2i_msgs {
	BFI_IOCFC_H2I_CFG_REQ 		= 1,
	BFI_IOCFC_H2I_SET_INTR_REQ 	= 2,
	BFI_IOCFC_H2I_UPDATEQ_REQ 	= 3,
	BFI_IOCFC_H2I_FAA_QUERY_REQ	= 4,
	BFI_IOCFC_H2I_ADDR_REQ		= 5,
};
typedef enum bfi_iocfc_h2i_msgs bfi_iocfc_h2i_msgs_t;

enum bfi_iocfc_i2h_msgs {
	BFI_IOCFC_I2H_CFG_REPLY		= BFA_I2HM(1),
	BFI_IOCFC_I2H_UPDATEQ_RSP	= BFA_I2HM(3),
	BFI_IOCFC_I2H_FAA_QUERY_RSP	= BFA_I2HM(4),
	BFI_IOCFC_I2H_ADDR_MSG		= BFA_I2HM(5),
};
typedef enum bfi_iocfc_i2h_msgs bfi_iocfc_i2h_msgs_t;

struct bfi_iocfc_cfg_s {
	uint8_t	 num_cqs; 	/*!< Number of CQs to be used     */
	uint8_t	 sense_buf_len;	/*!< SCSI sense length	    */
	uint16_t	rsvd_1;
	uint32_t	endian_sig;	/*!< endian signature of host     */
	uint8_t	fw_xfer_rdy_tov;
	uint8_t	single_msix_vec;
	uint8_t rsvd[2];
	uint16_t num_ioim_reqs;
	uint16_t num_fwtio_reqs;

	/**
	 * Request and response circular queue base addresses, size and
	 * shadow index pointers.
	 */
	union bfi_addr_u  req_cq_ba[BFI_IOC_MAX_CQS];
	union bfi_addr_u  req_shadow_ci[BFI_IOC_MAX_CQS];
	uint16_t    req_cq_elems[BFI_IOC_MAX_CQS];
	union bfi_addr_u  rsp_cq_ba[BFI_IOC_MAX_CQS];
	union bfi_addr_u  rsp_shadow_pi[BFI_IOC_MAX_CQS];
	uint16_t    rsp_cq_elems[BFI_IOC_MAX_CQS];

	union bfi_addr_u  stats_addr;	/*!< DMA-able address for stats	  */
	union bfi_addr_u  cfgrsp_addr;	/*!< config response dma address  */
	union bfi_addr_u  ioim_snsbase[BFI_IOIM_SNSBUF_SEGS];
					/*!< IO sense buf base addr segments */
	struct bfa_iocfc_intr_attr_s intr_attr; /*!< IOC interrupt attributes */
};
typedef struct bfi_iocfc_cfg_s bfi_iocfc_cfg_t;

/**
 * Boot target wwn information for this port. This contains either the stored
 * or discovered boot target port wwns for the port.
 */
struct bfi_iocfc_bootwwns {
	wwn_t		wwn[ BFA_BOOT_BOOTLUN_MAX ];
	uint8_t		nwwns;
	uint8_t		rsvd[7];
};
typedef struct bfi_iocfc_bootwwns bfi_iocfc_bootwwns_t;

/**
 * Queue configuration response from firmware
 */
struct bfi_iocfc_qreg_s {
	uint32_t	cpe_q_ci_off[ BFI_IOC_MAX_CQS ];
	uint32_t	cpe_q_pi_off[ BFI_IOC_MAX_CQS ];
	uint32_t	cpe_qctl_off[ BFI_IOC_MAX_CQS ];
	uint32_t	rme_q_ci_off[ BFI_IOC_MAX_CQS ];
	uint32_t	rme_q_pi_off[ BFI_IOC_MAX_CQS ];
	uint32_t	rme_qctl_off[ BFI_IOC_MAX_CQS ];
	uint8_t		hw_qid[ BFI_IOC_MAX_CQS ];
};
typedef struct bfi_iocfc_qreg_s bfi_iocfc_qreg_t;

struct bfi_iocfc_cfgrsp_s {
	struct bfa_iocfc_fwcfg_s	fwcfg;
	struct bfa_iocfc_intr_attr_s	intr_attr;
	struct bfi_iocfc_bootwwns	bootwwns;
	struct bfi_pbc_s		pbc_cfg;
	struct bfi_iocfc_qreg_s		qreg;
};
typedef struct bfi_iocfc_cfgrsp_s bfi_iocfc_cfgrsp_t;

/**
 * BFI_IOCFC_H2I_CFG_REQ message
 */
struct bfi_iocfc_cfg_req_s {
	struct bfi_mhdr_s	mh;
	union bfi_addr_u	ioc_cfg_dma_addr;
};
typedef struct bfi_iocfc_cfg_req_s bfi_iocfc_cfg_req_t;

/**
 * BFI_IOCFC_I2H_CFG_REPLY message
 */
struct bfi_iocfc_cfg_reply_s {
	struct bfi_mhdr_s  mh;		/*!< Common msg header	  */
	uint8_t	 cfg_success;	/*!< cfg reply status	   */
	uint8_t	 lpu_bm;		/*!< LPUs assigned for this IOC */
	uint8_t	 rsvd[2];
};
typedef struct bfi_iocfc_cfg_reply_s bfi_iocfc_cfg_reply_t;

/**
 * BFI_IOCFC_H2I_SET_INTR_REQ message
 */
struct bfi_iocfc_set_intr_req_s {
	struct bfi_mhdr_s mh;		/*!< common msg header     	*/
	uint8_t		coalesce;	/*!< enable intr coalescing	*/
	uint8_t		rsvd[3];
	uint16_t	delay;		/*!< delay timer 0..1125us  	*/
	uint16_t	latency;	/*!< latency timer 0..225us 	*/
};
typedef struct bfi_iocfc_set_intr_req_s bfi_iocfc_set_intr_req_t;

/**
 * BFI_IOCFC_H2I_UPDATEQ_REQ message
 */
struct bfi_iocfc_updateq_req_s {
	struct bfi_mhdr_s mh;		/*!< common msg header		*/
	uint32_t reqq_ba;		/*!< reqq base addr		*/
	uint32_t rspq_ba;		/*!< rspq base addr		*/
	uint32_t reqq_sci;		/*!< reqq shadow ci		*/
	uint32_t rspq_spi;		/*!< rspq shadow pi		*/
};
typedef struct bfi_iocfc_updateq_req_s bfi_iocfc_updateq_req_t;

/**
 * BFI_IOCFC_I2H_UPDATEQ_RSP message
 */
struct bfi_iocfc_updateq_rsp_s {
	struct bfi_mhdr_s mh;		/*!< common msg header     	*/
	uint8_t	 status;		/*!< updateq  status		*/
	uint8_t	 rsvd[3];
};
typedef struct bfi_iocfc_updateq_rsp_s bfi_iocfc_updateq_rsp_t;

/**
 * H2I Messages
 */
union bfi_iocfc_h2i_msg_u {
	struct bfi_mhdr_s 		mh;
	struct bfi_iocfc_cfg_req_s 	cfg_req;
	struct bfi_iocfc_updateq_req_s updateq_req;
	uint32_t mboxmsg[BFI_IOC_MSGSZ];
};
typedef union bfi_iocfc_h2i_msg_u bfi_iocfc_h2i_msg_t;

/**
 * I2H Messages
 */
union bfi_iocfc_i2h_msg_u {
	struct bfi_mhdr_s	mh;
	struct bfi_iocfc_cfg_reply_s 		cfg_reply;
	struct bfi_iocfc_updateq_rsp_s updateq_rsp;
	uint32_t mboxmsg[BFI_IOC_MSGSZ];
};
typedef union bfi_iocfc_i2h_msg_u bfi_iocfc_i2h_msg_t;

/**
 * BFI_IOCFC_H2I_FAA_ENABLE_REQ BFI_IOCFC_H2I_FAA_DISABLE_REQ message
 */
struct bfi_faa_en_dis_s {
	struct bfi_mhdr_s mh;		/*!< common msg header		*/
};
typedef struct bfi_faa_en_dis_s bfi_faa_en_dis_t;

struct bfi_faa_addr_msg_s {
	struct	bfi_mhdr_s mh;		/*!< common msg header		*/
	uint8_t rsvd[4];
	wwn_t	pwwn;			/*!< Fabric acquired PWWN	*/
	wwn_t	nwwn;			/*!< Fabric acquired PWWN	*/
};
typedef struct bfi_faa_addr_msg_s bfi_faa_addr_msg_t;
/**
 * BFI_IOCFC_H2I_FAA_QUERY_REQ message
 */
struct bfi_faa_query_s {
	struct	bfi_mhdr_s mh;		/*!< common msg header		*/
	uint8_t faa_status;		/*!< FAA status			*/
	uint8_t addr_source;		/*!< PWWN source		*/
	uint8_t rsvd[2];
	wwn_t	faa;			/*!< Fabric acquired PWWN	*/
};
typedef struct bfi_faa_query_s bfi_faa_query_t;

/*
 * BFI_IOCFC_I2H_FAA_ENABLE_RSP, BFI_IOCFC_I2H_FAA_DISABLE_RSP message
 */
struct bfi_faa_en_dis_rsp_s {
	struct bfi_mhdr_s mh;		/*!< common msg header		*/
	uint8_t status;			/*!< updateq  status		*/
	uint8_t rsvd[3];
};
typedef struct bfi_faa_en_dis_rsp_s bfi_faa_en_dis_rsp_t;

/*
 * BFI_IOCFC_I2H_FAA_QUERY_RSP message
 */
typedef struct bfi_faa_query_s bfi_faa_query_rsp_t;

/**
 * FCPORT
 */

enum bfi_fcport_h2i {
	BFI_FCPORT_H2I_ENABLE_REQ		= (1),
	BFI_FCPORT_H2I_DISABLE_REQ		= (2),
	BFI_FCPORT_H2I_SET_SVC_PARAMS_REQ	= (3),
	BFI_FCPORT_H2I_STATS_GET_REQ		= (4),
	BFI_FCPORT_H2I_STATS_CLEAR_REQ		= (5),
};
typedef enum bfi_fcport_h2i bfi_fcport_h2i_t;

enum bfi_fcport_i2h {
	BFI_FCPORT_I2H_ENABLE_RSP		= BFA_I2HM(1),
	BFI_FCPORT_I2H_DISABLE_RSP		= BFA_I2HM(2),
	BFI_FCPORT_I2H_SET_SVC_PARAMS_RSP	= BFA_I2HM(3),
	BFI_FCPORT_I2H_STATS_GET_RSP		= BFA_I2HM(4),
	BFI_FCPORT_I2H_STATS_CLEAR_RSP		= BFA_I2HM(5),
	BFI_FCPORT_I2H_EVENT			= BFA_I2HM(6),
	BFI_FCPORT_I2H_TRUNK_SCN		= BFA_I2HM(7),
	BFI_FCPORT_I2H_ENABLE_AEN		= BFA_I2HM(8),
	BFI_FCPORT_I2H_DISABLE_AEN		= BFA_I2HM(9),
};
typedef enum bfi_fcport_i2h bfi_fcport_i2h_t;

/**
 * Generic REQ type
 */
struct bfi_fcport_req_s {
	struct bfi_mhdr_s  mh;		/*!< msg header			    */
	uint32_t	   msgtag;	/*!< msgtag for reply		    */
};
typedef struct bfi_fcport_req_s bfi_fcport_req_t;

/**
 * Generic RSP type
 */
struct bfi_fcport_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header	  */
	uint8_t		status;		/*!< port enable status	  */
	uint8_t		rsvd[3];
	struct	bfa_port_cfg_s port_cfg; /*!< port configuration  */
	uint32_t	msgtag;		/*!< msgtag for reply	  */
};
typedef struct bfi_fcport_rsp_s bfi_fcport_rsp_t;

/**
 * BFI_FCPORT_H2I_ENABLE_REQ
 */
struct bfi_fcport_enable_req_s {
	struct bfi_mhdr_s  mh;		/*!< msg header			    */
	uint32_t	   rsvd1;
	wwn_t		   nwwn;	/*!< node wwn of physical port	    */
	wwn_t		   pwwn;	/*!< port wwn of physical port	    */
	struct bfa_port_cfg_s port_cfg; /*!< port configuration	    */
	union bfi_addr_u   stats_dma_addr; /*!< DMA address for stats	    */
	uint32_t	   msgtag;	/*!< msgtag for reply		    */
	uint8_t		   use_flash_cfg;	/*!< get prot cfg from flash */
	uint8_t		   rsvd2[3];
};
typedef struct bfi_fcport_enable_req_s bfi_fcport_enable_req_t;

/**
 * BFI_FCPORT_H2I_SET_SVC_PARAMS_REQ
 */
struct bfi_fcport_set_svc_params_req_s {
	struct bfi_mhdr_s  mh;		/*!< msg header */
	uint16_t	   tx_bbcredit;	/*!< Tx credits */
	uint8_t		rsvd[2];
};
typedef struct bfi_fcport_set_svc_params_req_s bfi_fcport_set_svc_params_req_t;

/**
 * BFI_FCPORT_I2H_EVENT
 */
struct bfi_fcport_event_s {
	struct bfi_mhdr_s 	mh;	/*!< common msg header */
	struct bfa_port_link_s	link_state;
};
typedef struct bfi_fcport_event_s bfi_fcport_event_t;

/**
 * BFI_FCPORT_I2H_TRUNK_SCN
 */
struct bfi_fcport_trunk_link_s {
	wwn_t			trunk_wwn;
	uint8_t			fctl;		/* bfa_trunk_link_fctl_t */
	uint8_t			state;		/* bfa_trunk_link_state_t */
	uint8_t			speed;		/* bfa_port_speed_t */
	uint8_t			rsvd;
	uint32_t		deskew;
};
typedef struct bfi_fcport_trunk_link_s bfi_fcport_trunk_link_t;

#define BFI_FCPORT_MAX_LINKS	2
struct bfi_fcport_trunk_scn_s {
	struct bfi_mhdr_s 	mh;
	uint8_t			trunk_state;	/* bfa_trunk_state_t */
	uint8_t			trunk_speed;	/* bfa_port_speed_t */
	uint8_t			rsvd_a[2];
	struct bfi_fcport_trunk_link_s tlink[BFI_FCPORT_MAX_LINKS];
};
typedef struct bfi_fcport_trunk_scn_s bfi_fcport_trunk_scn_t;

/**
 * fcport H2I message
 */
union bfi_fcport_h2i_msg_u {
	struct bfi_mhdr_s			*mhdr;
	struct bfi_fcport_enable_req_s		*penable;
	struct bfi_fcport_req_s			*pdisable;
	struct bfi_fcport_set_svc_params_req_s	*psetsvcparams;
	struct bfi_fcport_req_s			*pstatsget;
	struct bfi_fcport_req_s			*pstatsclear;
};
typedef union bfi_fcport_h2i_msg_u bfi_fcport_h2i_msg_t;

/**
 * fcport I2H message
 */
union bfi_fcport_i2h_msg_u {
	struct bfi_msg_s			*msg;
	struct bfi_fcport_rsp_s			*penable_rsp;
	struct bfi_fcport_rsp_s			*pdisable_rsp;
	struct bfi_fcport_rsp_s			*psetsvcparams_rsp;
	struct bfi_fcport_rsp_s			*pstatsget_rsp;
	struct bfi_fcport_rsp_s			*pstatsclear_rsp;
	struct bfi_fcport_event_s		*event;
	struct bfi_fcport_trunk_scn_s		*trunk_scn;
};
typedef union bfi_fcport_i2h_msg_u bfi_fcport_i2h_msg_t;

/**
 * FCXP
 */

enum bfi_fcxp_h2i {
	BFI_FCXP_H2I_SEND_REQ = 1,
};
typedef enum bfi_fcxp_h2i bfi_fcxp_h2i_t;

enum bfi_fcxp_i2h {
	BFI_FCXP_I2H_SEND_RSP = BFA_I2HM(1),
};
typedef enum bfi_fcxp_i2h bfi_fcxp_i2h_t;

#define BFA_FCXP_MAX_SGES	2

/**
 * FCXP send request structure
 */
struct bfi_fcxp_send_req_s {
	struct bfi_mhdr_s  mh;		/*!< Common msg header		    */
	uint16_t	fcxp_tag;	/*!< driver request tag		    */
	uint16_t	max_frmsz;	/*!< max send frame size	    */
	uint16_t	vf_id;		/*!< vsan tag if applicable	    */
	uint16_t	rport_fw_hndl;	/*!< FW Handle for the remote port  */
	uint8_t		class;		/*!< FC class used for req/rsp	    */
	uint8_t		rsp_timeout;	/*!< timeout in secs, 0-no response */
	uint8_t		cts;		/*!< continue sequence		    */
	uint8_t		lp_fwtag;	/*!< lport tag			    */
	fchs_t		fchs;		/*!< request FC header structure    */
	uint32_t	req_len;	/*!< request payload length	    */
	uint32_t	rsp_maxlen;	/*!< max response length expected   */
	struct bfi_alen_s  req_alen;	/*!< request buffer		    */
	struct bfi_alen_s  rsp_alen;	/*!< response buffer		    */
};
typedef struct bfi_fcxp_send_req_s bfi_fcxp_send_req_t;

/**
 * FCXP send response structure
 */
struct bfi_fcxp_send_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< Common msg header		    */
	uint16_t	fcxp_tag;	/*!< send request tag		    */
	uint8_t	 req_status;	/*!< request status		    */
	uint8_t	 rsvd;
	uint32_t	rsp_len;	/*!< actual response length	    */
	uint32_t	residue_len;	/*!< residual response length	    */
	fchs_t	  fchs;		/*!< response FC header structure   */
};
typedef struct bfi_fcxp_send_rsp_s bfi_fcxp_send_rsp_t;


/**
 * UF
 */

enum bfi_uf_h2i {
	BFI_UF_H2I_BUF_POST = 1,
};
typedef enum bfi_uf_h2i bfi_uf_h2i_t;

enum bfi_uf_i2h {
	BFI_UF_I2H_FRM_RCVD = BFA_I2HM(1),
};
typedef enum bfi_uf_i2h bfi_uf_i2h_t;

#define BFA_UF_MAX_SGES	2

struct bfi_uf_buf_post_s {
	struct bfi_mhdr_s  mh;		/*!< Common msg header		*/
	uint16_t	buf_tag;	/*!< buffer tag			*/
	uint16_t	buf_len;	/*!< total buffer length	*/
	struct bfi_alen_s  alen;	/*!< buffer address/len pair	*/
};
typedef struct bfi_uf_buf_post_s bfi_uf_buf_post_t;

struct bfi_uf_frm_rcvd_s {
	struct bfi_mhdr_s  mh;		/*!< Common msg header		*/
	uint16_t	buf_tag;	/*!< buffer tag			*/
	uint16_t	rsvd;
	uint16_t	frm_len;	/*!< received frame length 	*/
	uint16_t	xfr_len;	/*!< tranferred length		*/
};
typedef struct bfi_uf_frm_rcvd_s bfi_uf_frm_rcvd_t;


/**
 * LPS
 */
enum bfi_lps_h2i_msgs {
	BFI_LPS_H2I_LOGIN_REQ	= 1,
	BFI_LPS_H2I_LOGOUT_REQ	= 2,
	BFI_LPS_H2I_N2N_PID_REQ	= 3,
};
typedef enum bfi_lps_h2i_msgs bfi_lps_h2i_msgs_t;

enum bfi_lps_i2h_msgs {
	BFI_LPS_I2H_LOGIN_RSP	= BFA_I2HM(1),
	BFI_LPS_I2H_LOGOUT_RSP	= BFA_I2HM(2),
	BFI_LPS_I2H_CVL_EVENT	= BFA_I2HM(3),
};
typedef enum bfi_lps_i2h_msgs bfi_lps_i2h_msgs_t;

struct bfi_lps_login_req_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint8_t		bfa_tag;
	uint8_t		alpa;
	uint16_t	pdu_size;
	wwn_t		pwwn;
	wwn_t		nwwn;
	uint8_t		fdisc;
	uint8_t		auth_en;
	uint8_t		lps_role;
	uint8_t		bb_scn;
	uint32_t	vvl_flag;
};
typedef struct bfi_lps_login_req_s bfi_lps_login_req_t;

struct bfi_lps_login_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint8_t		fw_tag;
	uint8_t		status;
	uint8_t		lsrjt_rsn;
	uint8_t		lsrjt_expl;
	wwn_t		port_name;
	wwn_t		node_name;
	uint16_t	bb_credit;
	uint8_t		f_port;
	uint8_t		npiv_en;
	uint32_t	lp_pid : 24;
	uint32_t	auth_req : 8;
	mac_t		lp_mac;
	mac_t		fcf_mac;
	uint8_t		ext_status;
	uint8_t		brcd_switch;	/*!< attached peer is brcd switch */
	uint8_t		bfa_tag;
	uint8_t		rsvd;
};
typedef struct bfi_lps_login_rsp_s bfi_lps_login_rsp_t;

struct bfi_lps_logout_req_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint8_t		fw_tag;
	uint8_t		rsvd[3];
	wwn_t		port_name;
};
typedef struct bfi_lps_logout_req_s bfi_lps_logout_req_t;

struct bfi_lps_logout_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint8_t		bfa_tag;
	uint8_t		status;
	uint8_t		rsvd[2];
};
typedef struct bfi_lps_logout_rsp_s bfi_lps_logout_rsp_t;

struct bfi_lps_cvl_event_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint8_t		bfa_tag;
	uint8_t		rsvd[3];
};
typedef struct bfi_lps_cvl_event_s bfi_lps_cvl_event_t;

struct bfi_lps_n2n_pid_req_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint8_t		fw_tag;
	uint32_t	lp_pid : 24;
};
typedef struct bfi_lps_n2n_pid_req_s bfi_lps_n2n_pid_req_t;

union bfi_lps_h2i_msg_u {
	struct bfi_mhdr_s		*msg;
	struct bfi_lps_login_req_s	*login_req;
	struct bfi_lps_logout_req_s	*logout_req;
	struct bfi_lps_n2n_pid_req_s	*n2n_pid_req;
};
typedef union bfi_lps_h2i_msg_u bfi_lps_h2i_msg_t;

union bfi_lps_i2h_msg_u {
	struct bfi_msg_s		*msg;
	struct bfi_lps_login_rsp_s	*login_rsp;
	struct bfi_lps_logout_rsp_s	*logout_rsp;
	struct bfi_lps_cvl_event_s	*cvl_event;
};
typedef union bfi_lps_i2h_msg_u bfi_lps_i2h_msg_t;


/**
 * RPORT
 */
enum bfi_rport_h2i_msgs {
	BFI_RPORT_H2I_CREATE_REQ = 1,
	BFI_RPORT_H2I_DELETE_REQ = 2,
	BFI_RPORT_H2I_SET_SPEED_REQ  = 3,
};
typedef enum bfi_rport_h2i_msgs bfa_rport_h2i_msgs_t;

enum bfi_rport_i2h_msgs {
	BFI_RPORT_I2H_CREATE_RSP = BFA_I2HM(1),
	BFI_RPORT_I2H_DELETE_RSP = BFA_I2HM(2),
	BFI_RPORT_I2H_QOS_SCN    = BFA_I2HM(3),
	BFI_RPORT_I2H_LIP_SCN_ONLINE    = BFA_I2HM(4),
	BFI_RPORT_I2H_LIP_SCN_OFFLINE   = BFA_I2HM(5),
	BFI_RPORT_I2H_NO_DEV	= BFA_I2HM(6),
};
typedef enum bfi_rport_i2h_msgs bfa_rport_i2h_msgs_t;

struct bfi_rport_create_req_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint16_t	bfa_handle;	/*!< host rport handle		*/
	uint16_t	max_frmsz;	/*!< max rcv pdu size		*/
	uint32_t	pid	 : 24,	/*!< remote port ID		*/
			lp_fwtag : 8;	/*!< local port tag		*/
	uint32_t	local_pid : 24,	/*!< local port ID		*/
			cisc	: 8;
	uint8_t		fc_class;	/*!< supported FC classes	*/
	uint8_t		vf_en;		/*!< virtual fabric enable	*/
	uint16_t	vf_id;		/*!< virtual fabric ID		*/
};
typedef struct bfi_rport_create_req_s bfi_rport_create_req_t;

struct bfi_rport_create_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint8_t		status;		/*!< rport creation status	*/
	uint8_t		rsvd[3];
	uint16_t	bfa_handle;	/*!< host rport handle		*/
	uint16_t	fw_handle;	/*!< firmware rport handle	*/
	struct bfa_rport_qos_attr_s qos_attr;  /* QoS Attributes */
};
typedef struct bfi_rport_create_rsp_s bfi_rport_create_rsp_t;

struct bfa_rport_speed_req_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint16_t	fw_handle;	/*!< firmware rport handle	*/
	uint8_t		speed;		/*!< rport's speed via RPSC	*/
	uint8_t		rsvd;
};
typedef struct bfa_rport_speed_req_s bfi_rport_speed_req_t;

struct bfi_rport_delete_req_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint16_t	fw_handle;	/*!< firmware rport handle	*/
	uint16_t	rsvd;
};
typedef struct bfi_rport_delete_req_s bfi_rport_delete_req_t;

struct bfi_rport_delete_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint16_t	bfa_handle;	/*!< host rport handle		*/
	uint8_t		status;		/*!< rport deletion status	*/
	uint8_t		rsvd;
};
typedef struct bfi_rport_delete_rsp_s bfi_rport_delete_rsp_t;

struct bfi_rport_qos_scn_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		*/
	uint16_t	bfa_handle;	/*!< host rport handle		*/
	uint16_t	rsvd;
	struct bfa_rport_qos_attr_s old_qos_attr;  /* Old QoS Attributes */
	struct bfa_rport_qos_attr_s new_qos_attr;  /* New QoS Attributes */
};
typedef struct bfi_rport_qos_scn_s bfi_rport_qos_scn_t;

struct bfi_rport_lip_scn_s {
	struct bfi_mhdr_s  mh;      /*!< common msg header      */
	uint16_t    bfa_handle; /*!< host rport handle      */
	uint8_t     status;     /*!< scn online status  */
	uint8_t    rsvd;
	struct bfa_fcport_loop_info_s loop_info;
};
typedef struct bfi_rport_lip_scn_s bfi_rport_lip_scn_t;

union bfi_rport_h2i_msg_u {
	struct bfi_msg_s 		*msg;
	struct bfi_rport_create_req_s	*create_req;
	struct bfi_rport_delete_req_s	*delete_req;
	struct bfi_rport_speed_req_s	*speed_req;
};
typedef union bfi_rport_h2i_msg_u bfi_rport_h2i_msg_t;

union bfi_rport_i2h_msg_u {
	struct bfi_msg_s 		*msg;
	struct bfi_rport_create_rsp_s	*create_rsp;
	struct bfi_rport_delete_rsp_s	*delete_rsp;
	struct bfi_rport_qos_scn_s	*qos_scn_evt;
	struct bfi_rport_lip_scn_s  *lip_scn;
};
typedef union bfi_rport_i2h_msg_u bfi_rport_i2h_msg_t;


/**
 * FCPIM
 */

/*
 * Initiator mode I-T nexus interface defines.
 */

enum bfi_itn_h2i {
	BFI_ITN_H2I_CREATE_REQ = 1,	/*!< i-t nexus creation */
	BFI_ITN_H2I_DELETE_REQ = 2,	/*!< i-t nexus deletion */
};
typedef enum bfi_itn_h2i bfi_itn_h2i_t;

enum bfi_itn_i2h {
	BFI_ITN_I2H_CREATE_RSP = BFA_I2HM(1),
	BFI_ITN_I2H_DELETE_RSP = BFA_I2HM(2),
	BFI_ITN_I2H_SLER_EVENT = BFA_I2HM(3),
};
typedef enum bfi_itn_i2h bfi_itn_i2h_t;

typedef enum {
	BFI_ITN_ROLE_IM		= 1,	/*!< Port role Initiator	    */
	BFI_ITN_ROLE_TM		= 2,	/*!< Port role Target		    */
} itn_role_t;

struct bfi_itn_create_req_s {
	struct bfi_mhdr_s	mh;	/*!< common msg header		 */
	uint16_t 	fw_handle;	/*!< f/w handle for itn	 */
	uint8_t		class;		/*!< FC class for IO		 */
	uint8_t		seq_rec;	/*!< sequence recovery support	 */
	uint8_t		msg_no;		/*!< seq id of the msg		 */
	itn_role_t	role;		/*!< port type: IM or TM	 */
};
typedef struct bfi_itn_create_req_s bfi_itn_create_req_t;

struct bfi_itn_create_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		 */
	uint16_t	bfa_handle;	/*!< bfa handle for itn	 */
	uint8_t		status;		/*!< fcp request status		 */
	uint8_t		seq_id;		/*!< seq id of the msg		 */
};
typedef struct bfi_itn_create_rsp_s bfi_itn_create_rsp_t;

struct bfi_itn_delete_req_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		 */
	uint16_t	fw_handle;	/*!< f/w itn handle		 */
	uint8_t		seq_id;		/*!< seq id of the msg		 */
	uint8_t		rsvd;
};
typedef struct bfi_itn_delete_req_s bfi_itn_delete_req_t;

struct bfi_itn_delete_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		 */
	uint16_t	bfa_handle;	/*!< bfa handle for itn	 */
	uint8_t		status;		/*!< fcp request status		 */
	uint8_t		seq_id;		/*!< seq id of the msg		 */
};
typedef struct bfi_itn_delete_rsp_s bfi_itn_delete_rsp_t;

struct bfi_itn_sler_event_s {
	struct bfi_mhdr_s  mh;		/*!< common msg header		 */
	uint16_t	bfa_handle;	/*!< bfa handle for itn	 */
	uint16_t	rsvd;
};
typedef struct bfi_itn_sler_event_s bfi_itn_sler_event_t;

union bfi_itn_h2i_msg_u {
	struct bfi_itn_create_req_s *create_req;
	struct bfi_itn_delete_req_s *delete_req;
	struct bfi_msg_s	*msg;
};
typedef union bfi_itn_h2i_msg_u bfi_itn_h2i_msg_t;

union bfi_itn_i2h_msg_u {
	struct bfi_itn_create_rsp_s *create_rsp;
	struct bfi_itn_delete_rsp_s *delete_rsp;
	struct bfi_itn_sler_event_s *sler_event;
	struct bfi_msg_s	*msg;
};
typedef union bfi_itn_i2h_msg_u bfi_itn_i2h_msg_t;

/*
 * Initiator mode IO interface defines.
 */

enum bfi_ioim_h2i {
	BFI_IOIM_H2I_IOABORT_REQ = 1,	/*!< IO abort request	 */
	BFI_IOIM_H2I_IOCLEANUP_REQ = 2,	/*!< IO cleanup request	 */
};
typedef enum bfi_ioim_h2i bfi_ioim_h2i_t;

enum bfi_ioim_i2h {
	BFI_IOIM_I2H_IO_RSP = BFA_I2HM(1),	/*!< non-fp IO response	 */
	BFI_IOIM_I2H_IOABORT_RSP = BFA_I2HM(2),	/*!< ABORT rsp	 */
};
typedef enum bfi_ioim_i2h bfi_ioim_i2h_t;

/**
 * IO command DIF info
 */
struct bfi_ioim_dif_s {
	uint32_t	dif_info[4];
};
typedef struct bfi_ioim_dif_s bfi_ioim_dif_t;

/**
 * FCP IO messages overview
 *
 * @note
 * - Max CDB length supported is 64 bytes.
 * - SCSI Linked commands and SCSI bi-directional Commands not
 *	supported.
 *
 */
struct bfi_ioim_req_s {
	struct bfi_mhdr_s  mh;		/*!< Common msg header		 */
	uint16_t	io_tag;		/*!< I/O tag			 */
	uint16_t	rport_hdl;	/*!< itn/rport firmware handle */
	fcp_cmnd_t	cmnd;		/*!< IO request info		 */

	/**
	 * SG elements array within the IO request must be double word
	 * aligned. This aligment is required to optimize SGM setup for the IO.
	 */
	struct bfi_sge_s	sges[BFI_SGE_INLINE_MAX];
	uint8_t	io_timeout;
	uint8_t	dif_en;
	uint8_t	rsvd_a[2];
	struct bfi_ioim_dif_s  dif;
};
typedef struct bfi_ioim_req_s bfi_ioim_req_t;

/**
 *  @brief
 * 	This table shows various IO status codes from firmware and their
 *	meaning. Host driver can use these status codes to further process
 *	IO completions.
 *
 * 	BFI_IOIM_STS_OK		: IO completed with error free SCSI &
 *						transport status.
 * 						- io-tag can be reused.
 *
 * 	BFA_IOIM_STS_SCSI_ERR		: IO completed with scsi error.
 *	- io-tag can be reused.
 *
 * 	BFI_IOIM_STS_HOST_ABORTED	: IO was aborted successfully due to
 *						host request.
 * 						- io-tag cannot be reused yet.
 *
 * 	BFI_IOIM_STS_ABORTED		: IO was aborted successfully
 *						internally by f/w.
 * 						- io-tag cannot be reused yet.
 *
 * 	BFI_IOIM_STS_TIMEDOUT	: IO timedout and ABTS/RRQ is happening
 *						in the firmware and
 * 						- io-tag cannot be reused yet.
 *
 * 	BFI_IOIM_STS_SQER_NEEDED	: Firmware could not recover the IO
 *					  with sequence level error
 *	logic and hence host needs to retry
 *					  this IO with a different IO tag
 *					- io-tag cannot be used yet.
 *
 * 	BFI_IOIM_STS_NEXUS_ABORT	: Second Level Error Recovery from host
 *					  is required because 2 consecutive ABTS
 *					  timedout and host needs logout and
 *					  re-login with the target
 *					- io-tag cannot be used yet.
 *
 * 	BFI_IOIM_STS_UNDERRUN	: IO completed with SCSI status good,
 *					  but the data tranferred is less than
 *					  the fcp data length in the command.
 *					  ex. SCSI INQUIRY where transferred
 *					  data length and residue count in FCP
 *					  response accounts for total fcp-dl
 *					  - io-tag can be reused.
 *
 * 	BFI_IOIM_STS_OVERRUN	: IO completed with SCSI status good,
 *					  but the data transerred is more than
 *					  fcp data length in the command. ex.
 *					  TAPE IOs where blocks can of unequal
 *					  lengths.
 *					- io-tag can be reused.
 *
 * 	BFI_IOIM_STS_RES_FREE	: Firmware has completed using io-tag
 *					  during abort process
 *					- io-tag can be reused.
 *
 * 	BFI_IOIM_STS_PROTO_ERR	: Firmware detected a protocol error.
 *					  ex target sent more data than
 *					  requested, or there was data frame
 *					  loss and other reasons
 *					- io-tag cannot be used yet.
 *
 * 	BFI_IOIM_STS_DIF_ERR	: Firwmare detected DIF error. ex: DIF
 *					CRC err or Ref Tag err or App tag err.
 *					- io-tag can be reused.
 *
 * 	BFA_IOIM_STS_TSK_MGT_ABORT	: IO was aborted because of Task
 *					  Management command from the host
 *					  - io-tag can be reused.
 *
 * 	BFI_IOIM_STS_UTAG		: Firmware does not know about this
 *					  io_tag.
 *					- io-tag can be reused.
 */
enum bfi_ioim_status {
	BFI_IOIM_STS_OK = 0,
	BFI_IOIM_STS_HOST_ABORTED = 1,
	BFI_IOIM_STS_ABORTED = 2,
	BFI_IOIM_STS_TIMEDOUT = 3,
	BFI_IOIM_STS_RES_FREE = 4,
	BFI_IOIM_STS_SQER_NEEDED = 5,
	BFI_IOIM_STS_PROTO_ERR = 6,
	BFI_IOIM_STS_UTAG = 7,
	BFI_IOIM_STS_PATHTOV = 8,
};
typedef enum bfi_ioim_status bfi_ioim_status_t;

/**
 * I/O response message
 */
struct bfi_ioim_rsp_s {
	struct bfi_mhdr_s	mh;	/*!< common msg header		*/
	uint16_t	io_tag;		/*!< completed IO tag		 */
	uint16_t	bfa_rport_hndl;	/*!< releated rport handle	 */
	uint8_t		io_status;	/*!< IO completion status	*/
	uint8_t		reuse_io_tag;	/*!< IO tag can be reused	*/
	uint16_t	abort_tag;	/*!< host abort request tag	*/
	uint8_t		scsi_status;	/*!< scsi status from target	 */
	uint8_t		sns_len;	/*!< scsi sense length		 */
	uint8_t		resid_flags;	/*!< IO residue flags		 */
	uint8_t		rsvd_a;
	uint32_t	residue;	/*!< IO residual length in bytes */
	uint32_t	io_lat;		/*!< IO latency ticks		 */
	uint32_t	rsvd_c[2];
};
typedef struct bfi_ioim_rsp_s bfi_ioim_rsp_t;

struct bfi_ioim_abort_req_s {
	struct bfi_mhdr_s  mh;	/*!< Common msg header  */
	uint16_t	io_tag;	/*!< I/O tag	*/
	uint16_t	abort_tag;	/*!< unique request tag */
};
typedef struct bfi_ioim_abort_req_s bfi_ioim_abort_req_t;

/*
 * Initiator mode task management command interface defines.
 */

enum bfi_tskim_h2i {
	BFI_TSKIM_H2I_TM_REQ	= 1, /*!< task-mgmt command	*/
	BFI_TSKIM_H2I_ABORT_REQ = 2, /*!< task-mgmt command	*/
};
typedef enum bfi_tskim_h2i bfi_tskim_h2i_t;

enum bfi_tskim_i2h {
	BFI_TSKIM_I2H_TM_RSP = BFA_I2HM(1),
};
typedef enum bfi_tskim_i2h bfi_tskim_i2h_t;

struct bfi_tskim_req_s {
	struct bfi_mhdr_s  mh;	/*!< Common msg header	*/
	uint16_t	tsk_tag;	/*!< task management tag	*/
	uint16_t	itn_fhdl;	/*!< itn firmware handle	*/
	lun_t	lun;	/*!< LU number	*/
	uint8_t	tm_flags;	/*!< see fcp_tm_cmnd_t	*/
	uint8_t	t_secs;	/*!< Timeout value in seconds	*/
	uint8_t	rsvd[2];
};
typedef struct bfi_tskim_req_s bfi_tskim_req_t;

struct bfi_tskim_abortreq_s {
	struct bfi_mhdr_s  mh;	/*!< Common msg header	*/
	uint16_t	tsk_tag;	/*!< task management tag	*/
	uint16_t	rsvd;
};
typedef struct bfi_tskim_abortreq_s bfi_tskim_abortreq_t;

enum bfi_tskim_status {
	/*
	 * Following are FCP-4 spec defined status codes,
	 * **DO NOT CHANGE THEM **
	 */
	BFI_TSKIM_STS_OK	= 0,
	BFI_TSKIM_STS_NOT_SUPP = 4,
	BFI_TSKIM_STS_FAILED	= 5,

	/**
	 * Defined by BFA
	 */
	BFI_TSKIM_STS_TIMEOUT  = 10,	/*!< TM request timedout	*/
	BFI_TSKIM_STS_ABORTED  = 11,	/*!< Aborted on host request */
	BFI_TSKIM_STS_UTAG  = 12,	/*!< unknown tag for request */
};
typedef enum bfi_tskim_status bfi_tskim_status_t;

struct bfi_tskim_rsp_s {
	struct bfi_mhdr_s  mh;		/*!< Common msg header		 */
	uint16_t	tsk_tag;	/*!< task mgmt cmnd tag		 */
	uint8_t	tsk_status;	/*!< @ref bfi_tskim_status */
	uint8_t	rsvd;
};
typedef struct bfi_tskim_rsp_s bfi_tskim_rsp_t;

/**
 * FCPTM
 */
#define BFI_TIO_SNSLEN 256

enum bfi_tiods_status {
	BFI_TIO_DS_OK = 1,
	BFI_TIO_DS_SEQ_CNT_ERR = 2,
	BFI_TIO_DS_RO_OOO_ERR = 3,
	BFI_TIO_DS_DATA_OUT_OF_RANGE = 4,
	BFI_TIO_DS_SEQ_LENGTH_ERR = 5,
	BFI_TIO_DS_EDTOV_TIMER_EXP = 6,
	BFI_TIO_DS_XFER_RDY_TIMER_EXP = 7,
	BFI_TIO_DS_UN_EXP_FRM_ERR = 8,
	BFI_TIO_DS_TAG_NOT_FOUND_ERR = 9,
};
typedef enum bfi_tiods_status bfi_tiods_status_t;

enum bfi_tiods_si {
	BFI_TIO_DS_SI_UNKNOWN = 0,
	BFI_TIO_DS_SI_WITH_US = 1,
	BFI_TIO_DS_SI_NOT_WITH_US = 2,
};
typedef enum bfi_tiods_si bfi_tiods_si_t;
/*
 * Target mode SCSI command interface defines.
 */
enum bfi_tio_h2i {
	BFI_TIO_H2I_RDDATA = 1,
	BFI_TIO_H2I_RD_RSP = 2,
	BFI_TIO_H2I_WRDATA = 3,
	BFI_TIO_H2I_WR_RSP = 4,
	BFI_TIO_H2I_GOOD_RSP = 5,
	BFI_TIO_H2I_RSP = 6,
	BFI_TIO_H2I_ABORT_RSP = 7,
	BFI_TIO_H2I_CLEANUP = 8,
	BFI_TIO_H2I_SEND_ABTS = 9,
};
typedef enum bfi_tio_h2i bfi_tio_h2i_t;

enum bfi_tio_i2h {
	BFI_TIO_I2H_CMND = BFA_I2HM(1),
	BFI_TIO_I2H_IO_COMP = BFA_I2HM(2), /*!< for conf and rsp both */
	BFI_TIO_I2H_DATA_ERR = BFA_I2HM(3),
	BFI_TIO_I2H_ABORT = BFA_I2HM(4),
	BFI_TIO_I2H_CLEANUP_COMP = BFA_I2HM(5),
	BFI_TIO_I2H_UF = BFA_I2HM(6),
};
typedef enum bfi_tio_i2h bfi_tio_i2h_t;

/**
 * FCP IO messages overview
 *
 * @note
 * - Max CDB length supported is 64 bytes.
 * - SCSI Linked commands and SCSI bi-directional Commands not
 *   supported.
 *
 */
struct bfi_ds_s {
	struct bfi_mhdr_s	mh;		/*!< Common msg header	*/
	uint16_t		fwtio_tag;	/*!< I/O tag */
	uint16_t		ox_id;
	uint32_t		relative_offset;
	uint32_t		data_length;
	uint16_t		tin_tag;
	uint8_t			ds_tag;
	uint8_t			conf;
	uint32_t		fcp_dl;
	/**
	 * SG element or link within the IO request must be double word
	 * aligned. This aligment is required to optimize SGM setup for the IO.
	 */
	struct bfi_sge_s	sges[BFI_SGE_INLINE_MAX];
};
typedef struct bfi_ds_s bfi_ds_t;

struct bfi_tio_rsp_s {
	struct bfi_mhdr_s	mh;		/*!< Common msg header	*/
	uint16_t		fwtio_tag;	/*!< I/O tag		*/
	uint16_t		ox_id;
	uint16_t		tin_tag;
	uint16_t		rsvd;
	fcp_resp_t		fcp_rsp;
	fcp_rspinfo_t		fcp_rspinfo;
};
typedef struct bfi_tio_rsp_s bfi_tio_rsp_t;

struct bfi_tio_abort_s {
	struct bfi_mhdr_s	mh;		/*!< Common msg header	*/
	uint16_t		fwtio_tag;	/*!< I/O tag		*/
	uint16_t		ox_id;
	uint16_t		tin_tag;
	uint8_t			abort_status;
		/*!< used only in abts rsp FC_CAT_BA_ACC or FC_CAT_BA_RJT */
	uint8_t			rsvd;
};
typedef struct bfi_tio_abort_s bfi_tio_abort_t;

struct bfi_tio_uf_s {
	struct bfi_mhdr_s	mh;	/*!< Common msg header	*/
	uint8_t			si_owner;
	uint8_t			fchs_off; /* fchs offset in buffer */
	uint8_t			rsvd[2];
	fchs_t			fchs;	/*!< I/O tag		*/
	union {
		fc_rec_t	rec;
		fc_srr_t	srr;
		fc_rrq_t	rrq;
	} cmnd;
};
typedef struct bfi_tio_uf_s bfi_tio_uf_t;

struct bfi_tio_cleanup_s
{
	struct bfi_mhdr_s	mh;		/*!< Common msg header	*/
	uint16_t		fwtio_tag;	/*!< I/O tag		*/
	uint16_t		tin_tag;
	uint8_t			io_status;	/*!< status		*/
	uint8_t			rsvd[3];
};
typedef struct bfi_tio_cleanup_s bfi_tio_cleanup_t;

struct bfi_tio_cmnd_s {
	struct bfi_mhdr_s	mh;	/*!< Common msg header	*/
	uint8_t			fchs_off; /* fchs offset in buffer */
	uint8_t			rsvd[3];
};
typedef struct bfi_tio_cmnd_s bfi_tio_cmnd_t;

struct bfi_tio_abort_ind_s {
	struct bfi_mhdr_s	mh;	/*!< Common msg header	*/
	uint16_t		fwtio_tag;	/*!< I/O tag */
	uint16_t		ox_id;
	uint32_t		rsvd1:8;
	uint32_t		d_id:24;
	uint32_t		rsvd2:8;
	uint32_t		s_id:24;
};
typedef struct bfi_tio_abort_ind_s bfi_tio_abort_ind_t;

/* used for abts, rsp, cleanup, res_free completions */
struct bfi_tio_comp_s {
	struct bfi_mhdr_s	mh;		/*!< Common msg header	*/
	uint16_t		fwtio_tag;	/*!< I/O tag		*/
	uint16_t		io_status;	/*!<status		*/
};
typedef struct bfi_tio_comp_s bfi_tio_comp_t;

/* used for ds completions */
struct bfi_ds_comp_s {
	struct bfi_mhdr_s	mh;		/*!< Common msg header	*/
	uint16_t		fwtio_tag;	/*!< I/O tag		*/
	uint8_t			io_status;	/*!< status		*/
	uint8_t			si;		/*!< si status		*/
	uint8_t			ds_tag;
	uint8_t			rsvd[3];
};
typedef struct bfi_ds_comp_s bfi_ds_comp_t;

union bfi_tio_i2h_msg_u {
	struct bfi_mhdr_s		mh;	/*!< Common msg header	*/
	struct bfi_tio_cmnd_s		cmnd;
	struct bfi_ds_comp_s		xfer_comp;
	struct bfi_tio_abort_ind_s	abort_ind;
	struct bfi_tio_comp_s		comp;
					/*!< abts, rsp, cleanup, res_free */
};
typedef union bfi_tio_i2h_msg_u bfi_tio_i2h_msg_t;

union bfi_tio_h2i_msg_u {
	struct bfi_mhdr_s		mh;	/*!< Common msg header	*/
	struct bfi_ds_s			scsi_data;
	struct bfi_tio_rsp_s 		scsi_rsp;
	struct bfi_tio_abort_s		abort;
	struct bfi_tio_cleanup_s 	cleanup;
};
typedef union bfi_tio_h2i_msg_u bfi_tio_h2i_msg_t;

/**
 * LPORT
 */
enum bfa_lport_role_e {
	BFI_LPORT_ROLE_FCP_IM 	= 0x01,	/*!< FCP initiator role */
	BFI_LPORT_ROLE_FCP_TM 	= 0x02,	/*!< FCP target role */
	BFI_LPORT_ROLE_FCP_IPFC	= 0x04,	/*!< IP over FC role */
	BFI_LPORT_ROLE_FCP_MAX 	=
			BFI_LPORT_ROLE_FCP_IPFC | BFI_LPORT_ROLE_FCP_IM
};

/**
 * SBOOT
 */

typedef enum {
	BFI_SBOOT_H2I_INIT_REQ		= 1,	/*!< initialize request */
	BFI_SBOOT_H2I_OPENBLUN_REQ	= 2,	/*!< open boot lun	*/
	BFI_SBOOT_H2I_DISC_RPORTS_REQ	= 3,	/*!< list remote ports  */
	BFI_SBOOT_H2I_DISC_LUNS_REQ	= 4,	/*!< list LUNs of rport */
	BFI_SBOOT_H2I_IORW_REQ		= 5,	/*!< block read request */
	BFI_SBOOT_H2I_UNINIT_REQ	= 8	/*!< uninitialization   */
} bfi_sboot_h2i_t;

typedef enum {
	BFI_SBOOT_I2H_INIT_RSP		= BFA_I2HM(1),
	BFI_SBOOT_I2H_OPENBLUN_RSP	= BFA_I2HM(2),
	BFI_SBOOT_I2H_DISC_RPORTS_RSP	= BFA_I2HM(3),
	BFI_SBOOT_I2H_DISC_LUNS_RSP	= BFA_I2HM(4),
	BFI_SBOOT_I2H_IORW_RSP		= BFA_I2HM(5),
	BFI_SBOOT_I2H_OFFLINE_EVT	= BFA_I2HM(6),
	BFI_SBOOT_I2H_IO_RETRY		= BFA_I2HM(7),
	BFI_SBOOT_I2H_UNINIT_RSP	= BFA_I2HM(8)
} bfi_sboot_i2h_t;

/**
 * Structure of remote port DMAed by firmware
 */
typedef struct {
	wwn_t			pwwn;		/*!< port wwn of target port  */
	uint32_t		fc_pid;		/*!< FC port ID		      */
	uint16_t		maxfrsz;	/*!< maximum receive pdu size */
	uint16_t		bbcredit;	/*!< tx bb-credits	      */
	scsi_inquiry_vendor_t	vendor_id;	/*!< vendor ID from INQUIRY   */
	scsi_inquiry_prodid_t	product_id;	/*!< product ID from INQUIRY  */
	scsi_inquiry_prodrev_t	product_rev;	/*!< product rev from INQUIRY */
	uint32_t		rsvd_b;		/*!< for 8-byte multiples     */
} bfi_sboot_rport_t;

/**
 * Structure of LUN list DMAed by firmware.
 */
typedef struct {
	lun_t			lun;		/*!< 8-byte lun number	      */
	uint32_t		nblocks;	/*!< lun size (read capacity) */
	uint32_t		blksize;	/*!< block size (read cap)    */
	scsi_inquiry_vendor_t	vendor_id;	/*!< vendor ID from INQUIRY   */
	scsi_inquiry_prodid_t	product_id;	/*!< product ID from INQUIRY  */
	scsi_inquiry_prodrev_t	product_rev;	/*!< product rev from INQUIRY */
	uint32_t		nblocks_hi;	/*!< lun size(read cap16)     */
} bfi_sboot_lun_t;

/**
 * San boot configuration.
 */
typedef struct {
	uint8_t		speed;		/*!< bfa_port_speed_t		*/
	uint8_t		topology;	/*!< bfa_port_topology_t	*/
	uint8_t		bootopt;	/*!< bfa_boot_bootopt_t     */
	uint8_t		nbluns;		/*!< number of boot-luns	*/
	uint16_t	rports_max;	/*!< size of remote ports array	*/
	uint16_t	rluns_max;	/*!< size of luns array		*/
	bfa_boot_bootlun_t blun[BFA_BOOT_BOOTLUN_MAX];
	uint32_t	rports_off;	/*!< remote port list		*/
	uint32_t	luns_off;	/*!< remote lun list		*/
} bfa_sboot_config_t;

/**
 * San boot initialization request.
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		speed;		/*!< bfa_port_speed_t		*/
	uint8_t		topology;	/*!< bfa_port_topology_t	*/
	uint8_t		bootopt;	/*!< bfa_boot_bootopt_t     */
	uint8_t		nbluns;		/*!< number of boot-luns	*/
	uint16_t	rports_max;	/*!< size of remote ports array	*/
	uint16_t	rluns_max;	/*!< size of luns array		*/
} bfi_sboot_init_req_t;

/**
 * san boot initialization response.
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< common msg header		*/
	uint8_t		status;		/*!< initialization status	*/
	uint8_t		speed;	/*!< link speed			*/
	uint8_t		topology;	/*!< link topology		*/
	uint8_t		linkstate;	/*!< link state			*/
	uint32_t	mypid;		/*!< local FC port ID		*/
} bfi_sboot_init_rsp_t;

/**
 * San boot uninitialization request.
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint32_t	rsvd_a;		/*!< reserved			*/
} bfi_sboot_uninit_req_t;

/**
 * san boot uninitialization response.
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint32_t	status;		/*!< success/failure		*/
} bfi_sboot_uninit_rsp_t;

/**
 * San boot open boot-lun request
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	wwn_t		pwwn;		/*!< target port name		*/
	lun_t		lun;		/*!< boot LU number		*/
	uint32_t	autodisc;	/*!< auto discover		*/
	bfi_addr_t	rplu_list;	/*!< rport list addr		*/
} bfi_sboot_openblun_req_t;

/**
 * san boot open boot-lun response.
 */
typedef struct {
	bfi_mhdr_t		mh;		/*!< common msg header	      */
	uint8_t			status;		/*!< open boot lun status     */
	uint8_t			rsvd[3];
	uint32_t		rpid;	/*!< Remote Port id */
	uint64_t		pwwn;	/*!< targets PWWN */
	uint64_t		lun;	/*!< boot LUN */
} bfi_sboot_openblun_rsp_t;

/**
* san boot: remote port list request
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	bfi_addr_t	rplu_list;	/*!< rport list addr		*/
	uint32_t	nrports;	/*!< max rports to discover */
} bfi_sboot_disc_rports_req_t;

/**
* san boot: remote port list response
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		status;		/*!< initialization status	*/
	uint8_t		rsvd;
	uint16_t	nrports;	/*!< number of remote ports	*/
} bfi_sboot_disc_rports_rsp_t;

/**
* san boot: request for list of LUNs of a remote port
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint32_t	rpid;		/*!< remote port PID		*/
	bfi_addr_t	rplu_list;	/*!< lun list addr		*/
	wwn_t		pwwn;		/*!< remote port name		*/
	uint32_t	nluns;		/*!< max luns to discover	*/
} bfi_sboot_disc_luns_req_t;

/**
* san boot: LUN list response
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		status;		/*!< initialization status	*/
	uint8_t		rsvd;
	uint16_t	nluns;		/*!< number of luns		*/
} bfi_sboot_disc_luns_rsp_t;


#define SBOOT_TGT_TYPE_FC (0)
#define SBOOT_TGT_TYPE_ISCSI (8)

/**
 * iSCSI boot initialization request.
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		init_type;	/*!< init type iSCSI == 8	*/
	uint16_t	retry_count;	/*!< TCP connection retries	*/
	uint16_t	tcp_timeout;	/*!< TCP connection timeo(secs)	*/
	bfa_boolean_t	use_dhcp;	/*!< Use DHCP to set up TCP/IP	*/
	bfa_boolean_t	use_dhcp_203;	/*!< DHCP option 203 for target	*/
	/* TCP/IP interface configuration parms */
	uint32_t	this_addr;	/*!< clients IP address		*/
	uint32_t	netmask;	/*!< clients netmask value	*/
	uint32_t	gateway;	/*!< clients routing gateway	*/
	/* iSCSI target configuration parms */
	uint32_t	tgt_addr;	/*!< Target IP address		*/
	uint16_t	tgt_port;	/*!< Target TCP port, default 3260 */
	char		initiator_name[256];
	char		target_name[256];
} bfi_iboot_init_req_t;

/**
 * iSCSI boot initialization response.
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		init_type;	/*!< init type iSCSI == 8	*/
	uint8_t		status;		/*!< initialization status	*/
	uint8_t		linkstate;	/*!< link state			*/
	uint32_t	my_iscsi;	/*!< iSCSI connection handle	*/
} bfi_iboot_init_rsp_t;

/**
 * iSCSI boot open boot-lun request
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		open_type;	/*!< open type iSCSI == 8	*/
	lun_t		lun;		/*!< boot LU number		*/
} bfi_iboot_openblun_req_t;

/**
 * iSCSI boot open boot-lun response.
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		open_type;	/*!< open type iSCSI == 8	*/
	uint8_t		status;		/*!< open boot lun status	*/
	uint8_t		rsvd[3];
	uint64_t	lun;		/*!< boot LUN			*/
} bfi_iboot_openblun_rsp_t;

/**
* iSCSI boot: IO read/write request
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		rw_type;	/*!< open type iSCSI == 8	*/
	uint8_t		iotype;		/*!< IO type: read/write	*/
	uint8_t		rsvd[3];
	uint32_t	blkoff;		/*!< offset in disk blocks	*/
	uint32_t	nblks;		/*!< IO size in blocks		*/
	bfi_addr_t	bufaddr;	/*!< IO buffer address		*/
} bfi_iboot_iorw_req_t;

/**
 * IO read/write request response
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		open_type;	/*!< open type iSCSI == 8	*/
	uint8_t		status;		/*!< IO completion status	*/
	uint8_t		rsvd[3];
} bfi_iboot_iorw_rsp_t;


/**
 * IO type values for IO read/write request
 */
typedef enum {
	BFI_SBOOT_IOWRITE = 1,
	BFI_SBOOT_IOREAD = 2,
} bfi_sboot_iotype_t;

/**
* san boot: IO read/write request
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		iotype;		/*!< IO type: read/write	*/
	uint8_t		rsvd[3];
	uint32_t	blkoff;		/*!< offset in disk blocks	*/
	uint32_t	nblks;		/*!< IO size in blocks		*/
	bfi_addr_t	bufaddr;	/*!< IO buffer address		*/
} bfi_sboot_iorw_req_t;

/**
 * IO read/write request response
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		status;		/*!< IO completion status	*/
	uint8_t		rsvd[3];
} bfi_sboot_iorw_rsp_t;

/**
 * Offline message
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header		*/
	uint8_t		rsn;		/*!< reason for offline */
	uint8_t		rsvd[3];
} bfi_sboot_offline_evt_t;

/**
 * IO retry message
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header  */
	uint8_t		rsn;		/*!< reason for IO retry */
	uint8_t		rsvd[3];
} bfi_sboot_io_retry_t;

/**
 * Host to firmware message.
 */
typedef union {
	bfi_mhdr_t			*mhdr;
	bfi_sboot_init_req_t		*init;
	bfi_sboot_openblun_req_t	*openblun;
	bfi_sboot_disc_rports_req_t	*disc_rports;
	bfi_sboot_disc_luns_req_t	*disc_luns;
	bfi_sboot_iorw_req_t		*iorw;
	bfi_sboot_uninit_req_t		*uninit;
	bfi_iboot_init_req_t		*iscsi_init;
	bfi_iboot_openblun_req_t	*iscsi_openblun;
	bfi_iboot_iorw_req_t		*iscsi_iorw;
} bfi_sboot_h2i_msg_t;

/**
 * Firmware to host message.
 */
typedef union {
	bfi_mbmsg_t			*msg;
	bfi_sboot_init_rsp_t		*init;
	bfi_sboot_openblun_rsp_t	*open_blun;
	bfi_sboot_disc_rports_rsp_t	*disc_rports;
	bfi_sboot_disc_luns_rsp_t	*disc_luns;
	bfi_sboot_iorw_rsp_t		*iorw;
	bfi_sboot_offline_evt_t		*offline;
	bfi_sboot_io_retry_t		*io_retry;
	bfi_sboot_uninit_rsp_t		*uninit;
	bfi_iboot_init_rsp_t		*iscsi_init;
	bfi_iboot_openblun_rsp_t	*iscsi_open_blun;
	bfi_iboot_iorw_rsp_t		*iscsi_iorw;
} bfi_sboot_i2h_msg_t;

#pragma pack()

/*
 * Crossbow PCI MSI-X vector defines
 */
enum {
	BFI_MSIX_CPE_QMIN_CB = 0,
	BFI_MSIX_CPE_QMAX_CB = 7,
	BFI_MSIX_RME_QMIN_CB = 8,
	BFI_MSIX_RME_QMAX_CB = 15,
	BFI_MSIX_CB_MAX = 22,
};

#define BFI_MSIX_RME_P0_QMIN_CB	BFI_MSIX_RME_QMIN_CB
#define BFI_MSIX_RME_P0_QMAX_CB	11
#define BFI_MSIX_RME_P1_QMIN_CB	12
#define BFI_MSIX_RME_P1_QMAX_CB	BFI_MSIX_RME_QMAX_CB

/*
 * Catapult FC PCI MSI-X vector defines
 */
enum {
	BFI_MSIX_LPU_ERR_CT = 0,
	BFI_MSIX_CPE_QMIN_CT = 1,
	BFI_MSIX_CPE_QMAX_CT = 4,
	BFI_MSIX_RME_QMIN_CT = 5,
	BFI_MSIX_RME_QMAX_CT = 8,
	BFI_MSIX_CT_MAX = 9,
};

#endif /* __BFI_MS_H__ */
