/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
* @file: bfad_iprp.h - BFA FCS IP mode remote port callbacks
 */

#ifndef __BFAD_FCB_IPFC_H__
#define __BFAD_FCB_IPFC_H__

typedef struct bfad_iprp_s bfad_iprp_t;

/*
 * IPRP callbacks
 */

/**
 * @brief
 * 	Memory allocation for remote port instance. Called before PRLI is
 * 	initiated to the remote IP port.
 *
 * @param[in] bfad		- driver instance
 * @param[out] iprp		- FCS remote port (IM) instance
 * @param[out] iprp_drv		- driver remote port (IM) instance
 *
 * @return None
 */
void            bfa_fcb_iprp_alloc(struct bfad_s *bfad, struct bfa_fcs_iprp_s **iprp,
				   bfad_iprp_t **iprp_drv);

/**
 * @brief
 * 	Free remote port (IM) instance.
 *
 * @param[in] bfad	- driver instance
 * @param[in] iprp_drv	- driver remote port instance
 *
 * @return None
 */
void            bfa_fcb_iprp_free(struct bfad_s *bfad, bfad_iprp_t *iprp_drv);

/**
 * @brief
 * 	Notification of when login with a remote IP device is complete.
 *
 * @param[in] bfad	- driver instance
 * @param[in] iprp_drv	- driver remote port instance
 *
 * @return None
 */
void            bfa_fcb_iprp_online(struct bfa_fcs_iprp_s *iprp,
				    bfad_iprp_t *iprp_drv);

/**
 * @brief
 * 	Notification when login with the remote device is severed.
 *
 * @param[in] bfad	- driver instance
 * @param[in] iprp_drv	- driver remote port instance
 *
 * @return None
 */
void            bfa_fcb_iprp_offline(struct bfa_fcs_iprp_s *iprp,
				     bfad_iprp_t *iprp_drv);

#endif /* __BFAD_FCB_IPFC_H__ */
