/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCS_H__
#define __BFA_FCS_H__

#include <cs/bfa_cs.h>
#include <defs/bfa_defs.h>
#include <bfa/bfa_modules.h>
#include <protocol/bfa_fc.h>
#include <fcs/bfa_fcs_tmops.h>

#define BFA_FCS_OS_STR_LEN		64

/*
 * !!! Only append to the enums defined here to avoid any versioning
 * !!! needed between trace utility and driver version
 */
enum {
	BFA_TRC_FCS_FCS		= 1,
	BFA_TRC_FCS_PORT	= 2,
	BFA_TRC_FCS_RPORT	= 3,
	BFA_TRC_FCS_FCPIM	= 4,
	BFA_TRC_FCS_FCPTM	= 5,
	BFA_TRC_FCS_AUTH	= 6,
};


struct bfa_fcs_s;

#define __fcs_min_cfg(__fcs)	(__fcs)->min_cfg
void bfa_fcs_modexit_comp(struct bfa_fcs_s *fcs);

#define BFA_FCS_BRCD_SWITCH_OUI  0x051e
#define	BFA_FCS_RETRY_TIMEOUT 2000
#define BFA_FCS_MAX_NS_RETRIES	5
#define BFA_FCS_PID_IS_WKA(pid)  ((bfa_os_ntoh3b(pid) > 0xFFF000) ?  1 : 0)
#define BFA_FCS_MAX_RPORT_LOGINS	1024


/**
 * @brief ns data structure
 */
struct bfa_fcs_lport_ns_s {
	bfa_sm_t	sm;		/*!< state machine */
	struct bfa_timer_s timer;
	struct bfa_fcs_lport_s *port;	/*!< parent port */
	struct bfa_fcxp_s *fcxp;
	struct bfa_fcxp_wqe_s fcxp_wqe;
	uint8_t num_rnnid_retries;
	uint8_t	num_rsnn_nn_retries;
};
typedef struct bfa_fcs_lport_ns_s bfa_fcs_lport_ns_t;

/**
 * @brief fabric scn data structure
 */
struct bfa_fcs_lport_scn_s {
	bfa_sm_t	sm;		/*!< state machine */
	struct bfa_timer_s timer;
	struct bfa_fcs_lport_s *port;	/*!< parent port */
	struct bfa_fcxp_s *fcxp;
	struct bfa_fcxp_wqe_s fcxp_wqe;
};
typedef struct bfa_fcs_lport_scn_s bfa_fcs_lport_scn_t;

/**
 * @brief ms fdmi data structure
 */
struct bfa_fcs_lport_fdmi_s {
	bfa_sm_t	sm;		/*!< state machine */
	struct bfa_timer_s timer;
	struct bfa_fcs_lport_ms_s *ms;	/*!< parent ms */
	struct bfa_fcxp_s *fcxp;
	struct bfa_fcxp_wqe_s fcxp_wqe;
	uint8_t retry_cnt;	/*!< retry count */
	uint8_t rsvd[3];
};
typedef struct bfa_fcs_lport_fdmi_s bfa_fcs_lport_fdmi_t;

/**
 * @brief fabric ms data structure
 */
struct bfa_fcs_lport_ms_s {
	bfa_sm_t	sm;		/*!< state machine */
	struct bfa_timer_s timer;
	struct bfa_fcs_lport_s *port;	/*!< parent port */
	struct bfa_fcxp_s *fcxp;
	struct bfa_fcxp_wqe_s fcxp_wqe;
	struct bfa_fcs_lport_fdmi_s fdmi;	/*!< FDMI component of MS */
	uint8_t retry_cnt;	/*!< retry count */
	uint8_t rsvd[3];
};
typedef struct bfa_fcs_lport_ms_s bfa_fcs_lport_ms_t;

/**
 * @brief fabric topology data structure
 */
struct bfa_fcs_lport_fab_s {
	struct bfa_fcs_lport_ns_s ns;	/*!< NS component of port */
	struct bfa_fcs_lport_scn_s scn;	/*!< scn component of port */
	struct bfa_fcs_lport_ms_s ms;	/*!< MS component of port */
};
typedef struct bfa_fcs_lport_fab_s bfa_fcs_lport_fab_t;

/**
 * @brief loop topology data structure
 */

#define 	MAX_ALPA_COUNT 		127

struct bfa_fcs_lport_loop_s {
	uint8_t	 num_alpa;	/*!< Num of ALPA entries in the map */
	uint8_t	 alpabm_valid; /*!< alpa bitmap valid or not (1 or 0) */
	uint8_t	 alpa_pos_map[MAX_ALPA_COUNT];	/*!< ALPA Positional Map */
	struct bfa_fcs_lport_s *port;	/*!< parent port */
};
typedef struct bfa_fcs_lport_loop_s bfa_fcs_lport_loop_t;

/**
 * @brief n2n topology data structure
 */

struct bfa_fcs_lport_n2n_s {
	uint32_t	rsvd;
	uint16_t	reply_oxid;	/*!< ox_id from the req flogi to be */
					 /* used in flogi acc */
	wwn_t	   rem_port_wwn;	/*!< Attached port's wwn */
};
typedef struct bfa_fcs_lport_n2n_s bfa_fcs_lport_n2n_t;

/**
 * @brief port topology data structure
 */
union bfa_fcs_lport_topo_u {
	struct bfa_fcs_lport_fab_s pfab;
	struct bfa_fcs_lport_loop_s ploop;
	struct bfa_fcs_lport_n2n_s pn2n;
};
typedef union bfa_fcs_lport_topo_u bfa_fcs_lport_topo_t;

/**
 * @brief port data structure
 */
struct bfa_fcs_lport_s {
	struct bfa_q_s	 qe;	/*!< used by port/vport */
	bfa_sm_t	       sm;	/*!< state machine */
	struct bfa_fcs_fabric_s *fabric;	/*!< parent fabric */
	struct bfa_lport_cfg_s  port_cfg;	/*!< port configuration */
	struct bfa_timer_s link_timer;	/*!< timer for link offline */
	uint32_t	pid : 24;	/*!< FC address */
	uint8_t	 lp_tag;		/*!< lport tag */
	uint16_t	num_rports;	/*!< Num of r-ports */
	struct bfa_q_s	 rport_q;	/*!< queue of discovered r-ports */
	struct bfa_fcs_s *fcs;	/*!< FCS instance */
	union bfa_fcs_lport_topo_u port_topo;	/*!< fabric/loop/n2n details */
	struct bfad_port_s *bfad_port;	/*!< driver peer instance */
	struct bfa_fcs_vport_s *vport;	/*!< NULL for base ports */
	struct bfa_fcxp_s *fcxp;
	struct bfa_fcxp_wqe_s fcxp_wqe;
	struct bfa_lport_stats_s stats;
	struct bfa_wc_s	wc;	/*!< waiting counter for events */
};
typedef struct bfa_fcs_lport_s bfa_fcs_lport_t;
#define BFA_FCS_GET_HAL_FROM_PORT(port)  (port->fcs->bfa)
#define BFA_FCS_GET_NS_FROM_PORT(port)  (&port->port_topo.pfab.ns)
#define BFA_FCS_GET_SCN_FROM_PORT(port)  (&port->port_topo.pfab.scn)
#define BFA_FCS_GET_MS_FROM_PORT(port)  (&port->port_topo.pfab.ms)
#define BFA_FCS_GET_FDMI_FROM_PORT(port)  (&port->port_topo.pfab.ms.fdmi)
#define	BFA_FCS_VPORT_IS_INITIATOR_MODE(port) \
		(port->port_cfg.roles & BFA_LPORT_ROLE_FCP_IM)
#define	BFA_FCS_VPORT_IS_TARGET_MODE(port) \
		(port->port_cfg.roles & BFA_LPORT_ROLE_FCP_TM)
#define	BFA_FCS_VPORT_IS_IPFC_MODE(port) \
		(port->port_cfg.roles & BFA_LPORT_ROLE_FCP_IPFC)

/**
 * @brief authentication data structure
 */

struct bfa_fcs_auth_s {
	bfa_sm_t	sm;	/*!< state machine */
	bfa_boolean_t   policy;	/*!< authentication enabled/disabled */
	enum auth_status_s status;	/*!< authentication status */ 
	auth_rjt_codes_t  rjt_code;	/*!< auth reject status */
	auth_rjt_code_exps_t  rjt_code_exp;	/*!< auth reject reason */
	enum bfa_auth_algo algo;	/*!< Authentication algorithm */
	enum bfa_auth_algo neg_algo;    /*!< Negotiated hash algorithm */
	struct bfa_auth_stats_s stats;	/*!< Statistics */
	auth_dh_gid_t   group;	/*!< DH(diffie-hellman) Group */
	enum bfa_auth_secretsource source;	/*!< Secret source */
	char	    secret[BFA_AUTH_SECRET_STRING_LEN];
				/*!< secret string */
	uint8_t	 secret_len;	/*!< secret string length */
	uint8_t	 nretries;	/*!< number of retries */
	struct bfa_fcs_fabric_s *fabric;	/*!< pointer to fabric */
	uint8_t	 sentcode;	/*!< pointer to response data */
	uint8_t	*response;	/*!< pointer to response data */
	struct bfa_timer_s delay_timer; 	/*!< delay timer */
	struct bfa_fcxp_s *fcxp;		/*!< pointer to fcxp */
	struct bfa_fcxp_wqe_s fcxp_wqe;
};
typedef struct bfa_fcs_auth_s bfa_fcs_auth_t;

/*
 * forward declaration
 */
struct bfad_vf_s;
typedef struct bfad_vf_s bfad_vf_t;

enum bfa_fcs_fabric_type {
	BFA_FCS_FABRIC_UNKNOWN = 0,
	BFA_FCS_FABRIC_SWITCHED = 1,
	BFA_FCS_FABRIC_N2N = 2,
	BFA_FCS_FABRIC_LOOP = 3
};
typedef enum bfa_fcs_fabric_type bfa_fcs_fabric_type_t;

/**
 * @brief fabric data structure
 */
struct bfa_fcs_fabric_s {
	struct bfa_q_s   qe;		/*!< queue element */
	bfa_sm_t	 sm;		/*!< state machine */
	struct bfa_fcs_s *fcs;		/*!< FCS instance */
	struct bfa_fcs_lport_s  bport;	/*!< base logical port */
	enum bfa_fcs_fabric_type fab_type; /*!< fabric type */
	enum bfa_port_type oper_type;	/*!< current link topology */
	uint8_t	 is_vf;		/*!< is virtual fabric? */
	uint8_t	 is_npiv;	/*!< is NPIV supported ? */
	uint8_t	 is_auth;	/*!< is Security/Auth supported ? */
	uint16_t	bb_credit;	/*!< BB credit from fabric */
	uint16_t	vf_id;		/*!< virtual fabric ID */
	uint16_t	num_vports;	/*!< num vports */
	uint16_t	rsvd;
	struct bfa_q_s	 vport_q;	/*!< queue of virtual ports */
	struct bfa_q_s	 vf_q;		/*!< queue of virtual fabrics */
	struct bfad_vf_s      *vf_drv;	/*!< driver vf structure */
	struct bfa_timer_s link_timer;	/*!< Link Failure timer. Vport */
	wwn_t	   fabric_name;		/*!< attached fabric name */
	bfa_boolean_t   auth_reqd;	/*!< authentication required	*/
	struct bfa_timer_s delay_timer;	/*!< delay timer		*/
	union {
		uint16_t swp_vfid;	/*!< switch port VF id		*/
	} event_arg;
	struct bfa_fcs_auth_s  auth;	/*!< authentication config	*/
	struct bfa_wc_s	wc;	/*!< wait counter for delete	*/
	struct bfa_vf_stats_s  stats; 	/*!< fabric/vf stats		*/
	struct bfa_lps_s	*lps;	/*!< lport login services	*/
	uint8_t	fabric_ip_addr[BFA_FCS_FABRIC_IPADDR_SZ];
					/*!< attached fabric's ip addr  */
	struct bfa_wc_s	stop_wc;	/*!< wait counter for stop */
};
typedef struct bfa_fcs_fabric_s bfa_fcs_fabric_t;

#define bfa_fcs_fabric_npiv_capable(__f)    (__f)->is_npiv
#define bfa_fcs_fabric_is_switched(__f)			\
	((__f)->fab_type == BFA_FCS_FABRIC_SWITCHED)
#define bfa_fcs_fabric_is_direct_attached(__f)			\
	((__f)->fab_type == BFA_FCS_FABRIC_N2N)

/**
 * @brief
 *   The design calls for a single implementation of base fabric and vf.
 */
#define bfa_fcs_vf_t struct bfa_fcs_fabric_s

struct bfa_vf_event_s {
	uint32_t	undefined;
};
typedef struct bfa_vf_event_s bfa_vf_event_t;

struct bfa_fcs_s;
struct bfa_fcs_fabric_s;

/*
 * @todo : need to move to a global config file.
 * Maximum Rports supported per port (physical/logical).
 */
#define BFA_FCS_MAX_RPORTS_SUPP  256	/* @todo : tentative value */

#define bfa_fcs_lport_t struct bfa_fcs_lport_s

/**
 * Symbolic Name related defines
 *  Total bytes 255.
 *  Physical Port's symbolic name 128 bytes.
 *  For Vports, Vport's symbolic name is appended to the Physical port's
 *  Symbolic Name.
 *
 *  Physical Port's symbolic name Format : (Total 128 bytes)
 *  Adapter Model number/name : 16 bytes
 *  Driver Version     : 10 bytes
 *  Host Machine Name  : 30 bytes
 * 	Host OS Info	   : 44 bytes
 * 	Host OS PATCH Info : 16 bytes
 *  ( remaining 12 bytes reserved to be used for separator)
 */
#define BFA_FCS_PORT_SYMBNAME_SEPARATOR 		" | "

#define BFA_FCS_PORT_SYMBNAME_MODEL_SZ			16
#define BFA_FCS_PORT_SYMBNAME_VERSION_SZ 		10
#define BFA_FCS_PORT_SYMBNAME_MACHINENAME_SZ 	30
#define BFA_FCS_PORT_SYMBNAME_OSINFO_SZ			44
#define BFA_FCS_PORT_SYMBNAME_OSPATCH_SZ		16

/**
 * Get FC port ID for a logical port.
 */
#define bfa_fcs_lport_get_fcid(_lport)	((_lport)->pid)
#define bfa_fcs_lport_get_pwwn(_lport)	((_lport)->port_cfg.pwwn)
#define bfa_fcs_lport_get_nwwn(_lport)	((_lport)->port_cfg.nwwn)
#define bfa_fcs_lport_get_psym_name(_lport)	((_lport)->port_cfg.sym_name)
#define bfa_fcs_lport_get_nsym_name(_lport)	\
			((_lport)->port_cfg.node_sym_name)
#define bfa_fcs_lport_is_initiator(_lport)			\
	((_lport)->port_cfg.roles & BFA_LPORT_ROLE_FCP_IM)
#define bfa_fcs_lport_is_target(_lport)				\
	((_lport)->port_cfg.roles & BFA_LPORT_ROLE_FCP_TM)
#define bfa_fcs_lport_get_nrports(_lport)	\
	((_lport) ? (_lport)->num_rports : 0)

static inline struct bfad_port_s *
bfa_fcs_lport_get_drvport(struct bfa_fcs_lport_s *port)
{
	return port->bfad_port;
}

/**
 * @brief Return operating type for the local port.
 */
#define bfa_fcs_lport_get_opertype(_lport)	(_lport)->fabric->oper_type

/**
 * @brief Return attached fabric name for the local port.
 */
#define bfa_fcs_lport_get_fabric_name(_lport)	(_lport)->fabric->fabric_name

/**
 * @brief Return attached fabric's ip address for the local port.
 */
#define bfa_fcs_lport_get_fabric_ipaddr(_lport)	(_lport)->fabric->fabric_ip_addr

/**
 * bfa fcs port public functions
 */

bfa_boolean_t   bfa_fcs_lport_is_online(struct bfa_fcs_lport_s *port);
void bfa_fcs_cfg_base_port(struct bfa_fcs_s *fcs,
			   struct bfa_lport_cfg_s *port_cfg);
struct bfa_fcs_lport_s *bfa_fcs_get_base_port(struct bfa_fcs_s *fcs);
void bfa_fcs_lport_get_rports(struct bfa_fcs_lport_s *port,
			      wwn_t rport_wwns[], int *nrports);
void bfa_fcs_lport_get_rport_quals(struct bfa_fcs_lport_s *port,
	      bfa_rport_qualifier_t rport[], int *nrports);

wwn_t bfa_fcs_lport_get_rport(struct bfa_fcs_lport_s *port, wwn_t wwn,
			      int index, int nrports, bfa_boolean_t bwwn);

struct bfa_fcs_lport_s *bfa_fcs_lookup_port(struct bfa_fcs_s *fcs,
					    uint16_t vf_id, wwn_t lpwwn);

void bfa_fcs_lport_get_info(struct bfa_fcs_lport_s *port,
			    struct bfa_lport_info_s *port_info);
char *bfa_fcs_lport_get_symname(struct bfa_fcs_lport_s *port);
void bfa_fcs_lport_set_symname(struct bfa_fcs_lport_s *port, char *symname);
void bfa_fcs_lport_get_attr(struct bfa_fcs_lport_s *port,
			    struct bfa_lport_attr_s *port_attr);
void bfa_fcs_lport_get_stats(struct bfa_fcs_lport_s *fcs_port,
			     struct bfa_lport_stats_s *port_stats);
void bfa_fcs_lport_clear_stats(struct bfa_fcs_lport_s *fcs_port);
enum bfa_port_speed bfa_fcs_lport_get_rport_max_speed(
				struct bfa_fcs_lport_s *port);
void bfa_fcs_lport_enable_ipfc_roles(struct bfa_fcs_lport_s *fcs_port);
void bfa_fcs_lport_disable_ipfc_roles(struct bfa_fcs_lport_s *fcs_port);
void bfa_fcb_process_farp(void *fcs_port, void *buf);

/* MS FCS routines */
void bfa_fcs_lport_ms_init(struct bfa_fcs_lport_s *port);
void bfa_fcs_lport_ms_offline(struct bfa_fcs_lport_s *port);
void bfa_fcs_lport_ms_online(struct bfa_fcs_lport_s *port);
void bfa_fcs_lport_ms_fabric_rscn(struct bfa_fcs_lport_s *port);

/* FDMI FCS routines */
void bfa_fcs_lport_fdmi_init(struct bfa_fcs_lport_ms_s *ms);
void bfa_fcs_lport_fdmi_offline(struct bfa_fcs_lport_ms_s *ms);
void bfa_fcs_lport_fdmi_online(struct bfa_fcs_lport_ms_s *ms);
void	    bfa_fcs_lport_uf_recv(struct bfa_fcs_lport_s *lport, fchs_t *fchs,
				     uint16_t len);
void bfa_fcs_lport_attach(struct bfa_fcs_lport_s *lport, struct bfa_fcs_s *fcs,
			  uint16_t vf_id, struct bfa_fcs_vport_s *vport);
void bfa_fcs_lport_init(struct bfa_fcs_lport_s *lport,
				struct bfa_lport_cfg_s *port_cfg);
void	bfa_fcs_lport_online(struct bfa_fcs_lport_s *port);
void	bfa_fcs_lport_offline(struct bfa_fcs_lport_s *port);
void	bfa_fcs_lport_delete(struct bfa_fcs_lport_s *port);
void	bfa_fcs_lport_stop(struct bfa_fcs_lport_s *port);

struct bfa_fcs_rport_s *bfa_fcs_lport_get_rport_by_pid(
			struct bfa_fcs_lport_s *port, uint32_t pid);
struct bfa_fcs_rport_s *bfa_fcs_lport_get_rport_by_old_pid(
			struct bfa_fcs_lport_s *port, uint32_t pid);
struct bfa_fcs_rport_s *bfa_fcs_lport_get_rport_by_pwwn(
			struct bfa_fcs_lport_s *port, wwn_t pwwn);
struct bfa_fcs_rport_s *bfa_fcs_lport_get_rport_by_nwwn(
			struct bfa_fcs_lport_s *port, wwn_t nwwn);
struct bfa_fcs_rport_s *bfa_fcs_lport_get_rport_by_qualifier(
			struct bfa_fcs_lport_s *port, wwn_t pwwn, uint32_t pid);
void	    bfa_fcs_lport_add_rport(struct bfa_fcs_lport_s *port,
				       struct bfa_fcs_rport_s *rport);
void	    bfa_fcs_lport_del_rport(struct bfa_fcs_lport_s *port,
				       struct bfa_fcs_rport_s *rport);
void bfa_fcs_lport_modinit(struct bfa_fcs_s *fcs);
void bfa_fcs_lport_modexit(struct bfa_fcs_s *fcs);
void bfa_fcs_lport_lip(struct bfa_fcs_lport_s *port);
void	    bfa_fcs_lport_ns_init(struct bfa_fcs_lport_s *vport);
void	    bfa_fcs_lport_ns_offline(struct bfa_fcs_lport_s *vport);
void	    bfa_fcs_lport_ns_online(struct bfa_fcs_lport_s *vport);
void	    bfa_fcs_lport_ns_query(struct bfa_fcs_lport_s *port);
void	    bfa_fcs_lport_scn_init(struct bfa_fcs_lport_s *vport);
void	    bfa_fcs_lport_scn_offline(struct bfa_fcs_lport_s *vport);
void	    bfa_fcs_lport_fab_scn_online(struct bfa_fcs_lport_s *vport);
void	    bfa_fcs_lport_scn_process_rscn(struct bfa_fcs_lport_s *port,
					      fchs_t *rx_frame, uint32_t len);
uint32_t bfa_fcs_lport_get_target_count(struct bfa_fcs_lport_s *fcs_port);
void	bfa_fcs_lport_lip_scn_online(bfa_fcs_lport_t *port);
struct bfa_fcs_rport_s *bfa_fcs_lport_get_target_rport(bfa_fcs_lport_t *port,
		wwn_t wwn, int index, bfa_boolean_t bwwn);

struct bfa_fcs_vport_s {
	struct bfa_q_s		qe;		/*!< queue elem		*/
	bfa_sm_t		sm;		/*!< state machine	*/
	bfa_fcs_lport_t		lport;		/*!< logical port	*/
	struct bfa_timer_s	timer;
	struct bfad_vport_s	*vport_drv;	/*!< Driver private	*/
	struct bfa_vport_stats_s vport_stats;	/*!< vport statistics	*/
	struct bfa_lps_s	*lps;		/*!< Lport login service */
	int			fdisc_retries;
};
typedef struct bfa_fcs_vport_s bfa_fcs_vport_t;

#define bfa_fcs_vport_get_port(vport)			\
	((struct bfa_fcs_lport_s  *)(&vport->port))

/**
 * bfa fcs vport public functions
 */
bfa_status_t bfa_fcs_vport_create(struct bfa_fcs_vport_s *vport,
				  struct bfa_fcs_s *fcs, uint16_t vf_id,
				  struct bfa_lport_cfg_s *port_cfg,
				  struct bfad_vport_s *vport_drv);
bfa_status_t bfa_fcs_pbc_vport_create(struct bfa_fcs_vport_s *vport,
				      struct bfa_fcs_s *fcs, uint16_t vf_id,
				      struct bfa_lport_cfg_s *port_cfg,
				      struct bfad_vport_s *vport_drv);
bfa_boolean_t bfa_fcs_is_pbc_vport(struct bfa_fcs_vport_s *vport);
bfa_status_t bfa_fcs_vport_delete(struct bfa_fcs_vport_s *vport);
bfa_status_t bfa_fcs_vport_start(struct bfa_fcs_vport_s *vport);
bfa_status_t bfa_fcs_vport_stop(struct bfa_fcs_vport_s *vport);
void bfa_fcs_vport_get_attr(struct bfa_fcs_vport_s *vport,
			    struct bfa_vport_attr_s *vport_attr);
void bfa_fcs_vport_get_stats(struct bfa_fcs_vport_s *vport,
			     struct bfa_vport_stats_s *vport_stats);
void bfa_fcs_vport_clr_stats(struct bfa_fcs_vport_s *vport);
struct bfa_fcs_vport_s *bfa_fcs_vport_lookup(struct bfa_fcs_s *fcs,
					     uint16_t vf_id, wwn_t vpwwn);
void bfa_fcs_vport_cleanup(struct bfa_fcs_vport_s *vport);
void bfa_fcs_vport_online(struct bfa_fcs_vport_s *vport);
void bfa_fcs_vport_offline(struct bfa_fcs_vport_s *vport);
void bfa_fcs_vport_delete_comp(struct bfa_fcs_vport_s *vport);
void bfa_fcs_vport_fcs_delete(struct bfa_fcs_vport_s *vport);
void bfa_fcs_vport_fcs_stop(struct bfa_fcs_vport_s *vport);
void bfa_fcs_vport_stop_comp(struct bfa_fcs_vport_s *vport);

#define BFA_FCS_RPORT_DEF_DEL_TIMEOUT 	90 	/* in secs */
#define BFA_FCS_RPORT_MAX_RETRIES	(5)

#define BFA_FCS_RPORT_SCSI_FUNCTION(_rport) ((_rport)->scsi_function)
#define bfa_fcs_rport_get_fcid(_rport) ((_rport)->pid)
#define bfa_fcs_rport_get_pwwn(_rport) ((_rport)->pwwn)

/*
 * forward declarations
 */
struct bfad_rport_s;

struct bfa_fcs_itnim_s;
typedef struct bfa_fcs_itnim_s bfa_fcs_itnim_t;
struct bfa_fcs_tin_s;
typedef struct bfa_fcs_tin_s bfa_fcs_tin_t;
struct bfa_fcs_iprp_s;
typedef struct bfa_fcs_iprp_s bfa_fcs_iprp_t;

/* Rport Features (RPF) */
struct bfa_fcs_rpf_s {
	bfa_sm_t	sm;	/*!< state machine */
	struct bfa_fcs_rport_s *rport;	/*!< parent rport */
	struct bfa_timer_s 	timer;	/*!< general purpose timer */
	struct bfa_fcxp_s 	*fcxp;	/*!< FCXP needed for discarding */
	struct bfa_fcxp_wqe_s 	fcxp_wqe;	/*!< fcxp wait queue element */
	int	rpsc_retries;	/*!< max RPSC retry attempts */
	enum bfa_port_speed 	rpsc_speed;
	/*!< Current Speed from RPSC. O if RPSC fails */
	enum bfa_port_speed	assigned_speed;
	/**
	 * Speed assigned by the user.  will be used if RPSC is
	 * not supported by the rport.
	 */
};
typedef struct bfa_fcs_rpf_s bfa_fcs_rpf_t;

struct bfa_fcs_rport_s {
	struct bfa_q_s	qe;	/*!< used by port/vport */
	struct bfa_fcs_lport_s *port;	/*!< parent FCS port */
	struct bfa_fcs_s	*fcs;	/*!< fcs instance */
	struct bfad_rport_s	*rp_drv;	/*!< driver peer instance */
	uint32_t	pid;	/*!< port ID of rport */
	uint32_t	old_pid;	/*!< PID before rport goes offline */
	uint16_t	maxfrsize;	/*!< maximum frame size */
	uint16_t	reply_oxid;	/*!< OX_ID of inbound requests */
	fc_cos_t	fc_cos;	/*!< FC classes of service supp */
	bfa_boolean_t	cisc;	/*!< CISC capable device */
	bfa_boolean_t	prlo;	/*!< processing prlo or LOGO */
	bfa_boolean_t	plogi_pending;	/*!< Rx Plogi Pending */
	wwn_t	pwwn;	/*!< port wwn of rport */
	wwn_t	nwwn;	/*!< node wwn of rport */
	struct bfa_rport_symname_s psym_name; /*!< port symbolic name  */
	bfa_sm_t	sm;		/*!< state machine */
	struct bfa_timer_s timer;	/*!< general purpose timer */
	struct bfa_fcs_itnim_s *itnim;	/*!< ITN initiator mode role */
	struct bfa_fcs_tin_s *tin;	/*!< ITN initiator mode role */
	struct bfa_fcs_iprp_s *iprp;	/*!< IP/FC role */
	struct bfa_rport_s *bfa_rport;	/*!< BFA Rport */
	struct bfa_fcxp_s *fcxp;	/*!< FCXP needed for discarding */
	int	plogi_retries;	/*!< max plogi retry attempts */
	int	ns_retries;	/*!< max NS query retry attempts */
	struct bfa_fcxp_wqe_s 	fcxp_wqe; /*!< fcxp wait queue element */
	struct bfa_rport_stats_s stats;	/*!< rport stats */
	enum bfa_rport_function	scsi_function;  /*!< Initiator/Target */
	struct bfa_fcs_rpf_s rpf; 	/* Rport features module */
	bfa_boolean_t	scn_online;	/* SCN online flag */
};
typedef struct bfa_fcs_rport_s bfa_fcs_rport_t;

static inline struct bfa_rport_s *
bfa_fcs_rport_get_halrport(struct bfa_fcs_rport_s *rport)
{
	return rport->bfa_rport;
}

/**
 * bfa fcs rport API functions
 */
bfa_status_t bfa_fcs_rport_add(struct bfa_fcs_lport_s *port, wwn_t *pwwn,
			       struct bfa_fcs_rport_s *rport,
			       struct bfad_rport_s *rport_drv);
bfa_status_t bfa_fcs_rport_remove(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_get_attr(struct bfa_fcs_rport_s *rport,
			    struct bfa_rport_attr_s *attr);
void bfa_fcs_rport_get_stats(struct bfa_fcs_rport_s *rport,
			     struct bfa_rport_stats_s *stats);
void bfa_fcs_rport_clear_stats(struct bfa_fcs_rport_s *rport);
struct bfa_fcs_rport_s *bfa_fcs_rport_lookup(struct bfa_fcs_lport_s *port,
					     wwn_t rpwwn);
struct bfa_fcs_rport_s *bfa_fcs_rport_lookup_by_nwwn(
	struct bfa_fcs_lport_s *port, wwn_t rnwwn);
void bfa_fcs_rport_set_del_timeout(uint8_t rport_tmo);
void bfa_fcs_rport_set_max_logins(uint32_t max_logins);

void bfa_fcs_rport_set_speed(struct bfa_fcs_rport_s *rport,
			     enum bfa_port_speed speed);
void bfa_fcs_rport_uf_recv(struct bfa_fcs_rport_s *rport,
	 fchs_t *fchs, uint16_t len);
void bfa_fcs_rport_scn(struct bfa_fcs_rport_s *rport);

struct bfa_fcs_rport_s *bfa_fcs_rport_create(struct bfa_fcs_lport_s *port,
	 uint32_t pid);
void bfa_fcs_rport_delete(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_online(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_offline(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_start(struct bfa_fcs_lport_s *port, fchs_t *rx_fchs,
			 fc_plogi_t *plogi_rsp);
void bfa_fcs_rport_plogi_create(struct bfa_fcs_lport_s *port, fchs_t *rx_fchs,
				fc_plogi_t *plogi);
void bfa_fcs_rport_plogi(struct bfa_fcs_rport_s *rport, fchs_t *fchs,
			 fc_plogi_t *plogi);
void bfa_fcs_rport_logo_imp(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_prlo(struct bfa_fcs_rport_s *rport, uint16_t ox_id);

void bfa_fcs_rport_itnim_online(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_itnim_ack(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_tin_ack(struct bfa_fcs_rport_s *rport);
void bfa_fcs_rport_fcptm_offline_done(struct bfa_fcs_rport_s *rport);
int  bfa_fcs_rport_get_state(struct bfa_fcs_rport_s *rport);
bfa_fcs_rport_t *bfa_fcs_rport_create_by_wwn(bfa_fcs_lport_t *port, wwn_t wwn);
void  bfa_fcs_rpf_init(struct bfa_fcs_rport_s *rport);
void  bfa_fcs_rpf_rport_online(struct bfa_fcs_rport_s *rport);
void  bfa_fcs_rpf_rport_offline(struct bfa_fcs_rport_s *rport);
wwn_t bfa_fcs_rport_pwwn_get(void *rport);

/*
 * forward declarations
 */
struct bfad_itnim_s;

struct bfa_fcs_itnim_s {
	bfa_sm_t		sm;		/*!< state machine */
	struct bfa_fcs_rport_s 	*rport;		/*!< parent remote rport  */
	struct bfad_itnim_s	*itnim_drv;	/*!< driver peer instance */
	struct bfa_fcs_s	*fcs;		/*!< fcs instance	*/
	struct bfa_timer_s 	timer;		/*!< timer functions	*/
	struct bfa_itnim_s 	*bfa_itnim;	/*!< BFA itnim struct	*/
	uint32_t		prli_retries;	/*!< max prli retry attempts */
	bfa_boolean_t		seq_rec;	/*!< seq recovery support */
	bfa_boolean_t		rec_support;	/*!< REC supported	*/
	bfa_boolean_t		conf_comp;	/*!< FCP_CONF	support */
	bfa_boolean_t		task_retry_id;	/*!< task retry id supp	*/
	struct bfa_fcxp_wqe_s	fcxp_wqe;	/*!< wait qelem for fcxp  */
	struct bfa_fcxp_s	*fcxp;		/*!< FCXP in use	*/
	struct bfa_itnim_stats_s	stats;	/*!< itn statistics	*/
};
#define bfa_fcs_fcxp_alloc(__fcs, __req)	\
	bfa_fcxp_req_rsp_alloc(NULL, (__fcs)->bfa, 0, 0, \
			NULL, NULL, NULL, NULL, __req)

#define bfa_fcs_fcxp_alloc_wait(					\
		__bfa, __wqe, __alloc_cbfn, __alloc_cbarg, __req)	\
	bfa_fcxp_req_rsp_alloc_wait(__bfa, __wqe, __alloc_cbfn,		\
		__alloc_cbarg, NULL, 0, 0, NULL, NULL, NULL, NULL, __req)

#define BFA_FCS_NUM_RPORT_LOGINS(_fcs) ((_fcs)->num_rport_logins)

/**
 * @brief Return driver port instance for a itnim.
 */
static inline struct bfad_port_s *
bfa_fcs_itnim_get_drvport(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->rport->port->bfad_port;
}

/**
 * @brief Return fcs port instance for a itnim.
 */
static inline struct bfa_fcs_lport_s *
bfa_fcs_itnim_get_port(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->rport->port;
}

/**
 * @brief Return node wwn of itnim
 */
static inline wwn_t
bfa_fcs_itnim_get_nwwn(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->rport->nwwn;
}

/**
 * @brief Return port wwn of itnim
 */
static inline wwn_t
bfa_fcs_itnim_get_pwwn(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->rport->pwwn;
}

/**
 * @brief Return remote port FC address.
 */
static inline uint32_t
bfa_fcs_itnim_get_fcid(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->rport->pid;
}

/**
 * @brief Return maximum receive size of remote port.
 */
static inline	uint32_t
bfa_fcs_itnim_get_maxfrsize(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->rport->maxfrsize;
}

/**
 * @brief Return FC classes of service supported.
 */
static inline	fc_cos_t
bfa_fcs_itnim_get_cos(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->rport->fc_cos;
}

/**
 * @brief Return driver itnim instance.
 */
static inline struct bfad_itnim_s *
bfa_fcs_itnim_get_drvitn(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->itnim_drv;
}

/**
 * @brief Return hal itnim instance.
 */
static inline struct bfa_itnim_s *
bfa_fcs_itnim_get_halitn(struct bfa_fcs_itnim_s *itnim)
{
	return itnim->bfa_itnim;
}

/**
 * bfa fcs FCP Initiator mode API functions
 */
void bfa_fcs_itnim_get_attr(struct bfa_fcs_itnim_s *itnim,
			    struct bfa_itnim_attr_s *attr);
void bfa_fcs_itnim_get_stats(struct bfa_fcs_itnim_s *itnim,
			     struct bfa_itnim_stats_s *stats);
struct bfa_fcs_itnim_s *bfa_fcs_itnim_lookup(struct bfa_fcs_lport_s *port,
					     wwn_t rpwwn);
bfa_status_t bfa_fcs_itnim_attr_get(struct bfa_fcs_lport_s *port, wwn_t rpwwn,
				    struct bfa_itnim_attr_s *attr);
bfa_status_t bfa_fcs_itnim_stats_get(struct bfa_fcs_lport_s *port, wwn_t rpwwn,
				     struct bfa_itnim_stats_s *stats);
bfa_status_t bfa_fcs_itnim_stats_clear(struct bfa_fcs_lport_s *port,
				       wwn_t rpwwn);
struct bfa_fcs_itnim_s *bfa_fcs_itnim_create(struct bfa_fcs_rport_s *rport);
void bfa_fcs_itnim_delete(struct bfa_fcs_itnim_s *itnim);
void bfa_fcs_itnim_rport_offline(struct bfa_fcs_itnim_s *itnim);
void bfa_fcs_itnim_frp_online(struct bfa_fcs_itnim_s *itnim);
void bfa_fcs_itnim_brp_online(struct bfa_fcs_itnim_s *itnim);
bfa_status_t bfa_fcs_itnim_get_online_state(struct bfa_fcs_itnim_s *itnim);
void bfa_fcs_itnim_is_initiator(struct bfa_fcs_itnim_s *itnim);
void bfa_fcs_itnim_pause(struct bfa_fcs_itnim_s *itnim);
void bfa_fcs_itnim_resume(struct bfa_fcs_itnim_s *itnim);
void bfa_fcs_fcpim_uf_recv(struct bfa_fcs_itnim_s *itnim,
			fchs_t *fchs, uint16_t len);

#define	BFA_FCS_FDMI_SUPP_SPEEDS_4G		(FDMI_TRANS_SPEED_1G  |	\
						   FDMI_TRANS_SPEED_2G |\
						   FDMI_TRANS_SPEED_4G)

#define	BFA_FCS_FDMI_SUPP_SPEEDS_8G		(FDMI_TRANS_SPEED_1G  |	\
						   FDMI_TRANS_SPEED_2G |\
						   FDMI_TRANS_SPEED_4G |\
						   FDMI_TRANS_SPEED_8G)

#define	BFA_FCS_FDMI_SUPP_SPEEDS_16G		(FDMI_TRANS_SPEED_2G  |	\
						   FDMI_TRANS_SPEED_4G |\
						   FDMI_TRANS_SPEED_8G |\
						   FDMI_TRANS_SPEED_16G)

#define	BFA_FCS_FDMI_SUPP_SPEEDS_10G		FDMI_TRANS_SPEED_10G

#define BFA_FCS_FDMI_VENDOR_INFO_LEN	8
#define BFA_FCS_FDMI_FC4_TYPE_LEN		32

/*
 * HBA Attribute Block : BFA internal representation. Note : Some variable
 * sizes have been trimmed to suit BFA For Ex : Model will be "QLogic ". Based
 * on this the size has been reduced to 16 bytes from the standard's 64 bytes.
 */
struct bfa_fcs_fdmi_hba_attr_s {
	wwn_t		node_name;
	uint8_t		manufacturer[64];
	uint8_t		serial_num[64];
	uint8_t		model[16];
	uint8_t		model_desc[256];
	uint8_t		hw_version[8];
	uint8_t		driver_version[BFA_VERSION_LEN];
	uint8_t		option_rom_ver[BFA_VERSION_LEN];
	uint8_t		fw_version[BFA_VERSION_LEN];
	uint8_t		os_name[256];
	uint32_t	max_ct_pyld;
	struct 		bfa_lport_symname_s node_sym_name;
	uint8_t		vendor_info[BFA_FCS_FDMI_VENDOR_INFO_LEN];
	uint32_t	num_ports;
	wwn_t		fabric_name;
	uint8_t		bios_ver[BFA_VERSION_LEN];
};
typedef struct bfa_fcs_fdmi_hba_attr_s bfa_fcs_fdmi_hba_attr_t;

/*
 * Port Attribute Block
 */
struct bfa_fcs_fdmi_port_attr_s {
	uint8_t		supp_fc4_types[BFA_FCS_FDMI_FC4_TYPE_LEN];
	uint32_t	supp_speed;	/* supported speed */
	uint32_t	curr_speed;	/* current Speed */
	uint32_t	max_frm_size;	/* max frame size */
	uint8_t		os_device_name[256];	/* OS device Name */
	uint8_t		host_name[256];	/* host name */
	wwn_t		port_name;
	wwn_t		node_name;
	struct 		bfa_lport_symname_s port_sym_name;
	uint32_t	port_type;
	fc_cos_t	scos;
	wwn_t		port_fabric_name;
	uint8_t		port_act_fc4_type[BFA_FCS_FDMI_FC4_TYPE_LEN];
	uint32_t	port_state;
	uint32_t	num_ports;
};
typedef struct bfa_fcs_fdmi_port_attr_s bfa_fcs_fdmi_port_attr_t;

struct bfa_fcs_stats_s {
	struct {
		uint32_t	untagged; /*!< untagged receive frames */
		uint32_t	tagged;	/*!< tagged receive frames */
		uint32_t	vfid_unknown;	/*!< VF id is unknown */
	} uf;
};
typedef struct bfa_fcs_stats_s bfa_fcs_stats_t;

struct bfa_fcs_driver_info_s {
	uint8_t	 version[BFA_VERSION_LEN];		/* Driver Version */
	uint8_t	 host_machine_name[BFA_FCS_OS_STR_LEN];
	uint8_t	 host_os_name[BFA_FCS_OS_STR_LEN]; /* OS name and version */
	uint8_t	 host_os_patch[BFA_FCS_OS_STR_LEN]; /* patch or service pack */
	uint8_t	 os_device_name[BFA_FCS_OS_STR_LEN]; /* Driver Device Name */
};
typedef struct bfa_fcs_driver_info_s bfa_fcs_driver_info_t;

struct bfa_fcs_s {
	struct bfa_s	  *bfa;	/*!< corresponding BFA bfa instance */
	struct bfad_s	      *bfad; /*!< corresponding BDA driver instance */
	struct bfa_log_mod_s  *logm;	/*!< driver logging module instance */
	struct bfa_trc_mod_s  *trcmod;	/*!< tracing module */
	struct bfa_aen_s      *aen;	/*!< aen component */
	bfa_boolean_t	vf_enabled;	/*!< VF mode is enabled */
	bfa_boolean_t	fdmi_enabled;	/*!< FDMI is enabled */
	bfa_boolean_t	min_cfg;	/* min cfg enabled/disabled */
	uint16_t	port_vfid;	/*!< port default VF ID */
	struct bfa_fcs_driver_info_s driver_info;
	struct bfa_fcs_fabric_s fabric; /*!< base fabric state machine */
	struct bfa_fcs_stats_s	stats;	/*!< FCS statistics */
	struct bfa_wc_s		wc;	/*!< waiting counter */
	struct bfa_fcs_tm_ops_s *tm_ops; /*!< Target driver ops */
	uint32_t	num_rport_logins;
};
typedef struct bfa_fcs_s bfa_fcs_t;

/*
 * bfa fcs API functions
 */
void bfa_fcs_attach(struct bfa_fcs_s *fcs, struct bfa_s *bfa,
		    struct bfad_s *bfad,
		    bfa_boolean_t min_cfg);
void bfa_fcs_update_cfg(struct bfa_fcs_s *fcs);
void bfa_fcs_init(struct bfa_fcs_s *fcs);
void bfa_fcs_driver_info_init(struct bfa_fcs_s *fcs,
			      struct bfa_fcs_driver_info_s *driver_info);
void bfa_fcs_set_fdmi_param(struct bfa_fcs_s *fcs, bfa_boolean_t fdmi_enable);
void bfa_fcs_exit(struct bfa_fcs_s *fcs);
void bfa_fcs_trc_init(struct bfa_fcs_s *fcs, struct bfa_trc_mod_s *trcmod);
void bfa_fcs_log_init(struct bfa_fcs_s *fcs, struct bfa_log_mod_s *logmod);
void bfa_fcs_aen_init(struct bfa_fcs_s *fcs, struct bfa_aen_s *aen);
void bfa_fcs_start(bfa_fcs_t *fcs);
void bfa_fcs_stop(bfa_fcs_t *fcs);

/**
 * bfa fcs authentication public functions
 */
bfa_status_t    bfa_fcs_auth_get_attr(struct bfa_fcs_s *port,
				      struct bfa_auth_attr_s *attr);
bfa_status_t    bfa_fcs_auth_set_policy(struct bfa_fcs_s *port,
					bfa_boolean_t policy);
enum bfa_auth_status bfa_fcs_auth_get_status(struct bfa_fcs_s *port);
bfa_status_t    bfa_fcs_auth_set_algo(struct bfa_fcs_s *port,
				      enum bfa_auth_algo algo);
bfa_status_t    bfa_fcs_auth_get_stats(struct bfa_fcs_s *port,
				       struct bfa_auth_stats_s *stats);
bfa_status_t    bfa_fcs_auth_set_dh_group(struct bfa_fcs_s *port, int group);
bfa_status_t    bfa_fcs_auth_set_secretstring(struct bfa_fcs_s *port,
					      char *secret);
bfa_status_t    bfa_fcs_auth_set_secretstring_encrypt(struct bfa_fcs_s *port,
						      uint32_t secret[],
						      uint32_t len);
bfa_status_t    bfa_fcs_auth_set_secretsource(struct bfa_fcs_s *port,
					      enum bfa_auth_secretsource src);
bfa_status_t    bfa_fcs_auth_reset_stats(struct bfa_fcs_s *port);
bfa_status_t    bfa_fcs_auth_reinit(struct bfa_fcs_s *port);
void bfa_fcs_auth_uf_recv(struct bfa_fcs_fabric_s *fabric, int len);
void bfa_fcs_auth_start(struct bfa_fcs_fabric_s *fabric);
void bfa_fcs_auth_stop(struct bfa_fcs_fabric_s *fabric);

/**
 * bfa fcs vf public functions
 */
bfa_status_t bfa_fcs_vf_mode_enable(struct bfa_fcs_s *fcs, uint16_t vf_id);
bfa_status_t bfa_fcs_vf_mode_disable(struct bfa_fcs_s *fcs);
bfa_status_t bfa_fcs_vf_create(bfa_fcs_vf_t *vf, struct bfa_fcs_s *fcs,
			       uint16_t vf_id, struct bfa_lport_cfg_s *port_cfg,
			       struct bfad_vf_s *vf_drv);
bfa_status_t bfa_fcs_vf_delete(bfa_fcs_vf_t *vf);
void bfa_fcs_vf_list(struct bfa_fcs_s *fcs, uint16_t *vf_ids, int *nvfs);
void bfa_fcs_vf_list_all(struct bfa_fcs_s *fcs, uint16_t *vf_ids, int *nvfs);
void bfa_fcs_vf_get_attr(bfa_fcs_vf_t *vf, struct bfa_vf_attr_s *vf_attr);
void bfa_fcs_vf_get_stats(bfa_fcs_vf_t *vf,
			  struct bfa_vf_stats_s *vf_stats);
void bfa_fcs_vf_clear_stats(bfa_fcs_vf_t *vf);
void bfa_fcs_vf_get_ports(bfa_fcs_vf_t *vf, wwn_t vpwwn[], int *nports);
bfa_fcs_vf_t *bfa_fcs_vf_lookup(struct bfa_fcs_s *fcs, uint16_t vf_id);
uint16_t bfa_fcs_fabric_vport_count(struct bfa_fcs_fabric_s *fabric);

/*
 * fabric protected interface functions
 */
void bfa_fcs_fabric_attach(struct bfa_fcs_s *fcs);
void bfa_fcs_fabric_modinit(struct bfa_fcs_s *fcs);
void bfa_fcs_fabric_modexit(struct bfa_fcs_s *fcs);
void bfa_fcs_fabric_modsusp(struct bfa_fcs_s *fcs);
void bfa_fcs_fabric_link_up(struct bfa_fcs_fabric_s *fabric);
void bfa_fcs_fabric_link_down(struct bfa_fcs_fabric_s *fabric);
void bfa_fcs_fabric_addvport(struct bfa_fcs_fabric_s *fabric,
	struct bfa_fcs_vport_s *vport);
void bfa_fcs_fabric_delvport(struct bfa_fcs_fabric_s *fabric,
	struct bfa_fcs_vport_s *vport);
int bfa_fcs_fabric_is_online(struct bfa_fcs_fabric_s *fabric);
struct bfa_fcs_vport_s *bfa_fcs_fabric_vport_lookup(
			struct bfa_fcs_fabric_s *fabric, wwn_t pwwn);
void bfa_fcs_fabric_modstart(struct bfa_fcs_s *fcs);
void bfa_fcs_fabric_uf_recv(struct bfa_fcs_fabric_s *fabric, fchs_t *fchs,
	 uint16_t len);
bfa_boolean_t	bfa_fcs_fabric_is_loopback(struct bfa_fcs_fabric_s *fabric);
bfa_boolean_t	bfa_fcs_fabric_is_auth_failed(struct bfa_fcs_fabric_s *fabric);
enum bfa_port_type bfa_fcs_fabric_port_type(struct bfa_fcs_fabric_s *fabric);
void	bfa_fcs_fabric_psymb_init(struct bfa_fcs_fabric_s *fabric);
void	bfa_fcs_fabric_nsymb_init(struct bfa_fcs_fabric_s *fabric);
void	bfa_fcs_fabric_port_delete_comp(struct bfa_fcs_fabric_s *fabric);
bfa_status_t bfa_fcs_fabric_addvf(struct bfa_fcs_fabric_s *vf,
		struct bfa_fcs_s *fcs, struct bfa_lport_cfg_s *port_cfg,
		struct bfad_vf_s *vf_drv);
void bfa_fcs_auth_finished(struct bfa_fcs_fabric_s *fabric,
	auth_status_t status);
void bfa_fcs_fabric_set_fabric_name(struct bfa_fcs_fabric_s *fabric,
	       wwn_t fabric_name);
uint16_t bfa_fcs_fabric_get_switch_oui(struct bfa_fcs_fabric_s *fabric);
void bfa_fcs_uf_attach(struct bfa_fcs_s *fcs);
void bfa_fcs_port_attach(struct bfa_fcs_s *fcs);

void bfa_fcs_fabric_modstop(struct bfa_fcs_s *fcs);
void bfa_fcs_fabric_vport_stop_comp(struct bfa_fcs_fabric_s *fabric,
			struct bfa_fcs_vport_s *vport);
void bfa_fcs_fabric_port_stop_comp(struct bfa_fcs_fabric_s *fabric);

/**
 * BFA FCS callback interfaces
 */

/**
 * fcb Main fcs callbacks
 */

struct bfad_port_s;
struct bfad_vf_s;
struct bfad_vport_s;
struct bfad_rport_s;

void bfa_fcb_exit(void *drv);


/**
 * lport callbacks
 */
struct bfad_port_s *bfa_fcb_lport_new(struct bfad_s *bfad,
				      struct bfa_fcs_lport_s *port,
				      enum bfa_lport_role roles,
				      struct bfad_vf_s *vf_drv,
				      struct bfad_vport_s *vp_drv);
void bfa_fcb_lport_delete(struct bfad_s *bfad, enum bfa_lport_role roles,
			  struct bfad_vf_s *vf_drv,
			  struct bfad_vport_s *vp_drv);
void bfa_fcb_lport_online(struct bfad_s *bfad, enum bfa_lport_role roles,
			  struct bfad_vf_s *vf_drv,
			  struct bfad_vport_s *vp_drv);
void bfa_fcb_lport_offline(struct bfad_s *bfad, enum bfa_lport_role roles,
			   struct bfad_vf_s *vf_drv,
			   struct bfad_vport_s *vp_drv);
void bfa_fcb_lport_rscn_recv(struct bfad_s *bfad, struct bfad_vf_s *vf_drv,
				struct bfad_vport_s *vp_drv, uint32_t pid);
bfa_lport_role_t bfa_fcb_lport_get_role(struct bfad_s *bfad);

/**
 * vf callbacks
 */
void bfa_fcb_vf_stop(struct bfad_vf_s *vf_drv);

/**
 * vport callbacks
 */
void bfa_fcb_vport_delete(struct bfad_vport_s *vport_drv);
void bfa_fcb_pbc_vport_create(struct bfad_s *bfad, struct bfi_pbc_vport_s);
void bfa_fcb_vport_login(struct bfad_vport_s *vport_drv, bfa_status_t status);

/**
 * rport callbacks
 */
void bfa_fcb_rport_add(struct bfad_rport_s *rport_drv);
void bfa_fcb_rport_remove(struct bfad_rport_s *rport_drv);
bfa_status_t bfa_fcb_rport_alloc(struct bfad_s *bfad,
				 struct bfa_fcs_rport_s **rport,
				 struct bfad_rport_s **rport_drv);
void bfa_fcb_rport_free(struct bfad_s *bfad, struct bfad_rport_s **rport_drv);

void *bfa_fcb_fcs(struct bfad_s *bfad);

/**
 * itnim callbacks
 */
void bfa_fcb_itnim_alloc(struct bfad_s *bfad, struct bfa_fcs_itnim_s **itnim,
			 struct bfad_itnim_s **itnim_drv);
void bfa_fcb_itnim_free(struct bfad_s *bfad,
			struct bfad_itnim_s *itnim_drv);
void bfa_fcb_itnim_online(struct bfad_itnim_s *itnim_drv);
void bfa_fcb_itnim_offline(struct bfad_itnim_s *itnim_drv);
void bfa_fcb_itnim_tov_begin(struct bfad_itnim_s *itnim_drv);
void bfa_fcb_itnim_tov(struct bfad_itnim_s *itnim_drv);
void bfa_fcb_itnim_state_change(struct bfad_itnim_s *itnim_drv);
#endif /* __BFA_FCS_H__ */
