/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file pcifw.h PCI FW related headers
 */

#ifndef __PCIFW_H__
#define __PCIFW_H__

#pragma pack(1)

typedef struct {
	uint32_t signature; /* "$PnP" */
	uint8_t rev; /* Struct revision */
	uint8_t len; /* Header structure len in multiples of 16 bytes */
	uint16_t off; /* Offset to next header 00 if none */
	uint8_t rsvd; /* Reserved byte */
	uint8_t cksum; /* 8-bit checksum for this header */
	uint32_t pnp_dev_id; /* PnP Device Id */
	uint16_t mfstr; /* Pointer to manufacturer string */
	uint16_t prstr; /* Pointer to product string */
	uint8_t devtype[3]; /* Device Type Code */
	uint8_t devind; /* Device Indicator */
	uint16_t bcventr; /* Bootstrap entry vector */
	uint16_t rsvd2; /* Reserved */
	uint16_t sriv; /* Static resource information vector */
} pnp_hdr_t;

typedef struct {
	uint32_t sig; /* Signature "PCIR" */
	uint16_t vendid; /* Vendor ID */
	uint16_t devid; /* Device ID */
	uint16_t devlistoff; /* Device List Offset */
	uint16_t len; /* PCI Data Structure Length */
	uint8_t rev; /* PCI Data Structure Revision */
	uint8_t clcode[3]; /* Class Code */
	uint16_t imglen; /* Code image len in multiples of 512 bytes */
	uint16_t coderev; /* Revision level of code/data */
	uint8_t codetype; /* Code type 0x00 - BIOS */
	uint8_t indr; /* Last image indicator */
	uint16_t mrtimglen; /* Max Run Time Image Length */
	uint16_t cuoff; /* Config Utility Code Header Offset */
	uint16_t dmtfclp; /* DMTF CLP entry point offset */
} pci_3_0_ds_t;

typedef struct {
	uint16_t sig; /* Signature 0x55AA */
	uint8_t len; /* Option ROM length in units of 512 bytes */
	uint8_t inivec[3]; /* Initialization vector */
	uint8_t vendor_sig[4]; /* Vendor signature */
	uint16_t bofmds; /* Pointer to BOFM initiator data struct */
	uint16_t dateptr; /* pointer to date string - private */
	uint8_t rsvd[3]; /* Reserved field */
	uint16_t ovltable; /* Pointer to overlay table - private */
	uint8_t cksum;	/* Checksum of the whole image - private */
	uint16_t verlen; /* Length of version string - private */
	uint16_t verptr; /* Pointer to version string - private */
	uint16_t pcids; /* Pointer to PCI data structure */
	uint16_t pnphdr; /* Pointer to PnP expansion header */
} pci_optrom_hdr_t;

#pragma pack()

#endif
