/*
 * Copyright (c) 2005-2014 Brocade Communications Systems, Inc.
 * Copyright (c) 2014 QLogic Corporation.
 * All rights reserved
 * www.qlogic.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#include <asm/uaccess.h>
#include "bfad_ioctl_cna.h"
#include "bfad_ioctl.h"
#include "bfad_drv.h"
#include "bfad_aen.h"
#include "bfad_im.h"
#include "cna/bfa_diag.h"
#include "fcs/bfa_fcs.h"
#include <linux/module.h>

BFA_TRC_FILE(LDRV, IOCMD);

/* FC passthru call backs */
uint64_t
bfad_fcxp_get_req_sgaddr_cb(void *bfad_fcxp, int sgeid)
{
	struct bfad_fcxp    *drv_fcxp = bfad_fcxp;
	struct bfa_sge_s  *sge;
	uint64_t        addr;

	sge = drv_fcxp->req_sge + sgeid;
	addr = (uint64_t) (size_t) sge->sg_addr;

	return (addr);
}

uint32_t
bfad_fcxp_get_req_sglen_cb(void *bfad_fcxp, int sgeid)
{
	struct bfad_fcxp    *drv_fcxp = bfad_fcxp;
	struct bfa_sge_s  *sge;

	sge = drv_fcxp->req_sge + sgeid;
	return (sge->sg_len);
}

uint64_t
bfad_fcxp_get_rsp_sgaddr_cb(void *bfad_fcxp, int sgeid)
{
	struct bfad_fcxp    *drv_fcxp = bfad_fcxp;
	struct bfa_sge_s  *sge;
	uint64_t        addr;

	sge = drv_fcxp->rsp_sge + sgeid;
	addr = (uint64_t) (size_t) sge->sg_addr;
	return (addr);
}

uint32_t
bfad_fcxp_get_rsp_sglen_cb(void *bfad_fcxp, int sgeid)
{
	struct bfad_fcxp    *drv_fcxp = bfad_fcxp;
	struct bfa_sge_s  *sge;

	sge = drv_fcxp->rsp_sge + sgeid;
	return (sge->sg_len);
}

void
bfad_send_fcpt_cb(void *bfad_fcxp, struct bfa_fcxp_s *fcxp, void *cbarg,
		  bfa_status_t req_status, uint32_t rsp_len, uint32_t resid_len,
		  fchs_t *rsp_fchs)
{
	struct bfad_fcxp *drv_fcxp = bfad_fcxp;
	struct bfad_s   *bfad = cbarg;

	drv_fcxp->req_status = req_status;
	drv_fcxp->rsp_len = rsp_len;
	bfa_trc((struct bfad_s *) bfad, req_status);

	/*
	 * bfa_fcxp will be automatically freed by BFA on return from the
	 * callback function
	 */
	drv_fcxp->bfa_fcxp = NULL;

	/*
	 * Wake up the waiting thread
	 */
	complete(&drv_fcxp->comp);
}

static int
bfad_iocmd_itnim_trs_en_dis(struct bfad_s *bfad, void *cmd,
				bfa_boolean_t trs_disable_req)
{
	bfa_ioctl_itnim_attr_t *iocmd = (bfa_ioctl_itnim_attr_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port = NULL;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs,
			iocmd->vf_id, iocmd->lpwwn);
	if (!fcs_port) {
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		bfa_trc(bfad, 0);
	} else {
		if (trs_disable_req)
			iocmd->status = bfa_fcpim_trs_disable(
					&bfad->bfa,
					iocmd->lpwwn,
					iocmd->rpwwn);
		else
			iocmd->status = bfa_fcpim_trs_enable(
					&bfad->bfa,
					iocmd->lpwwn,
					iocmd->rpwwn);
	}

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	return 0;
}

static int
bfad_iocmd_itnim_trs_clear(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_trs_clear(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

static int
bfad_iocmd_itnim_trs_query(struct bfad_s *bfad, void *cmd,
				unsigned int payload_len)
{
	bfa_ioctl_itnim_trs_cfg_t *iocmd = (bfa_ioctl_itnim_trs_cfg_t *)cmd;
	bfa_trs_cfg_t *trs_cfg = NULL;
	bfa_trs_mask_t *trs_mask = NULL;
	bfa_trs_list_t *trs_list = NULL;
	unsigned long flags;
	bfa_q_t *qe, *qen;
	uint16_t cnt = 0;
	int size;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	trs_cfg = bfad_trs_get_cfg(bfad);
	if (trs_cfg->count == 0) {
		iocmd->trs_count = trs_cfg->count;
		iocmd->status = BFA_STATUS_OK;
		return 0;
	}

	size = trs_cfg->count * sizeof(bfa_trs_list_t);
	trs_list = (bfa_trs_list_t *)bfad_get_bufptr(iocmd, size,
			sizeof(bfa_ioctl_itnim_trs_cfg_t));
	if (!trs_list) {
		bfa_trc(bfad, size);
		return ENOMEM;
	}

	memset(trs_list, 0, size);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->trs_count = trs_cfg->count;
	bfa_q_iter_safe(&trs_cfg->trs_q, qe, qen) {
		trs_mask = (bfa_trs_mask_t *)qe;
		trs_list[cnt] = trs_mask->trs_list;
		++cnt;
	}

	iocmd->status = BFA_STATUS_OK;
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfad_copy_buf_to_user(iocmd->trs_list, trs_list, size);
	bfad_free_bufptr(trs_list);
	return 0;
}

static int
bfad_iocmd_itnim_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_itnim_attr_t *iocmd = (bfa_ioctl_itnim_attr_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port =
		bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->lpwwn);
	if (!fcs_port) {
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		bfa_trc(bfad, 0);
	} else {
		iocmd->status =
			bfa_fcs_itnim_attr_get(fcs_port, iocmd->rpwwn,
					       &iocmd->attr);
	}
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

static int
bfad_iocmd_itnim_get_iostats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_itnim_iostats_t *iocmd = (bfa_ioctl_itnim_iostats_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	struct bfa_fcs_itnim_s *itnim;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port =
		bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->lpwwn);
	if (!fcs_port) {
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		bfa_trc(bfad, 0);
	} else {
		itnim = bfa_fcs_itnim_lookup(fcs_port, iocmd->rpwwn);
		if (itnim == NULL)
			iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		else {
			iocmd->status = BFA_STATUS_OK;
			bfa_itnim_get_stats(bfa_fcs_itnim_get_halitn(itnim),
						&iocmd->iostats);
		}
	}
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

static int
bfad_iocmd_itnim_get_itnstats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_itnim_itnstats_t *iocmd = (bfa_ioctl_itnim_itnstats_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	struct bfa_fcs_itnim_s *itnim;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port =
		bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->lpwwn);
	if (!fcs_port) {
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		bfa_trc(bfad, 0);
	} else {
		itnim = bfa_fcs_itnim_lookup(fcs_port, iocmd->rpwwn);
		if (itnim == NULL)
			iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		else {
			iocmd->status = BFA_STATUS_OK;
			bfa_fcs_itnim_stats_get(fcs_port, iocmd->rpwwn,
				&iocmd->itnstats);
		}
	}
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

static int
bfad_iocmd_itnim_reset_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_itnim_reset_stats_t *iocmd = (bfa_ioctl_itnim_reset_stats_t *)cmd;
	struct bfa_fcs_lport_s                *fcs_port;
	struct bfa_fcs_itnim_s               *itnim;
	unsigned long                 flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port =
		bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->lpwwn);
	if (!fcs_port) {
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		bfa_trc(bfad, 0);
	} else {
		itnim = bfa_fcs_itnim_lookup(fcs_port, iocmd->rpwwn);
		if (itnim == NULL)
			iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		else {
			iocmd->status = BFA_STATUS_OK;
			bfa_fcs_itnim_stats_clear(fcs_port, iocmd->rpwwn);
			bfa_itnim_clear_stats(bfa_fcs_itnim_get_halitn(itnim));
		}
	}
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_ioc_enable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	int             rc = 0;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	/* If IOC is not in disabled state - return */
	if (!bfa_ioc_is_disabled(&bfad->bfa.ioc)) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_OK;
		return rc;
	}

	init_completion(&bfad->enable_comp);
	bfa_iocfc_enable(&bfad->bfa);
	iocmd->status = BFA_STATUS_OK;
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	wait_for_completion(&bfad->enable_comp);

	return rc;
}

int
bfad_iocmd_ioc_disable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	int             rc = 0;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (bfa_ioc_is_disabled(&bfad->bfa.ioc)) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_OK;
		return rc;
	}
	if (bfad->disable_active) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		return EBUSY;
	}

	bfad->disable_active = BFA_TRUE;
	init_completion(&bfad->disable_comp);
	bfa_iocfc_disable(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	wait_for_completion(&bfad->disable_comp);
	bfad->disable_active = BFA_FALSE;

	iocmd->status = BFA_STATUS_OK;

	return rc;
}

/*
 * TODO - one time init
 */
static void
bfad_set_driver_attr(struct bfad_s *bfad, struct bfa_ioc_driver_attr_s *driver_attr)
{
	strcpy(driver_attr->driver, BFAD_DRIVER_NAME);
	strncpy(driver_attr->driver_ver, bfa_version, BFA_VERSION_LEN);
}

int
bfad_iocmd_ioc_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_ioc_attr_t *iocmd = (bfa_ioctl_ioc_attr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	memset(&iocmd->ioc_attr, 0, sizeof(struct bfa_ioc_attr_s));
	iocmd->status = BFA_STATUS_OK;

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_get_attr(&bfad->bfa, &iocmd->ioc_attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfad_set_driver_attr(bfad, &iocmd->ioc_attr.driver_attr);

	/*
	 * TODO - replace this with version number from the firmware compiled as
	 * part of the driver
	 */
	strcpy(iocmd->ioc_attr.driver_attr.fw_ver,
	       iocmd->ioc_attr.adapter_attr.fw_ver);
	strcpy(iocmd->ioc_attr.driver_attr.bios_ver,
	       iocmd->ioc_attr.adapter_attr.optrom_ver);

	/* copy chip rev info first otherwise it will be overwritten */
	memcpy(bfad->pci_attr.chip_rev, iocmd->ioc_attr.pci_attr.chip_rev,
		sizeof(bfad->pci_attr.chip_rev));
	memcpy(&iocmd->ioc_attr.pci_attr, &bfad->pci_attr,
		sizeof(struct bfa_ioc_pci_attr_s));	

	/*
	 * TODO
	 * Set intr attr
	 */


	return 0;
}

/**
 * Fetch IOCFC attributes
 */
int
bfad_iocmd_iocfc_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_iocfc_attr_t	*iocmd = (bfa_ioctl_iocfc_attr_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	iocmd->status = BFA_STATUS_OK;
	bfa_iocfc_get_attr(&bfad->bfa, &iocmd->iocfc_attr);

	return 0;
}

/**
 * FAA query
 */
int
bfad_iocmd_faa_query(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_faa_attr_t	*iocmd =
			(bfa_ioctl_faa_attr_t *)cmd;
	struct bfad_hal_comp	fcomp;
	unsigned long		flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	iocmd->status = BFA_STATUS_OK;
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_faa_query(&bfad->bfa, &iocmd->ioctl_attr,
		bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK)
		return 0;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

	return 0;
}

int
bfad_iocmd_ioc_get_info(struct bfad_s *bfad, void *cmd)
{
	uint8_t	i;
	bfa_ioctl_ioc_info_t *iocmd = (bfa_ioctl_ioc_info_t *)cmd;
	struct bfad_im_port_s *im_port;
	struct bfa_port_attr_s pattr;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	memset(iocmd, 0, sizeof(bfa_ioctl_ioc_info_t));

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcport_get_attr(&bfad->bfa, &pattr);
	iocmd->nwwn = pattr.nwwn;
	iocmd->pwwn = pattr.pwwn;

	iocmd->ioc_type = bfa_get_type(&bfad->bfa);
	iocmd->mac = bfa_get_mac(&bfad->bfa);
	iocmd->factory_mac = bfa_get_mfg_mac(&bfad->bfa);
	bfa_get_adapter_serial_num(&bfad->bfa, iocmd->serialnum);
	iocmd->factorynwwn = pattr.factorynwwn;
	iocmd->factorypwwn = pattr.factorypwwn;
	iocmd->bfad_num = bfad->inst_no;

	im_port = bfad->pport.im_port;
	iocmd->host = im_port->shost->host_no;
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	strcpy(iocmd->name, bfad->adapter_name);
	strcpy(iocmd->port_name, bfad->port_name);
	strcpy(iocmd->hwpath, bfad->pci_name);
	/*
	 * set adapter hw path
	 */
	strcpy(iocmd->adapter_hwpath, bfad->pci_name);
	for (i = 0; iocmd->adapter_hwpath[i] != ':' && i < BFA_STRING_32; i++);
	for (; iocmd->adapter_hwpath[++i] != ':' && i < BFA_STRING_32; );
	iocmd->adapter_hwpath[i] = '\0';

	iocmd->status = BFA_STATUS_OK;

	return 0;
}
int
bfad_iocmd_ioc_get_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_ioc_stats_t *iocmd = (bfa_ioctl_ioc_stats_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	bfa_ioc_get_stats(&bfad->bfa, &iocmd->ioc_stats);
	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_ioc_reset_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	bfa_ioc_clear_stats(&bfad->bfa);
	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_ioc_get_fwstats(struct bfad_s *bfad, void *cmd,
			   unsigned int payload_len)
{
	bfa_ioctl_ioc_fwstats_t *iocmd = (bfa_ioctl_ioc_fwstats_t *)cmd;
	void	*iocmd_bufptr;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_ioc_fwstats_t),
			sizeof(struct bfa_fw_stats_s)) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		goto out;
	}

	bfa_trc(bfad, iocmd->buf_size);
	if (iocmd->buf_size != sizeof(struct bfa_fw_stats_s)) {
		bfa_trc(bfad, sizeof(struct bfa_fw_stats_s));
		iocmd->status = BFA_STATUS_EINVAL;
		goto out;
	}
	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->buf_size,
				sizeof(bfa_ioctl_ioc_fwstats_t));
	if (!iocmd_bufptr) {
		bfa_trc(bfad, 0x4444);
		return ENOMEM;
	}

	memset(iocmd_bufptr, 0, iocmd->buf_size);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =	bfa_fw_stats_get(&bfad->bfa, iocmd_bufptr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
		goto out_free_buf;
	}

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->buf_size);

out_free_buf:
	bfad_free_bufptr(iocmd_bufptr);
out:
	bfa_trc(bfad, 0x6666);
	return 0;
}

int
bfad_iocmd_ioc_reset_fwstats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fw_stats_clear(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfa_trc(bfad, iocmd->status);
	return 0;
}

int
bfad_iocmd_ioc_get_pcifn_cfg(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_pcifn_cfg_t *iocmd = (bfa_ioctl_pcifn_cfg_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_ablk_query(&bfad->bfa.modules.ablk,
				&iocmd->pcifn_cfg,
				bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:
	return 0;
}

int
bfad_iocmd_ioc_fw_sig_inv(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_ioc_fwsig_invalidate(&bfad->bfa.ioc);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	return 0;
}
int
bfad_iocmd_iocfc_set_intr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_iocfc_intr_t *iocmd = (bfa_ioctl_iocfc_intr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_iocfc_israttr_set(&bfad->bfa, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_ioc_set_adapter_name(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_ioc_set_adapter_name_t *iocmd = (bfa_ioctl_ioc_set_adapter_name_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	strcpy(bfad->adapter_name, iocmd->name);
	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_ioc_set_port_name(struct bfad_s *bfad, void *cmd)
{
	/*
	 * TBD: ioctl name need to changed
	 */
	bfa_ioctl_ioc_set_adapter_name_t *iocmd = (bfa_ioctl_ioc_set_adapter_name_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	strcpy(bfad->port_name, iocmd->name);
	iocmd->status = BFA_STATUS_OK;

	return 0;
}

/*
 * PORT
 */
int
bfad_iocmd_port_enable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_port_enable(&bfad->bfa.modules.port, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
		return 0;
	}

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	return 0;
}

int
bfad_iocmd_port_disable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_port_disable(&bfad->bfa.modules.port,
					 bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
		return 0;
	}

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	return 0;
}

int
bfad_iocmd_port_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_port_attr_t *iocmd = (bfa_ioctl_port_attr_t *)cmd;
	struct bfa_lport_attr_s port_attr;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcport_get_attr(&bfad->bfa, &iocmd->attr);
	bfa_fcs_lport_get_attr(&bfad->bfa_fcs.fabric.bport, &port_attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->attr.topology != BFA_PORT_TOPOLOGY_NONE)
		iocmd->attr.pid = port_attr.pid;
	else
		iocmd->attr.pid = 0;

	iocmd->attr.port_type = port_attr.port_type;
	iocmd->attr.loopback = port_attr.loopback;
	iocmd->attr.authfail = port_attr.authfail;
	strncpy(iocmd->attr.port_symname.symname, 
			port_attr.port_cfg.sym_name.symname,
			sizeof(port_attr.port_cfg.sym_name.symname));

	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_port_get_stats(struct bfad_s *bfad, void *cmd,
			  unsigned int payload_len)
{
	bfa_ioctl_port_stats_t *iocmd = (bfa_ioctl_port_stats_t *)cmd;
	struct bfad_hal_comp fcomp;
	void   *iocmd_bufptr;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_port_stats_t),
			sizeof(bfa_port_stats_t)) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	if (iocmd->buf_size != sizeof(bfa_port_stats_t)) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}
	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->buf_size,
			sizeof(bfa_ioctl_port_stats_t));
	if (!iocmd_bufptr)
		return ENOMEM;

	memset(iocmd_bufptr, 0, iocmd->buf_size);

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =	bfa_port_get_stats(&bfad->bfa.modules.port,
				iocmd_bufptr, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
		goto out;
	}

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->buf_size);
out:
	bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

int
bfad_iocmd_port_reset_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_port_clear_stats(&bfad->bfa.modules.port, bfad_hcb_comp,
					  &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
		return 0;
	}

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

	return 0;
}

int
bfad_iocmd_port_cfg_topology(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_port_cfg_topo_t *iocmd = (bfa_ioctl_port_cfg_topo_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcport_cfg_topology(&bfad->bfa, iocmd->topo);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_port_cfg_speed(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_port_cfg_speed_t *iocmd = (bfa_ioctl_port_cfg_speed_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcport_cfg_speed(&bfad->bfa, iocmd->speed);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_port_cfg_hardalpa(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_port_alpa_t *iocmd = (bfa_ioctl_port_alpa_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcport_cfg_hardalpa(&bfad->bfa, iocmd->alpa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_port_clr_hardalpa(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_port_alpa_t *iocmd = (bfa_ioctl_port_alpa_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcport_clr_hardalpa(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_port_cfg_maxfrsize(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_port_cfg_maxfrsize_t *iocmd = (bfa_ioctl_port_cfg_maxfrsize_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcport_cfg_maxfrsize(&bfad->bfa, iocmd->maxfrsize);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

/*
 * LPORT
 */
int
bfad_iocmd_lport_get_attr(struct bfad_s *bfad, void *cmd)
{
	struct bfa_fcs_lport_s *fcs_port;
	bfa_ioctl_lport_attr_t *iocmd = (bfa_ioctl_lport_attr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	bfa_fcs_lport_get_attr(fcs_port, &iocmd->port_attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_lport_reset_stats(struct bfad_s *bfad, void *cmd)
{
	struct bfa_fcs_lport_s *fcs_port;
	bfa_ioctl_lport_reset_stats_t *iocmd = (bfa_ioctl_lport_reset_stats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	bfa_fcs_lport_clear_stats(fcs_port);
	bfa_fcpim_port_clear_iostats(&bfad->bfa, fcs_port->lp_tag);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_lport_get_iostats(struct bfad_s *bfad, void *cmd)
{
	struct bfa_fcs_lport_s *fcs_port;
	bfa_ioctl_lport_iostats_t *iocmd = (bfa_ioctl_lport_iostats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}
	bfa_fcpim_port_iostats(&bfad->bfa, &iocmd->iostats,
		fcs_port->lp_tag);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_lport_get_stats(struct bfad_s *bfad, void *cmd)
{
	struct bfa_fcs_lport_s *fcs_port;
	bfa_ioctl_lport_stats_t *iocmd = (bfa_ioctl_lport_stats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	bfa_fcs_lport_get_stats(fcs_port, &iocmd->port_stats);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_lport_get_rports(struct bfad_s *bfad, void *cmd,
			    unsigned int payload_len)
{
	bfa_ioctl_lport_get_rports_t *iocmd = (bfa_ioctl_lport_get_rports_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	unsigned long   flags;
	void	*iocmd_bufptr;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (iocmd->nrports == 0) {
		bfa_trc(bfad, 0);
		return EINVAL;
	}

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_lport_get_rports_t),
			sizeof(bfa_rport_qualifier_t) *
			iocmd->nrports) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd,
			sizeof(bfa_rport_qualifier_t) * iocmd->nrports,
			sizeof(bfa_ioctl_lport_get_rports_t));
	if (iocmd_bufptr == NULL) {
		bfa_trc(bfad, 0);
		iocmd->status = BFA_STATUS_ENOMEM;
		goto out;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(
			&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		bfa_trc(bfad, 0);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out_free_buf;
	}

	memset(iocmd_bufptr, 0, sizeof(bfa_rport_qualifier_t) * iocmd->nrports);
	bfa_fcs_lport_get_rport_quals(fcs_port,
		(bfa_rport_qualifier_t *)iocmd_bufptr, &iocmd->nrports);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfad_copy_buf_to_user(iocmd->rbuf_ptr, iocmd_bufptr,
			iocmd->nrports * sizeof(bfa_rport_qualifier_t));

	iocmd->status = BFA_STATUS_OK;

out_free_buf:
	bfad_free_bufptr(iocmd_bufptr);
out:
	return 0;
}

/*
 * RPORT
 */
int
bfad_iocmd_rport_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_rport_attr_t *iocmd = (bfa_ioctl_rport_attr_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	struct bfa_fcs_rport_s *fcs_rport;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	if (iocmd->pid)
		fcs_rport = bfa_fcs_lport_get_rport_by_qualifier(fcs_port,
					iocmd->rpwwn, iocmd->pid);
	else
		fcs_rport = bfa_fcs_rport_lookup(fcs_port, iocmd->rpwwn);
	if (fcs_rport == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		goto out;
	}

	bfa_fcs_rport_get_attr(fcs_rport, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}


int
bfad_iocmd_rport_get_addr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_rport_scsi_addr_t *iocmd = (bfa_ioctl_rport_scsi_addr_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	struct bfa_fcs_itnim_s *fcs_itnim;
	struct bfad_itnim_s *drv_itnim;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	fcs_itnim = bfa_fcs_itnim_lookup(fcs_port, iocmd->rpwwn);
	if (fcs_itnim == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		goto out;
	}

	drv_itnim = fcs_itnim->itnim_drv;

	if (drv_itnim && drv_itnim->im_port) {
		if (bfad_os_get_host_no(drv_itnim, &iocmd->host)
		!= BFA_STATUS_OK) {
			spin_unlock_irqrestore(&bfad->bfad_lock, flags);
			iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
			goto out;
		}
	} else {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		goto out;
	}

	iocmd->target = drv_itnim->scsi_tgt_id;
		
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->bus = 0;
	iocmd->lun = 0;
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_rport_get_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_rport_stats_t *iocmd = (bfa_ioctl_rport_stats_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	struct bfa_fcs_rport_s *fcs_rport;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	fcs_rport = bfa_fcs_rport_lookup(fcs_port, iocmd->rpwwn);
	if (fcs_rport == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		goto out;
	}

	bfa_fcs_rport_get_stats(fcs_rport, &iocmd->stats);
	bfa_rport_get_stats(bfa_fcs_rport_get_halrport(fcs_rport),
				&iocmd->stats.hal_stats);

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_rport_clr_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_rport_reset_stats_t *iocmd = (bfa_ioctl_rport_reset_stats_t *)cmd;
	struct bfa_fcs_lport_s *fcs_port;
	struct bfa_fcs_rport_s *fcs_rport;
	unsigned long   flags;


	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	fcs_rport = bfa_fcs_rport_lookup(fcs_port, iocmd->rpwwn);
	if (fcs_rport == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		goto out;
	}

	bfa_fcs_rport_clear_stats(fcs_rport);
	bfa_rport_clear_stats(bfa_fcs_rport_get_halrport(fcs_rport));

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_rport_set_speed(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_rport_set_speed_t *iocmd = (bfa_ioctl_rport_set_speed_t *)cmd;
	bfa_fcs_lport_t *fcs_port;
	bfa_fcs_rport_t *fcs_rport;
	unsigned long	flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_port = bfa_fcs_lookup_port(&bfad->bfa_fcs, iocmd->vf_id, iocmd->pwwn);
	if (fcs_port == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_LWWN;
		goto out;
	}

	fcs_rport = bfa_fcs_rport_lookup(fcs_port, iocmd->rpwwn);
	if (fcs_rport == NULL) {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_RWWN;
		goto out;
	}
	bfa_fcs_rport_set_speed(fcs_rport,iocmd->speed);

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

/* VPORT */
int
bfad_iocmd_vport_start(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vport_t *iocmd = (bfa_ioctl_vport_t *)cmd;
	struct bfa_fcs_vport_s *fcs_vport;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vport =
		bfa_fcs_vport_lookup(&bfad->bfa_fcs, iocmd->vf_id, iocmd->vpwwn);
	if (fcs_vport == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VWWN;
		goto out;
	}

	iocmd->status = bfa_fcs_vport_start(fcs_vport);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

out:
	return 0;
}

int
bfad_iocmd_vport_stop(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vport_t *iocmd = (bfa_ioctl_vport_t *)cmd;
	struct bfa_fcs_vport_s *fcs_vport;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vport =
		bfa_fcs_vport_lookup(&bfad->bfa_fcs, iocmd->vf_id, iocmd->vpwwn);
	if (fcs_vport == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VWWN;
		goto out;
	}

	iocmd->status = bfa_fcs_vport_stop(fcs_vport);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

out:
	return 0;
}

int
bfad_iocmd_vport_get_attr(struct bfad_s *bfad, void *cmd)
{
	struct bfa_fcs_vport_s *fcs_vport;
	bfa_ioctl_vport_attr_t *iocmd = (bfa_ioctl_vport_attr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vport =
		bfa_fcs_vport_lookup(&bfad->bfa_fcs, iocmd->vf_id, iocmd->vpwwn);
	if (fcs_vport == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VWWN;
		goto out;
	}

	bfa_fcs_vport_get_attr(fcs_vport, &iocmd->vport_attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_vport_get_stats(struct bfad_s *bfad, void *cmd)
{
	struct bfa_fcs_vport_s *fcs_vport;
	bfa_ioctl_vport_stats_t *iocmd = (bfa_ioctl_vport_stats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vport =
		bfa_fcs_vport_lookup(&bfad->bfa_fcs, iocmd->vf_id, iocmd->vpwwn);
	if (fcs_vport == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VWWN;
		goto out;
	}

	bfa_fcs_vport_get_stats(fcs_vport, &iocmd->vport_stats);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

int
bfad_iocmd_vport_clr_stats(struct bfad_s *bfad, void *cmd)
{
	struct bfa_fcs_vport_s *fcs_vport;
	bfa_ioctl_vport_reset_stats_t *iocmd = (bfa_ioctl_vport_reset_stats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vport =
		bfa_fcs_vport_lookup(&bfad->bfa_fcs, iocmd->vf_id, iocmd->vpwwn);
	if (fcs_vport == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VWWN;
		goto out;
	}

	bfa_fcs_vport_clr_stats(fcs_vport);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:
	return 0;
}

/*
 * VF
 */
int
bfad_iocmd_vf_mode_enable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vf_t  *iocmd = (bfa_ioctl_vf_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	/*
	 * TBD: need to update API to add vf_id
	 */
	/*
	 * iocmd->status = bfa_fcs_vf_mode_enable(&bfad->bfa_fcs, iocmd->vf_id);
	 */
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_vf_mode_enable(&bfad->bfa_fcs, iocmd->vf_id);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);


	return 0;
}

int
bfad_iocmd_vf_mode_disable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vf_t  *iocmd = (bfa_ioctl_vf_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_vf_mode_disable(&bfad->bfa_fcs);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);


	return 0;
}

int
bfad_iocmd_vf_add(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vf_add_t *iocmd = (bfa_ioctl_vf_add_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	iocmd->status = bfad_vf_create(bfad, iocmd->vf_id, &iocmd->port_cfg);

	if (iocmd->status != BFA_STATUS_OK) {
		BFA_DEV_PRINTF(bfad, BFA_ERR, "Failed to create vf %d\n",
			       iocmd->vf_id);
	}


	return 0;
}

int
bfad_iocmd_vf_del(struct bfad_s *bfad, void *cmd)
{
	bfa_fcs_vf_t   *fcs_vf;
	bfa_ioctl_vf_t  *iocmd = (bfa_ioctl_vf_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vf = bfa_fcs_vf_lookup(&bfad->bfa_fcs, (uint16_t) iocmd->vf_id);

	if (fcs_vf == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VFID;
		goto out;
	}

	iocmd->status = bfa_fcs_vf_delete(fcs_vf);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		BFA_DEV_PRINTF(bfad, BFA_ERR, "Failed to delete vf %d\n",
			       iocmd->vf_id);
	}

out:

	return 0;

}

int
bfad_iocmd_vf_list(struct bfad_s *bfad, unsigned int cmd, void *pcmd,
		   unsigned int payload_len)
{
	bfa_ioctl_vf_list_t *iocmd = (bfa_ioctl_vf_list_t *)pcmd;
	int             nvfs;
	unsigned long   flags;
	void	*iocmd_bufptr;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_vf_list_t),
			sizeof(uint16_t) * iocmd->nvfs) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, sizeof(uint16_t) * iocmd->nvfs,
				sizeof(bfa_ioctl_vf_list_t));
	if (iocmd_bufptr == NULL)
		return ENOMEM;

	memset(iocmd_bufptr, 0, sizeof(uint16_t) * iocmd->nvfs);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (cmd == IOCMD_VF_LIST)
		bfa_fcs_vf_list(&bfad->bfa_fcs,
				(uint16_t *)iocmd_bufptr, &nvfs);
	else
		bfa_fcs_vf_list_all(&bfad->bfa_fcs,
				(uint16_t *)iocmd_bufptr, &nvfs);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->nvfs = nvfs;

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, nvfs * sizeof(int));

	bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

int
bfad_iocmd_vf_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vf_attr_t *iocmd = (bfa_ioctl_vf_attr_t *)cmd;
	bfa_fcs_vf_t   *fcs_vf;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	/*
	 * TBD: need to have this BFA API
	 */
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vf = bfa_fcs_vf_lookup(&bfad->bfa_fcs, iocmd->vf_id);
	if (fcs_vf == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VFID;
		goto out;
	}

	bfa_fcs_vf_get_attr(fcs_vf, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:

	return 0;
}

int
bfad_iocmd_vf_get_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vf_stats_t *iocmd = (bfa_ioctl_vf_stats_t *)cmd;
	bfa_fcs_vf_t   *fcs_vf;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vf = bfa_fcs_vf_lookup(&bfad->bfa_fcs, iocmd->vf_id);
	if (fcs_vf == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VFID;
		goto out;
	}

	bfa_fcs_vf_get_stats(fcs_vf, &iocmd->stats);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:

	return 0;
}

int
bfad_iocmd_vf_clr_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vf_reset_stats_t *iocmd = (bfa_ioctl_vf_reset_stats_t *)cmd;
	bfa_fcs_vf_t   *fcs_vf;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vf = bfa_fcs_vf_lookup(&bfad->bfa_fcs, iocmd->vf_id);
	if (fcs_vf == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VFID;
		goto out;
	}

	bfa_fcs_vf_clear_stats(fcs_vf);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	iocmd->status = BFA_STATUS_OK;

out:

	return 0;
}

/*
 * FABRIC
 */
int
bfad_iocmd_fabric_get_lports(struct bfad_s *bfad, void *cmd,
			     unsigned int payload_len)
{
	bfa_ioctl_fabric_get_lports_t *iocmd = (bfa_ioctl_fabric_get_lports_t *)cmd;
	bfa_fcs_vf_t   *fcs_vf;
	uint32_t        nports;
	unsigned long   flags;
	void	*iocmd_bufptr;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->nports);
	bfa_trc(bfad, iocmd->vf_id);

	if (iocmd->nports == 0) {
		iocmd->status = BFA_STATUS_EINVAL;
		goto out;
	}
	nports = iocmd->nports;

	if (bfad_chk_iocmd_sz(payload_len,
		sizeof(bfa_ioctl_fabric_get_lports_t),
		sizeof(wwn_t[iocmd->nports])) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, sizeof(wwn_t[iocmd->nports]),
				sizeof(bfa_ioctl_fabric_get_lports_t));
	if (iocmd_bufptr == NULL) {
		bfa_trc(bfad, 0);
		return ENOMEM;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	fcs_vf = bfa_fcs_vf_lookup(&bfad->bfa_fcs, iocmd->vf_id);
	if (fcs_vf == NULL) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_UNKNOWN_VFID;
		goto out_free_buf;
	}

	memset(iocmd_bufptr, 0, sizeof(wwn_t[iocmd->nports]));
	bfa_fcs_vf_get_ports(fcs_vf, (wwn_t *)iocmd_bufptr, &nports);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->nports = nports;
	bfa_trc(bfad, iocmd->nports);

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr,
			nports * sizeof(wwn_t));

	iocmd->status = BFA_STATUS_OK;

out_free_buf:
	bfad_free_bufptr(iocmd_bufptr);
out:
	return 0;
}

/*
 * FLASH
 */
int
bfad_iocmd_flash_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_flash_attr_t *iocmd = (bfa_ioctl_flash_attr_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_flash_get_attr(BFA_FLASH(&bfad->bfa), &iocmd->attr,
			bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_flash_read_part(struct bfad_s *bfad, void *cmd,
			   unsigned int payload_len)
{
	bfa_ioctl_flash_t *iocmd = (bfa_ioctl_flash_t *)cmd;
	struct bfad_hal_comp fcomp;
	void	*iocmd_bufptr;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_flash_t),
			 iocmd->bufsz) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->bufsz,
				sizeof(bfa_ioctl_flash_t));
	if (!iocmd_bufptr) {
		bfa_trc(bfad, 0x4444);
		return ENOMEM;
	}

	memset(iocmd_bufptr, 0, iocmd->bufsz);
	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_flash_read_part(BFA_FLASH(&bfad->bfa), iocmd->type,
			iocmd->instance, iocmd_bufptr, iocmd->bufsz, 0,
			bfad_hcb_comp, &fcomp, 1);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->bufsz);
out:
	bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

int
bfad_iocmd_flash_erase_part(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_flash_t *iocmd = (bfa_ioctl_flash_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_flash_erase_part(BFA_FLASH(&bfad->bfa), iocmd->type,
			iocmd->instance, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_flash_update_part(struct bfad_s *bfad, void *cmd,
			     unsigned int payload_len)
{
	bfa_ioctl_flash_t *iocmd = (bfa_ioctl_flash_t *)cmd;
	void	*iocmd_bufptr;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_flash_t),
			iocmd->bufsz) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->bufsz,
				sizeof(bfa_ioctl_flash_t));
	if (iocmd_bufptr == NULL)
		return ENOMEM;

	bfad_copy_buf_from_user(iocmd_bufptr, iocmd->buf_ptr, iocmd->bufsz);

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_flash_update_part(BFA_FLASH(&bfad->bfa), iocmd->type,
			iocmd->instance, iocmd_bufptr, iocmd->bufsz,
			0, bfad_hcb_comp, &fcomp, 1);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:
	bfad_free_bufptr(iocmd_bufptr);

	return 0;
}

int
bfad_iocmd_flash_get_cfg_bootver(struct bfad_s *bfad, void *cmd,
				unsigned int payload_len)
{
	bfa_ioctl_flash_t *iocmd = (bfa_ioctl_flash_t *)cmd;
	struct bfad_hal_comp fcomp;
	void *iocmd_bufptr;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_flash_t),
			iocmd->bufsz) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->bufsz,
				sizeof(bfa_ioctl_flash_t));
	if (!iocmd_bufptr) {
		bfa_trc(bfad, 0x4444);
		return ENOMEM;
	}

	memset(iocmd_bufptr, 0, iocmd->bufsz);
	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_flash_get_boot_version(
				BFA_FLASH(&bfad->bfa),
				iocmd_bufptr,
				iocmd->bufsz,
				bfad_hcb_comp,
				&fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->bufsz);
out:
	bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

int
bfad_iocmd_ablk_optrom(struct bfad_s *bfad, unsigned int cmd, void *pcmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)pcmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (cmd == IOCMD_FLASH_ENABLE_OPTROM) 
		iocmd->status =
			bfa_ablk_optrom_en(&bfad->bfa.modules.ablk,
				bfad_hcb_comp, &fcomp);
	else
		iocmd->status =
			bfa_ablk_optrom_dis(&bfad->bfa.modules.ablk,
				bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_phy_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_phy_attr_t *iocmd = (bfa_ioctl_phy_attr_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_phy_get_attr(BFA_PHY(&bfad->bfa), iocmd->instance,
			&iocmd->attr, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_phy_get_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_phy_stats_t *iocmd = (bfa_ioctl_phy_stats_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_phy_get_stats(BFA_PHY(&bfad->bfa), iocmd->instance,
			&iocmd->stats, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_phy_read(struct bfad_s *bfad, void *cmd, unsigned int payload_len)
{
	bfa_ioctl_phy_t *iocmd = (bfa_ioctl_phy_t *)cmd;
	struct bfad_hal_comp fcomp;
	void	*iocmd_bufptr;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_phy_t),
			iocmd->bufsz) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->bufsz,
				sizeof(bfa_ioctl_phy_t));
	if (!iocmd_bufptr) {
		bfa_trc(bfad, 0x4444);
		return ENOMEM;
	}

	memset(iocmd_bufptr, 0, iocmd->bufsz);
	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_phy_read(BFA_PHY(&bfad->bfa),
			iocmd->instance, iocmd_bufptr, iocmd->bufsz, 0,
			bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->bufsz);
out:
	bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

int
bfad_iocmd_phy_update(struct bfad_s *bfad, void *cmd, unsigned int payload_len)
{
	bfa_ioctl_phy_t *iocmd = (bfa_ioctl_phy_t *)cmd;
	void	*iocmd_bufptr;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_phy_t),
			iocmd->bufsz) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->bufsz,
				sizeof(bfa_ioctl_phy_t));
	if (iocmd_bufptr == NULL)
		return ENOMEM;

	bfad_copy_buf_from_user(iocmd_bufptr, iocmd->buf_ptr, iocmd->bufsz);

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_phy_update(BFA_PHY(&bfad->bfa),
			iocmd->instance, iocmd_bufptr, iocmd->bufsz,
			0, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:
	bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

/*
 * DIAG
 */
int
bfad_iocmd_diag_memrd(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_kva_t *iocmd = (bfa_ioctl_diag_kva_t *)cmd;
	uint32_t to_copy;
        uint64_t *ptr;
#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 31)
	void	*iocmd_bufptr;
#endif

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

        ptr = (uint64_t*)bfad;

	iocmd->status = BFA_STATUS_OK;
	iocmd->kva_address = (uint64_t)(size_t)ptr;
	to_copy = ((sizeof(struct bfad_s)) <= (iocmd->uLength * sizeof(uint32_t))) ?
                        (sizeof(struct bfad_s)) : (iocmd->uLength * sizeof(uint32_t));

	if (to_copy) {
#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 31)
		/* The IOCMD bufptr is - iocmd ptr offset'ed by size of IOCMD type */
		iocmd_bufptr = (char *)iocmd + sizeof(bfa_ioctl_diag_kva_t);
		memcpy(iocmd_bufptr, (uint8_t *)(size_t)(iocmd->uAddress), to_copy);
#else
		bfad_copy_buf_to_user(iocmd->buf_ptr,
				(uint8_t *)(size_t)(iocmd->uAddress), to_copy);
#endif
	}

	return 0;
}

int
bfad_iocmd_diag_temp(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_get_temp_t *iocmd = (bfa_ioctl_diag_get_temp_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_diag_tsensor_query(BFA_DIAG_MOD(&bfad->bfa), &iocmd->result,
					   bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:

	return 0;
}

int
bfad_iocmd_diag_post(struct bfad_s *bfad, void *iocmd)
{
	return 0;
}

int
bfad_iocmd_diag_memtest(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_memtest_t *iocmd = (bfa_ioctl_diag_memtest_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_diag_memtest(BFA_DIAG_MOD(&bfad->bfa), &iocmd->memtest, iocmd->pat,
				     &iocmd->result, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:

	return 0;
}

int
bfad_iocmd_diag_loopback(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_loopback_t *iocmd = (bfa_ioctl_diag_loopback_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long	flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcdiag_loopback(&bfad->bfa, iocmd->opmode, iocmd->speed,
				      iocmd->lpcnt, iocmd->pat, &iocmd->result,
				      bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:

	return 0;
}

int
bfad_iocmd_diag_fwping(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_fwping_t *iocmd = (bfa_ioctl_diag_fwping_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, 0x7777);
	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_diag_fwping(BFA_DIAG_MOD(&bfad->bfa), iocmd->cnt, iocmd->pattern,
				    &iocmd->result, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	bfa_trc(bfad, 0x77771);
	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:

	bfa_trc(bfad, 0x77772);
	return 0;
}

int
bfad_iocmd_diag_queuetest(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_qtest_t *iocmd = (bfa_ioctl_diag_qtest_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, 0x8888);
	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcdiag_queuetest(&bfad->bfa, iocmd->force, iocmd->queue,
				    &iocmd->result, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	bfa_trc(bfad, 0x88881);
	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:

	bfa_trc(bfad, 0x88882);
	return 0;
}

int
bfad_iocmd_diag_pom(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_pom_t *iocmd = (bfa_ioctl_diag_pom_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	/*
	 * TBD: iocmd->status = bfa_diag_pom_query(&bfad->bfa,
	 * &iocmd->pom_attr);
	 */


	return 0;
}

int
bfad_iocmd_diag_sfp(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_sfp_show_t *iocmd = (bfa_ioctl_sfp_show_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_sfp_show(BFA_SFP_MOD(&bfad->bfa), &iocmd->sfp,
				 bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:

	return 0;
}

/**
 * SFP media
 */
int
bfad_iocmd_sfp_media(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_sfp_media_t *iocmd = (bfa_ioctl_sfp_media_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_sfp_media(BFA_SFP_MOD(&bfad->bfa), &iocmd->media,
			      bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_SFP_NOT_READY)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:

	return 0;
}

/**
 * SFP speed
 */
int
bfad_iocmd_sfp_speed(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_sfp_speed_t *iocmd = (bfa_ioctl_sfp_speed_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_sfp_speed(BFA_SFP_MOD(&bfad->bfa), iocmd->speed,
			      bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_SFP_NOT_READY)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

 out:

	return 0;
}

int
bfad_iocmd_diag_led(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_led_t *iocmd = (bfa_ioctl_diag_led_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_diag_ledtest(BFA_DIAG_MOD(&bfad->bfa), &iocmd->ledtest);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);


	return 0;
}

int
bfad_iocmd_diag_beacon_lport(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_beacon_t *iocmd = (bfa_ioctl_diag_beacon_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_diag_beacon_port(BFA_DIAG_MOD(&bfad->bfa), 
						iocmd->beacon, 
						iocmd->link_e2e_beacon,
							 iocmd->second);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);


	return 0;
}

int
bfad_iocmd_diag_lb_stat(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_diag_lb_stat_t *iocmd = (bfa_ioctl_diag_lb_stat_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, 0x4440);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcdiag_lb_is_running(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);


	return 0;
}

int
bfad_iocmd_diag_dport_enable(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_dport_enable_t *iocmd = (bfa_ioctl_dport_enable_t *)pcmd;
	unsigned long   flags;
	struct bfad_hal_comp fcomp;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_dport_enable(&bfad->bfa, iocmd->lpcnt,
					iocmd->pat, bfad_hcb_comp,
					&fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
	} else {
		wait_for_completion(&fcomp.comp);
		iocmd->status = fcomp.status;
	}

	return 0;
}

int
bfad_iocmd_diag_dport_disable(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)pcmd;
	unsigned long   flags;
	struct bfad_hal_comp fcomp;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_dport_disable(&bfad->bfa, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
	} else {
		wait_for_completion(&fcomp.comp);
		iocmd->status = fcomp.status;
	}

	return 0;
}

int
bfad_iocmd_diag_dport_start(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_dport_enable_t *iocmd = (bfa_ioctl_dport_enable_t *)pcmd;
	unsigned long   flags;
	struct bfad_hal_comp fcomp;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_dport_start(&bfad->bfa, iocmd->lpcnt,
					iocmd->pat, bfad_hcb_comp,
					&fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
	} else {
		wait_for_completion(&fcomp.comp);
		iocmd->status = fcomp.status;
	}

	return 0;
}

int
bfad_iocmd_diag_dport_show(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_diag_dport_show_t *iocmd =
				(bfa_ioctl_diag_dport_show_t *)pcmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_dport_show(&bfad->bfa, &iocmd->result);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

/*
 * CEE
 */
int
bfad_iocmd_cee_attr (struct bfad_s *bfad, void *cmd, unsigned int payload_len)
{
	bfa_ioctl_cee_attr_t *iocmd = (bfa_ioctl_cee_attr_t *)cmd;
	void *iocmd_bufptr;
	struct bfad_hal_comp cee_comp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_cee_attr_t),
			sizeof(bfa_cee_attr_t)) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	if (iocmd->buf_size != sizeof(bfa_cee_attr_t)) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}
	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->buf_size,
				sizeof(bfa_ioctl_cee_attr_t));
	if (!iocmd_bufptr)
		return ENOMEM;	

	memset(iocmd_bufptr, 0, iocmd->buf_size);
	cee_comp.status = 0;
	init_completion(&cee_comp.comp);
	down(&bfad_sem);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_cee_get_attr(&bfad->bfa.modules.cee, iocmd_bufptr,
	    bfad_hcb_comp, &cee_comp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK) {
		up(&bfad_sem);
		bfa_trc(bfad, 0x5555);
		goto out_free_buffer;
	}
	wait_for_completion(&cee_comp.comp);
	up(&bfad_sem);

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->buf_size);

out_free_buffer:
	if (iocmd_bufptr)
		bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

int
bfad_iocmd_cee_get_stats (struct bfad_s *bfad, void *cmd,
			  unsigned int payload_len)
{
	bfa_ioctl_cee_stats_t *iocmd = (bfa_ioctl_cee_stats_t *)cmd;
	void *iocmd_bufptr;
	struct bfad_hal_comp cee_comp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_cee_stats_t),
			sizeof(bfa_cee_stats_t)) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	if (iocmd->buf_size != sizeof(bfa_cee_stats_t)) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}
	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->buf_size,
				sizeof(bfa_ioctl_cee_stats_t));
	if (!iocmd_bufptr)
		return ENOMEM;	

	memset(iocmd_bufptr, 0, iocmd->buf_size);
	cee_comp.status = 0;
	init_completion(&cee_comp.comp);
	down(&bfad_sem);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_cee_get_stats(&bfad->bfa.modules.cee, iocmd_bufptr,
	    bfad_hcb_comp, &cee_comp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK) {
		up(&bfad_sem);
		bfa_trc(bfad, 0x5555);
		goto out_free_buffer;
	}
	wait_for_completion(&cee_comp.comp);
	up(&bfad_sem);

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->buf_size);

out_free_buffer:
	if (iocmd_bufptr)
		bfad_free_bufptr(iocmd_bufptr);
	return 0;
}

int
bfad_iocmd_cee_reset_stats (struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_cee_reset_stats(&bfad->bfa.modules.cee, NULL, NULL);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, 0x5555);
	}
	return 0;
}

/*
 * DEBUG
 */
int
bfad_iocmd_debug_drv_state(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_debug_t *iocmd = (void *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	/*
	 * TBD: we are not support for now
	 */
	iocmd->status = BFA_STATUS_FAILED;

	return 0;
}

int
bfad_iocmd_debug_fw_core(struct bfad_s *bfad, void *cmd,
			 unsigned int payload_len)
{
	bfa_ioctl_debug_t *iocmd = (bfa_ioctl_debug_t *)cmd;
	void	*iocmd_bufptr;
	unsigned long	flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (bfad_chk_iocmd_sz(payload_len, sizeof(bfa_ioctl_debug_t),
			BFA_DEBUG_FW_CORE_CHUNK_SZ) != BFA_STATUS_OK) {
		iocmd->status = BFA_STATUS_VERSION_FAIL;
		return 0;
	}

	bfa_trc(bfad, iocmd->bufsz);
	if (iocmd->bufsz < BFA_DEBUG_FW_CORE_CHUNK_SZ ||
			!is_aligned(iocmd->bufsz, 2) ||
			!is_aligned(iocmd->offset, 4)) {

		bfa_trc(bfad, BFA_DEBUG_FW_CORE_CHUNK_SZ);
		iocmd->status = BFA_STATUS_EINVAL;
		goto out;
	}

	iocmd_bufptr = bfad_get_bufptr(iocmd, iocmd->bufsz,
				sizeof(bfa_ioctl_debug_t));
	if (!iocmd_bufptr) {
		bfa_trc(bfad, 0x4444);
		return ENOMEM;
	}

	memset(iocmd_bufptr, 0, iocmd->bufsz);
	spin_lock_irqsave(&bfad->bfad_lock, flags);

	iocmd->status = bfa_debug_fwcore(&bfad->bfa, iocmd_bufptr,
				(uint32_t *)&iocmd->offset, &iocmd->bufsz);

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, 0x5555);
		goto out_free_buf;
	}

	bfa_trc(bfad, iocmd->bufsz);

	bfad_copy_buf_to_user(iocmd->buf_ptr, iocmd_bufptr, iocmd->bufsz);

out_free_buf:
	bfad_free_bufptr(iocmd_bufptr);
out:
	bfa_trc(bfad, 0x6666);

	return 0;
}

int
bfad_iocmd_debug_fw_trace_clear(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_debug_fwsave_clear(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_porglog_get(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_debug_t *iocmd = (bfa_ioctl_debug_t *)cmd;
#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 31)
	void *iocmd_bufptr;
#endif

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	/*
	 * TBD: Verify the size of the buffer
	 */
	bfa_trc(bfad, iocmd->bufsz);

	if (iocmd->bufsz < sizeof(struct bfa_plog_s)) {
		bfa_trc(bfad, sizeof(struct bfa_plog_s));
		iocmd->status = BFA_STATUS_EINVAL;
		goto out;
	}

	iocmd->status = BFA_STATUS_OK;

#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 31)
	/* The IOCMD bufptr is - iocmd ptr offset'ed by size of IOCMD type */
	iocmd_bufptr = (char *)iocmd + sizeof(bfa_ioctl_debug_t);
	memcpy(iocmd_bufptr, (uint8_t *) &bfad->plog_buf,
		sizeof(struct bfa_plog_s));
#else
	bfad_copy_buf_to_user(iocmd->buf_ptr, &bfad->plog_buf,
			sizeof(struct bfa_plog_s));
#endif
out:
	return 0;
}

int
bfad_iocmd_porglog_clear(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_plog_clear(&bfad->plog_buf);
	iocmd->status = BFA_STATUS_OK;


	return 0;
}

int
bfad_iocmd_porglog_ctl(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_portlogctl_t *iocmd = (bfa_ioctl_portlogctl_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if (iocmd->ctl == BFA_TRUE)
		bfa_plog_enable(&bfad->plog_buf);
	else
		bfa_plog_disable(&bfad->plog_buf);

	iocmd->status = BFA_STATUS_OK;


	return 0;
}

int
bfad_iocmd_start_dtrc(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc_init(bfad->trcmod);
	iocmd->status = BFA_STATUS_OK;


	return 0;
}

int
bfad_iocmd_stop_dtrc(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc_stop(bfad->trcmod);
	iocmd->status = BFA_STATUS_OK;


	return 0;
}

int
bfad_iocmd_log_set_level(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_log_t *iocmd = (bfa_ioctl_log_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->log_level);
	if (iocmd->mod_id == BFA_LOG_MODULE_ID_ALL) {
		iocmd->status = bfa_log_set_level_all(&bfad->log_data, 
						     iocmd->log_level);
	} else if (iocmd->mod_id == BFA_LOG_AEN_ALL) {
		iocmd->status = bfa_log_set_level_aen(&bfad->log_data, 
						     iocmd->log_level);
	} else {
		if (iocmd->mod_id == BFA_LOG_DRV_ALL) 
			iocmd->mod_id = BFA_LOG_LINUX_ID;
		iocmd->status = bfa_log_set_level(&bfad->log_data, 
					iocmd->mod_id, iocmd->log_level);
	}


	return 0;
}

int
bfad_iocmd_log_get_level(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_log_t *iocmd = (bfa_ioctl_log_t *)cmd;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	if ((iocmd->mod_id <= BFA_LOG_UNUSED_ID) || 
	    (iocmd->mod_id > BFA_LOG_MODULE_ID_MAX)) 
		iocmd->mod_id = BFA_LOG_LINUX_ID;

	iocmd->log_level = bfa_log_get_level(&bfad->log_data, iocmd->mod_id);

	iocmd->status = BFA_STATUS_OK;
	bfa_trc(bfad, iocmd->log_level);


	return 0;
}

int
bfad_iocmd_qos(struct bfad_s *bfad, unsigned int cmd, void *pcmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)pcmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (cmd == IOCMD_QOS_ENABLE)
		bfa_fcport_cfg_qos(&bfad->bfa, BFA_TRUE);
	else if (cmd == IOCMD_QOS_DISABLE)
		bfa_fcport_cfg_qos(&bfad->bfa, BFA_FALSE);
	else {
		bfa_trc(bfad, 0);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		return EINVAL;
	}
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;


	return 0;
}

int
bfad_iocmd_qos_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_qos_attr_t *iocmd = (bfa_ioctl_qos_attr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcport_qos_get_attr(&bfad->bfa, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_qos_get_vc_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_qos_vc_attr_t *iocmd = (bfa_ioctl_qos_vc_attr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcport_qos_get_vc_attr(&bfad->bfa, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_qos_get_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_qos_stats_t *iocmd = (bfa_ioctl_qos_stats_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;
	struct bfa_cb_pending_q_s cb_qe;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	bfa_pending_q_init(&cb_qe, (bfa_cb_cbfn_t)bfad_hcb_comp,
				&fcomp, &iocmd->stats);

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcport_get_qos_stats(&bfad->bfa, &cb_qe);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
		goto out;
	}

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:
	return 0;
}

int
bfad_iocmd_qos_reset_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;
	struct bfa_cb_pending_q_s cb_qe;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	bfa_pending_q_init(&cb_qe, (bfa_cb_cbfn_t)bfad_hcb_comp,
				&fcomp, NULL);

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcport_clear_qos_stats(&bfad->bfa, &cb_qe);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK) {
		bfa_trc(bfad, iocmd->status);
		goto out;
	}

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_qos_set_bw(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_qos_bw_t	*iocmd = (bfa_ioctl_qos_bw_t *)pcmd;
	unsigned long		flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcport_set_qos_bw(&bfad->bfa, &iocmd->qos_bw);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_ratelim(struct bfad_s *bfad, unsigned int cmd, void *pcmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)pcmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (cmd == IOCMD_RATELIM_ENABLE)
		bfa_fcport_cfg_ratelim(&bfad->bfa, BFA_TRUE);
	else if (cmd == IOCMD_RATELIM_DISABLE)
		bfa_fcport_cfg_ratelim(&bfad->bfa, BFA_FALSE);
	else {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		bfa_trc(bfad, 0);
		return EINVAL;
	}
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;


	return 0;
}

int
bfad_iocmd_ratelim_speed(struct bfad_s *bfad, unsigned int cmd, void *pcmd)
{
	bfa_ioctl_trl_speed_t *iocmd = (bfa_ioctl_trl_speed_t *)pcmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	
	iocmd->status = bfa_fcport_cfg_ratelim_speed(&bfad->bfa, iocmd->speed);	

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);


	return 0;
}

int
bfad_iocmd_fcpim_failover(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_t *iocmd = (bfa_ioctl_fcpim_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcpim_path_tov_set(&bfad->bfa, iocmd->path_tov);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;


	return 0;
}

int
bfad_iocmd_fcpim_get_del_itn_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_del_itn_stats_t	*iocmd = (bfa_ioctl_fcpim_del_itn_stats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcpim_get_del_itn_stats(&bfad->bfa, &iocmd->modstats);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_fcpim_get_modstats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_modstats_t	*iocmd = (bfa_ioctl_fcpim_modstats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcpim_get_modstats(&bfad->bfa, &iocmd->modstats);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;

	return 0;
}


int
bfad_iocmd_fcpim_clr_modstats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_modstatsclr_t	*iocmd = (bfa_ioctl_fcpim_modstatsclr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcpim_clr_modstats(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;

	return 0;
}

int
bfad_iocmd_fcpim_qdepth(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_qdepth_t	*iocmd = (bfa_ioctl_fcpim_qdepth_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->q_depth);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	bfa_fcpim_qdepth_set(&bfad->bfa, iocmd->q_depth);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = BFA_STATUS_OK;


	return 0;
}

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 31))
/* Function to reset the LUN SCAN mode */
static void
bfad_iocmd_lunmask_reset_lunscan_mode(struct bfad_s *bfad, int lunmask_cfg)
{
	struct bfad_im_port_s *pport_im = bfad->pport.im_port;
	struct bfad_vport_s *vport = NULL;

	/* Set the scsi device LUN SCAN flags for base port */
	bfad_reset_sdev_bflags(pport_im, lunmask_cfg);

	/* Set the scsi device LUN SCAN flags for the vports */
	list_for_each_entry(vport, &bfad->vport_list, list_entry)
		bfad_reset_sdev_bflags(vport->drv_port.im_port, lunmask_cfg);
}
#endif

int
bfad_iocmd_lunmask_enable(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)pcmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_lunmask_update(&bfad->bfa, BFA_TRUE);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 31))
	/*
	 * Invoke bfad_iocmd_lunmask_reset_lunscan_mode() to set the
	 * LUN Scanning mode to be Sequential scan.
	 */
	if (iocmd->status == BFA_STATUS_OK)
		bfad_iocmd_lunmask_reset_lunscan_mode(bfad, BFA_TRUE);
#endif
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_lunmask_disable(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)pcmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_lunmask_update(&bfad->bfa, BFA_FALSE);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 31))
	/*
	 * Invoke bfad_iocmd_lunmask_reset_lunscan_mode() to set the
	 * LUN Scanning mode to be default REPORT_LUNS based scan.
	 */
	if (iocmd->status == BFA_STATUS_OK)
		bfad_iocmd_lunmask_reset_lunscan_mode(bfad, BFA_FALSE);
#endif
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcpim_iopf_start(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_iopf_t  *iocmd = (bfa_ioctl_fcpim_iopf_t *)cmd;
	bfa_timeval_t tv;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_os_gettimeofday(&tv);
	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_iopf_start(&bfad->bfa, iocmd->vf_id,
		iocmd->pwwn, iocmd->rpwwn, iocmd->flow_count, iocmd->lun,
		iocmd->lun_status, tv.tv_sec);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;

}

int
bfad_iocmd_fcpim_iopf_stop(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_iopf_t  *iocmd = (bfa_ioctl_fcpim_iopf_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	/* lun end, all_range is not valid for iopf stop */
	iocmd->status = bfa_fcpim_iopf_stop(&bfad->bfa, iocmd->vf_id,
		iocmd->pwwn, iocmd->rpwwn, iocmd->flow_count, iocmd->lun);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;


}

int
bfad_iocmd_fcpim_iopf_query(struct bfad_s *bfad, void *cmd)
{
	int size;
	unsigned long   flags;
	bfa_ioctl_fcpim_iopf_query_t *iocmd =
		(bfa_ioctl_fcpim_iopf_query_t *)cmd;
	struct bfa_fcpim_iopf_cfg_s *lun_st = NULL;;

	
	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }
	size = sizeof(struct bfa_fcpim_iopf_cfg_s) * BFA_IOPF_MAX;
	bfa_trc(bfad, iocmd->bfad_num);

	lun_st = (struct bfa_fcpim_iopf_cfg_s *)bfad_get_bufptr(iocmd, size,
		 sizeof(bfa_ioctl_fcpim_iopf_query_t));
	bfad_copy_buf_from_user(lun_st, iocmd->iopf_cfg, size);

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_iopf_query(&bfad->bfa, &iocmd->flow_count,
			lun_st);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	bfad_copy_buf_to_user(iocmd->iopf_cfg, lun_st, size);
	bfad_free_bufptr(lun_st);

	return 0;
}

int
bfad_iocmd_fcpim_iopf_show(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_iopf_show_t *iocmd =
		(bfa_ioctl_fcpim_iopf_show_t *)cmd;
	struct bfa_fcpim_iopf_s *iopf = &iocmd->iopf;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);

	spin_lock_irqsave(&bfad->bfad_lock, flags);

	iocmd->status = bfa_fcpim_iopf_show(&bfad->bfa, iocmd->vf_id,
		 iocmd->pwwn, iocmd->rpwwn, iocmd->lun, iocmd->itn_lun, iopf);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcpim_iopf_reset(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_iopf_t *iocmd =
		(bfa_ioctl_fcpim_iopf_t *)cmd;
	unsigned long   flags;
	bfa_timeval_t tv;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	bfa_os_gettimeofday(&tv);

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_iopf_reset(&bfad->bfa, iocmd->vf_id,
		iocmd->pwwn, iocmd->rpwwn, tv.tv_sec, iocmd->flow_count,
		iocmd->lun);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcpim_lunmask_query(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_lunmask_query_t *iocmd =
		(bfa_ioctl_fcpim_lunmask_query_t *)cmd;
	struct bfa_lunmask_cfg_s *lun_mask = &iocmd->lun_mask;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_lunmask_query(&bfad->bfa, lun_mask);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcpim_lunmask_add(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_lunmask_t  *iocmd = (bfa_ioctl_fcpim_lunmask_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_lunmask_add(&bfad->bfa, iocmd->vf_id,
		&iocmd->pwwn, iocmd->rpwwn, iocmd->lun);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcpim_lunmask_clear(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t  *iocmd =
		(bfa_ioctl_gen_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_lunmask_clear(&bfad->bfa);

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcpim_lunmask_delete(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_lunmask_t  *iocmd = (bfa_ioctl_fcpim_lunmask_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	bfa_trc(bfad, iocmd->bfad_num);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_lunmask_delete(&bfad->bfa,
	iocmd->vf_id, &iocmd->pwwn, iocmd->rpwwn, iocmd->lun);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_boot_cfg(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_boot_t *iocmd = (bfa_ioctl_boot_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_flash_update_part(BFA_FLASH(&bfad->bfa),
			BFA_FLASH_PART_BOOT, bfa_port_id_get(&bfad->bfa),
			&iocmd->cfg, sizeof(struct bfa_boot_cfg_s), 0,
			bfad_hcb_comp, &fcomp, 1);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_boot_query(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_boot_t *iocmd = (bfa_ioctl_boot_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_flash_read_part(BFA_FLASH(&bfad->bfa),
			BFA_FLASH_PART_BOOT, bfa_port_id_get(&bfad->bfa),
			&iocmd->cfg, sizeof(struct bfa_boot_cfg_s), 0,
			bfad_hcb_comp, &fcomp, 1);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_preboot_query(struct bfad_s *bfad, void *cmd)
{
        bfa_ioctl_preboot_t *iocmd = (bfa_ioctl_preboot_t *)cmd;
        unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

        spin_lock_irqsave(&bfad->bfad_lock, flags);
        bfa_iocfc_get_pbc_boot_cfg(&bfad->bfa, &iocmd->cfg);
	iocmd->status = BFA_STATUS_OK;
        spin_unlock_irqrestore(&bfad->bfad_lock, flags);

        return 0;
}

int
bfad_ioctl_ioc_get_version(unsigned long arg)
{
	bfa_ioctl_ioc_get_version_t iocmd;
	int             rc = 0;

	iocmd.version = BFA_IOCTL_VERSION;
	iocmd.status = BFA_STATUS_OK;

	if (copy_to_user
            ((void __user *)arg, (uint8_t *) &iocmd, sizeof(iocmd)))
                rc = EIO;
	return rc;
}

/*
 * Assume the instance numbers are in ascending order.
 * Returns the number of ioc instances in iocmd.cnt;
 * iocmd.bm[] has the bits set corresponding to the instances numbers
 */

int
bfad_ioctl_ioc_get_inst(unsigned long arg)
{
	struct bfad_s         *bfad = NULL;
	bfa_ioctl_ioc_get_inst_t iocmd;
	int idx = 0, bt_idx, rc = 0;
	int bt_sz = sizeof(iocmd.bm[0]) * 8;

	memset(&iocmd, 0, sizeof(iocmd));

	down(&bfad_sem);
	while (!bfad_scan_done) {
		up(&bfad_sem);
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
		down(&bfad_sem);
	}

	list_for_each_entry(bfad, &bfad_list, list_entry) {
		if (bfad->inst_no >= (idx + 1) * bt_sz) {
			idx++;
			if (idx > sizeof(iocmd.bm)) {
				/*
				 * too many instances
				 */
				rc = EIO;
				goto ext;
			}
		}
		bt_idx = bfad->inst_no - idx * bt_sz;
		if (bt_idx < 0) {
			rc = EIO;	/* inst_no not in ascending order */
			goto ext;
		}
		iocmd.bm[idx] |= (1 << bt_idx);
		iocmd.cnt++;
	}
	if (copy_to_user
            ((void __user *)arg, (uint8_t *) &iocmd, sizeof(iocmd)))
                rc = EIO;

ext:
	up(&bfad_sem);
	return rc;
}

int
bfad_iocmd_auth_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_auth_attr_t *iocmd = (bfa_ioctl_auth_attr_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_auth_get_attr(&bfad->bfa_fcs, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_get_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_auth_stats_t *iocmd = (bfa_ioctl_auth_stats_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_auth_get_stats(&bfad->bfa_fcs, &iocmd->stats);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_set_policy(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_auth_policy_t *iocmd = (bfa_ioctl_auth_policy_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_auth_set_policy(&bfad->bfa_fcs, iocmd->mode);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_set_algo(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_auth_algo_t *iocmd = (bfa_ioctl_auth_algo_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_auth_set_algo(&bfad->bfa_fcs, iocmd->algo);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_set_group(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_auth_group_t *iocmd = (bfa_ioctl_auth_group_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_auth_set_dh_group(&bfad->bfa_fcs, iocmd->group);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_set_secretstring(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_auth_secretstring_t *iocmd = (bfa_ioctl_auth_secretstring_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcs_auth_set_secretstring_encrypt(&bfad->bfa_fcs,
		iocmd->encrypt, iocmd->len);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_set_secretsource(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_auth_secretsource_t *iocmd = (bfa_ioctl_auth_secretsource_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcs_auth_set_secretsource(&bfad->bfa_fcs, iocmd->source);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_reset_stats(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_auth_reset_stats(&bfad->bfa_fcs);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_auth_reinit(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcs_auth_reinit(&bfad->bfa_fcs);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;

}

/*
 * FCPORT
 */
int
bfad_iocmd_fcport_enable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcport_enable(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcport_disable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcport_disable(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);


	return 0;
}

int
bfad_iocmd_fcport_get_stats(struct bfad_s *bfad, void *cmd)
{
    bfa_ioctl_fcport_stats_t *iocmd = (bfa_ioctl_fcport_stats_t *)cmd;
    struct bfad_hal_comp fcomp;
    unsigned long   flags;
    struct bfa_cb_pending_q_s cb_qe;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

    init_completion(&fcomp.comp);
    bfa_pending_q_init(&cb_qe, (bfa_cb_cbfn_t)bfad_hcb_comp,
    				&fcomp, &iocmd->stats);

    spin_lock_irqsave(&bfad->bfad_lock, flags);
    iocmd->status =
        bfa_fcport_get_stats(&bfad->bfa, &cb_qe);
    spin_unlock_irqrestore(&bfad->bfad_lock, flags);
    if (iocmd->status != BFA_STATUS_OK) {
        bfa_trc(bfad, iocmd->status);
        goto out;
    }

    wait_for_completion(&fcomp.comp);
    iocmd->status = fcomp.status;

out:
    return 0;

}

int
bfad_iocmd_fcport_reset_stats(struct bfad_s *bfad, void *cmd)
{
    bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
    struct bfad_hal_comp fcomp;
    unsigned long   flags;
    struct bfa_cb_pending_q_s cb_qe;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

    init_completion(&fcomp.comp);
    bfa_pending_q_init(&cb_qe, (bfa_cb_cbfn_t)bfad_hcb_comp, &fcomp, NULL);

    spin_lock_irqsave(&bfad->bfad_lock, flags);
    iocmd->status =
        bfa_fcport_clear_stats(&bfad->bfa, &cb_qe);
    spin_unlock_irqrestore(&bfad->bfad_lock, flags);

    if (iocmd->status != BFA_STATUS_OK) {
        bfa_trc(bfad, iocmd->status);
        goto out;
    }

    wait_for_completion(&fcomp.comp);
    iocmd->status = fcomp.status;

out:

    return 0;
}

int
bfad_iocmd_trunk_enable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_trunk_enable(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_trunk_disable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_trunk_disable(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_trunk_get_attr(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_trunk_attr_t *iocmd = (bfa_ioctl_trunk_attr_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_trunk_get_attr(&bfad->bfa, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_target_mode(struct bfad_s *bfad, void *cmd)
{
	bfa_tgt_mode_ioctl_t *iocmd = (bfa_tgt_mode_ioctl_t*)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfad_target_mode_ioctl(bfad, iocmd);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_pcifn_create(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_pcifn_t *iocmd = (bfa_ioctl_pcifn_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_ablk_pf_create(&bfad->bfa.modules.ablk,
				&iocmd->pcifn_id, iocmd->port,
				iocmd->pcifn_class, iocmd->bw_min,
				iocmd->bw_max, bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:
	return 0;
}

int
bfad_iocmd_pcifn_delete(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_pcifn_t *iocmd = (bfa_ioctl_pcifn_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_ablk_pf_delete(&bfad->bfa.modules.ablk,
				iocmd->pcifn_id,
				bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:
	return 0;
}

int
bfad_iocmd_pcifn_bw(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_pcifn_t *iocmd = (bfa_ioctl_pcifn_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_ablk_pf_update(&bfad->bfa.modules.ablk,
				iocmd->pcifn_id,
				iocmd->bw_min, iocmd->bw_max,
				bfad_hcb_comp, &fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;
	bfa_trc(bfad, iocmd->status);

out:
	return 0;
}

int
bfad_iocmd_vhba_query(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_vhba_attr_t *iocmd = (bfa_ioctl_vhba_attr_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_vhba_query(&bfad->bfa, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfa_trc(bfad, iocmd->status);

	return 0;
}

int
bfad_iocmd_vhba_enable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	/* If IOC is not in disabled state - return */
	if (!bfa_ioc_is_disabled(&bfad->bfa.ioc)) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_OK;
		return 0;
	}

	init_completion(&bfad->enable_comp);
	bfa_iocfc_enable(&bfad->bfa);
	iocmd->status = BFA_STATUS_OK;
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	wait_for_completion(&bfad->enable_comp);

	return 0;
}

int
bfad_iocmd_vhba_disable(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_gen_t *iocmd = (bfa_ioctl_gen_t *)cmd;
	unsigned long flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (bfa_ioc_is_disabled(&bfad->bfa.ioc)) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		iocmd->status = BFA_STATUS_OK;
		return 0;
	}
	if (bfad->disable_active) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		return EBUSY;
	}

	bfad->disable_active = BFA_TRUE;
	init_completion(&bfad->disable_comp);
	bfa_iocfc_disable(&bfad->bfa);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	wait_for_completion(&bfad->disable_comp);
	bfad->disable_active = BFA_FALSE;

	iocmd->status = BFA_STATUS_OK;

	return 0;
}


int
bfad_iocmd_ethboot_cfg(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_ethboot_t *iocmd = (bfa_ioctl_ethboot_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_flash_update_part(BFA_FLASH(&bfad->bfa),
				BFA_FLASH_PART_PXECFG,
				bfa_port_id_get(&bfad->bfa), &iocmd->cfg,
				sizeof(struct bfa_ethboot_cfg_s), 0,
				bfad_hcb_comp, &fcomp, 1);

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

int
bfad_iocmd_ethboot_query(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_ethboot_t *iocmd = (bfa_ioctl_ethboot_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_flash_read_part(BFA_FLASH(&bfad->bfa),
				BFA_FLASH_PART_PXECFG,
				bfa_port_id_get(&bfad->bfa), &iocmd->cfg,
				sizeof(struct bfa_ethboot_cfg_s), 0,
				bfad_hcb_comp, &fcomp, 1);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:

	return 0;
}

/* Adapter / Port mode iocmds */
int
bfad_iocmd_adapter_cfg_mode(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_adapter_cfg_mode_t *iocmd =
			(bfa_ioctl_adapter_cfg_mode_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags = 0;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_ablk_adapter_config(&bfad->bfa.modules.ablk,
						iocmd->cfg.mode,
						iocmd->cfg.max_pf,
						iocmd->cfg.max_vf,
						bfad_hcb_comp,
						&fcomp);
        spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:
	return 0;
}

int
bfad_iocmd_port_cfg_mode(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_port_cfg_mode_t *iocmd =
			(bfa_ioctl_port_cfg_mode_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags = 0;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_ablk_port_config(&bfad->bfa.modules.ablk,
						iocmd->instance,
						iocmd->cfg.mode,
						iocmd->cfg.max_pf,
						iocmd->cfg.max_vf,
						bfad_hcb_comp,
						&fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status != BFA_STATUS_OK)
		goto out;

	wait_for_completion(&fcomp.comp);
	iocmd->status = fcomp.status;

out:
	return 0;
}

int
bfad_iocmd_fcpim_throttle_query(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_throttle_t *iocmd = (bfa_ioctl_fcpim_throttle_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_throttle_get(&bfad->bfa, (void *)&iocmd->throttle);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_fcpim_throttle_set(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fcpim_throttle_t *iocmd = (bfa_ioctl_fcpim_throttle_t *)cmd;
	unsigned long   flags;

	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
        if (bfad == NULL) {
                bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
                if (bfad == NULL)
                        return EINVAL;
        }

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fcpim_throttle_set(&bfad->bfa, iocmd->throttle.cfg_value);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_tfru_read(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_tfru_t *iocmd =
			(bfa_ioctl_tfru_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags = 0;
	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_tfru_read(BFA_FRU(&bfad->bfa),
					&iocmd->data, 
					iocmd->len,
					iocmd->offset,
					bfad_hcb_comp,
					&fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status == BFA_STATUS_OK) {
		wait_for_completion(&fcomp.comp);
		iocmd->status = fcomp.status;
	}

	return 0;
}

int
bfad_iocmd_tfru_write(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_tfru_t *iocmd =
			(bfa_ioctl_tfru_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags = 0;
	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_tfru_write(BFA_FRU(&bfad->bfa),
					&iocmd->data, 
					iocmd->len,
					iocmd->offset,
					bfad_hcb_comp,
					&fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status == BFA_STATUS_OK) {
		wait_for_completion(&fcomp.comp);
		iocmd->status = fcomp.status;
	}

	return 0;
}

int
bfad_iocmd_fruvpd_read(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fruvpd_t *iocmd =
			(bfa_ioctl_fruvpd_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags = 0;
	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fruvpd_read(BFA_FRU(&bfad->bfa),
					&iocmd->data, 
					iocmd->len,
					iocmd->offset,
					bfad_hcb_comp,
					&fcomp);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status == BFA_STATUS_OK) {
		wait_for_completion(&fcomp.comp);
		iocmd->status = fcomp.status;
	}

	return 0;
}

int
bfad_iocmd_fruvpd_update(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fruvpd_t *iocmd =
			(bfa_ioctl_fruvpd_t *)cmd;
	struct bfad_hal_comp fcomp;
	unsigned long flags = 0;
	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	init_completion(&fcomp.comp);
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fruvpd_update(BFA_FRU(&bfad->bfa),
					&iocmd->data, 
					iocmd->len,
					iocmd->offset,
					bfad_hcb_comp,
					&fcomp,
					iocmd->trfr_cmpl);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	if (iocmd->status == BFA_STATUS_OK) {
		wait_for_completion(&fcomp.comp);
		iocmd->status = fcomp.status;
	}

	return 0;
}

int
bfad_iocmd_fruvpd_get_max_size(struct bfad_s *bfad, void *cmd)
{
	bfa_ioctl_fruvpd_max_size_t *iocmd =
			(bfa_ioctl_fruvpd_max_size_t *)cmd;
	unsigned long flags = 0;
	/* BFAD is null - fetch bfad from bfad_num : ioctl path */
	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status = bfa_fruvpd_get_max_size(BFA_FRU(&bfad->bfa),
						&iocmd->max_size);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_port_cfg_bbcr(struct bfad_s *bfad, unsigned int cmd, void *pcmd)
{
	bfa_ioctl_bbcr_enable_t *iocmd = (bfa_ioctl_bbcr_enable_t *)pcmd;
	unsigned long flags;
	int rc;

	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (cmd == IOCMD_PORT_BBCR_ENABLE)
		rc = bfa_fcport_cfg_bbcr(&bfad->bfa, BFA_TRUE, iocmd->bb_scn);
	else if (cmd == IOCMD_PORT_BBCR_DISABLE)
		rc = bfa_fcport_cfg_bbcr(&bfad->bfa, BFA_FALSE, 0);
	else {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		return EINVAL;
	}
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	iocmd->status = rc;
	return 0;
}

int
bfad_iocmd_port_get_bbcr_attr(struct bfad_s *bfad, void *pcmd)
{
	bfa_ioctl_bbcr_attr_t *iocmd = (bfa_ioctl_bbcr_attr_t *) pcmd;
	unsigned long flags;

	if (bfad == NULL) {
		bfad = bfad_find_bfad_by_inst_no(iocmd->bfad_num);
		if (bfad == NULL)
			return EINVAL;
	}

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	iocmd->status =
		bfa_fcport_get_bbcr_attr(&bfad->bfa, &iocmd->attr);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;
}

int
bfad_iocmd_handler(struct bfad_s *bfad, unsigned int cmd, void *iocmd,
		   unsigned long arg, unsigned int payload_len)
{
	int rc = EINVAL;

	switch (cmd) {
	/*
	 * IOC IOCTLs
	 */
	case IOCMD_IOC_ENABLE:
		rc = bfad_iocmd_ioc_enable(bfad, iocmd);
		break;
	case IOCMD_IOC_DISABLE:
		rc = bfad_iocmd_ioc_disable(bfad, iocmd);
		break;
	case IOCMD_IOC_GET_ATTR:
		rc = bfad_iocmd_ioc_get_attr(bfad, iocmd);
		break;
	case IOCMD_IOC_GET_INFO:
		rc = bfad_iocmd_ioc_get_info(bfad, iocmd);
		break;
	case IOCMD_IOC_GET_STATS:
		rc = bfad_iocmd_ioc_get_stats(bfad, iocmd);
		break;
	case IOCMD_IOC_RESET_STATS:
		rc = bfad_iocmd_ioc_reset_stats(bfad, iocmd);
		break;
	case IOCMD_IOC_SET_ADAPTER_NAME:
		rc = bfad_iocmd_ioc_set_adapter_name(bfad, iocmd);
		break;
	case IOCMD_IOC_SET_PORT_NAME:
		rc = bfad_iocmd_ioc_set_port_name(bfad, iocmd);
		break;
	case IOCMD_IOC_GET_FWSTATS:
		rc = bfad_iocmd_ioc_get_fwstats(bfad, iocmd, payload_len);
		break;
	case IOCMD_IOC_RESET_FWSTATS:
		rc = bfad_iocmd_ioc_reset_fwstats(bfad, iocmd);
		break;
	case IOCMD_IOC_PCIFN_CFG:
		rc = bfad_iocmd_ioc_get_pcifn_cfg(bfad, iocmd);
		break;
	case IOCMD_IOC_FW_SIG_INV:
		rc = bfad_iocmd_ioc_fw_sig_inv(bfad, iocmd);
		break;
	/*
	 * IOCFC IOCTLs
	 */
	case IOCMD_IOCFC_GET_ATTR:
		rc = bfad_iocmd_iocfc_get_attr(bfad, iocmd);
		break;
	case IOCMD_IOCFC_SET_INTR:
		rc = bfad_iocmd_iocfc_set_intr(bfad, iocmd);
		break;

	/*
	 * PORT IOCTLs
	 */
	case IOCMD_PORT_ENABLE:
		rc = bfad_iocmd_port_enable(bfad, iocmd);
		break;
	case IOCMD_PORT_DISABLE:
		rc = bfad_iocmd_port_disable(bfad, iocmd);
		break;
	case IOCMD_PORT_GET_STATS:
		rc = bfad_iocmd_port_get_stats(bfad, iocmd, payload_len);
		break;
	case IOCMD_PORT_RESET_STATS:
		rc = bfad_iocmd_port_reset_stats(bfad, iocmd);
		break;
	case IOCMD_PORT_GET_ATTR:
		rc = bfad_iocmd_port_get_attr(bfad, iocmd);
		break;
	case IOCMD_PORT_CFG_TOPO:
		rc = bfad_iocmd_port_cfg_topology(bfad, iocmd);
		break;
	case IOCMD_PORT_CFG_SPEED:
		rc = bfad_iocmd_port_cfg_speed(bfad, iocmd);
		break;
	case IOCMD_PORT_CFG_ALPA:
		rc = bfad_iocmd_port_cfg_hardalpa(bfad, iocmd);
		break;
	case IOCMD_PORT_CLR_ALPA:
		rc = bfad_iocmd_port_clr_hardalpa(bfad, iocmd);
		break;
	case IOCMD_PORT_CFG_MAXFRSZ:
		rc = bfad_iocmd_port_cfg_maxfrsize(bfad, iocmd);
		break;
	case IOCMD_PORT_BBCR_ENABLE:
	case IOCMD_PORT_BBCR_DISABLE:
		rc = bfad_iocmd_port_cfg_bbcr(bfad, cmd, iocmd);
		break;
	case IOCMD_PORT_BBCR_GET_ATTR:
		rc = bfad_iocmd_port_get_bbcr_attr(bfad, iocmd);
		break;
	/*
	 * LPORT IOCTLs
	 */
	case IOCMD_LPORT_GET_ATTR:
		rc = bfad_iocmd_lport_get_attr(bfad, iocmd);
		break;
	case IOCMD_LPORT_GET_STATS:
		rc = bfad_iocmd_lport_get_stats(bfad, iocmd);
		break;
	case IOCMD_LPORT_RESET_STATS:
		rc = bfad_iocmd_lport_reset_stats(bfad, iocmd);
		break;
	case IOCMD_LPORT_GET_RPORTS:
		rc = bfad_iocmd_lport_get_rports(bfad, iocmd, payload_len);
		break;
	case IOCMD_LPORT_GET_IOSTATS:
		rc = bfad_iocmd_lport_get_iostats(bfad, iocmd);
		break;

	/*
	 * RPORT IOCTLs
	 */
	case IOCMD_RPORT_ADD:
	case IOCMD_RPORT_REMOVE:
		rc = EINVAL;
		break;
	case IOCMD_RPORT_GET_ATTR:
		rc = bfad_iocmd_rport_get_attr(bfad, iocmd);
		break;
	case IOCMD_RPORT_GET_STATS:
		rc = bfad_iocmd_rport_get_stats(bfad, iocmd);
		break;
	case IOCMD_RPORT_RESET_STATS:
		rc = bfad_iocmd_rport_clr_stats(bfad, iocmd);
		break;
	case IOCMD_RPORT_GET_ADDR:
		rc = bfad_iocmd_rport_get_addr(bfad, iocmd);
		break;
	case IOCMD_RPORT_SET_SPEED:
		rc = bfad_iocmd_rport_set_speed(bfad, iocmd);
		break;
	/*
	 * VPORT IOCTLs
	 */
	case IOCMD_VPORT_START:
		rc = bfad_iocmd_vport_start(bfad, iocmd);
		break;
	case IOCMD_VPORT_STOP:
		rc = bfad_iocmd_vport_stop(bfad, iocmd);
		break;
	case IOCMD_VPORT_GET_ATTR:
		rc = bfad_iocmd_vport_get_attr(bfad, iocmd);
		break;
	case IOCMD_VPORT_GET_STATS:
		rc = bfad_iocmd_vport_get_stats(bfad, iocmd);
		break;
	case IOCMD_VPORT_RESET_STATS:
		rc = bfad_iocmd_vport_clr_stats(bfad, iocmd);
		break;
	/*
	 * VF IOCTLs
	 */
	case IOCMD_VF_ENABLE:
		rc = bfad_iocmd_vf_mode_enable(bfad, iocmd);
		break;
	case IOCMD_VF_DISABLE:
		rc = bfad_iocmd_vf_mode_disable(bfad, iocmd);
		break;
	case IOCMD_VF_ADD:
		rc = bfad_iocmd_vf_add(bfad, iocmd);
		break;
	case IOCMD_VF_DEL:
		rc = bfad_iocmd_vf_del(bfad, iocmd);
		break;
	case IOCMD_VF_LIST:
	case IOCMD_VF_LIST_ALL:
		rc = bfad_iocmd_vf_list(bfad, cmd, iocmd, payload_len);
		break;
	case IOCMD_VF_GET_ATTR:
		rc = bfad_iocmd_vf_get_attr(bfad, iocmd);
		break;
	case IOCMD_VF_GET_STATS:
		rc = bfad_iocmd_vf_get_stats(bfad, iocmd);
		break;
	case IOCMD_VF_RESET_STATS:
		rc = bfad_iocmd_vf_clr_stats(bfad, iocmd);
		break;
	/*
	 * FABRIC
	 */
	case IOCMD_FABRIC_GET_LPORTS:
		rc = bfad_iocmd_fabric_get_lports(bfad, iocmd, payload_len);
		break;
	/*
	 * FLASH IOCTLs
	 */
	case IOCMD_FLASH_GET_ATTR:
		rc = bfad_iocmd_flash_get_attr(bfad, iocmd);
		break;
	case IOCMD_FLASH_ERASE_PART:
		rc = bfad_iocmd_flash_erase_part(bfad, iocmd);
		break;
	case IOCMD_FLASH_UPDATE_PART:
		rc = bfad_iocmd_flash_update_part(bfad, iocmd, payload_len);
		break;
	case IOCMD_FLASH_MFG_INFO:
		rc = bfad_iocmd_flash_mfg_info(bfad, iocmd, payload_len);
		break;
	case IOCMD_FLASH_READ_PART:
		rc = bfad_iocmd_flash_read_part(bfad, iocmd, payload_len);
		break;
	case IOCMD_FLASH_ENABLE_OPTROM:
	case IOCMD_FLASH_DISABLE_OPTROM:
		rc = bfad_iocmd_ablk_optrom(bfad, cmd, iocmd);
		break;
	case IOCMD_FLASH_GET_CFG_BOOTVER:
		rc = bfad_iocmd_flash_get_cfg_bootver(bfad, iocmd, payload_len);
		break;

	case IOCMD_PHY_GET_ATTR:
		rc = bfad_iocmd_phy_get_attr(bfad, iocmd);
		break;
	case IOCMD_PHY_GET_STATS:
		rc = bfad_iocmd_phy_get_stats(bfad, iocmd);
		break;
	case IOCMD_PHY_UPDATE_FW:
		rc = bfad_iocmd_phy_update(bfad, iocmd, payload_len);
		break;
	case IOCMD_PHY_READ_FW:
		rc = bfad_iocmd_phy_read(bfad, iocmd, payload_len);
		break;
	/*
	 * DIAG IOCTLs
	 */
	case IOCMD_DIAG_MEMRD:
		rc = bfad_iocmd_diag_memrd(bfad, iocmd);
		break;
	case IOCMD_DIAG_TEMP:
		rc = bfad_iocmd_diag_temp(bfad, iocmd);
		break;
	case IOCMD_DIAG_POST:
		rc = bfad_iocmd_diag_post(bfad, iocmd);
		break;
	case IOCMD_DIAG_MEMTEST:
		rc = bfad_iocmd_diag_memtest(bfad, iocmd);
		break;
	case IOCMD_DIAG_LOOPBACK:
		rc = bfad_iocmd_diag_loopback(bfad, iocmd);
		break;
	case IOCMD_DIAG_FWPING:
		rc = bfad_iocmd_diag_fwping(bfad, iocmd);
		break;
	case IOCMD_DIAG_QUEUETEST:
		rc = bfad_iocmd_diag_queuetest(bfad, iocmd);
		break;
	case IOCMD_DIAG_POM:
		rc = bfad_iocmd_diag_pom(bfad, iocmd);
		break;
	case IOCMD_DIAG_SFP:
		rc = bfad_iocmd_diag_sfp(bfad, iocmd);
		break;
	case IOCMD_DIAG_LED:
		rc = bfad_iocmd_diag_led(bfad, iocmd);
		break;
	case IOCMD_DIAG_BEACON_LPORT:
		rc = bfad_iocmd_diag_beacon_lport(bfad, iocmd);
		break;
	case IOCMD_DIAG_LB_STAT:
		rc = bfad_iocmd_diag_lb_stat(bfad, iocmd);
		break;
	case IOCMD_DIAG_DPORT_ENABLE:
		rc = bfad_iocmd_diag_dport_enable(bfad, iocmd);
		break;
	case IOCMD_DIAG_DPORT_DISABLE:
		rc = bfad_iocmd_diag_dport_disable(bfad, iocmd);
		break;
	case IOCMD_DIAG_DPORT_SHOW:
		rc = bfad_iocmd_diag_dport_show(bfad, iocmd);
		break;
	case IOCMD_DIAG_DPORT_START:
		rc = bfad_iocmd_diag_dport_start(bfad, iocmd);
		break;
	/*
	 * SFP
	 */
	case IOCMD_SFP_MEDIA:
		rc = bfad_iocmd_sfp_media(bfad, iocmd);
		break;
	case IOCMD_SFP_SPEED:
		rc = bfad_iocmd_sfp_speed(bfad, iocmd);
		break;
	/*
	 * CEE IOCTLs
	 */
	case IOCMD_CEE_GET_ATTR:
		rc = bfad_iocmd_cee_attr(bfad, iocmd, payload_len);
		break;
	case IOCMD_CEE_GET_STATS:
		rc = bfad_iocmd_cee_get_stats(bfad, iocmd, payload_len);
		break;
	case IOCMD_CEE_RESET_STATS:
		rc = bfad_iocmd_cee_reset_stats(bfad, iocmd);
		break;
	/*
	 * DEBUG IOCTLs
	 */
	case IOCMD_DEBUG_DRV_STATE:
		rc = bfad_iocmd_debug_drv_state(bfad, iocmd);
		break;
	case IOCMD_DEBUG_FW_STATE_CLR:
		rc = bfad_iocmd_debug_fw_trace_clear(bfad, iocmd);
		break;
	case IOCMD_DEBUG_FW_CORE:
		rc = bfad_iocmd_debug_fw_core(bfad, iocmd, payload_len);
		break;
	case IOCMD_DEBUG_PORTLOG:
		rc = bfad_iocmd_porglog_get(bfad, iocmd);
		break;
	case IOCMD_DEBUG_PORTLOG_CLR:
		rc = bfad_iocmd_porglog_clear(bfad, iocmd);
		break;
	case IOCMD_DEBUG_PORTLOG_CTL:
		rc = bfad_iocmd_porglog_ctl(bfad, iocmd);
		break;
	case IOCMD_DEBUG_START_DTRC:
		rc = bfad_iocmd_start_dtrc(bfad, iocmd);
                break;
        case IOCMD_DEBUG_STOP_DTRC:
                rc = bfad_iocmd_stop_dtrc(bfad, iocmd);
                break;

	/*
	 * LOG IOCTLs
	 */
	case IOCMD_LOG_SET_LEVEL:
		rc = bfad_iocmd_log_set_level(bfad, iocmd);
		break;
	case IOCMD_LOG_GET_LEVEL:
		rc = bfad_iocmd_log_get_level(bfad, iocmd);
		break;

	/* QoS IOCTLS */
	case IOCMD_QOS_ENABLE:
	case IOCMD_QOS_DISABLE:
		rc = bfad_iocmd_qos(bfad, cmd, iocmd);
		break;

	case IOCMD_QOS_GET_ATTR:
		rc = bfad_iocmd_qos_get_attr(bfad, iocmd);
		break;

	case IOCMD_QOS_GET_VC_ATTR:
		rc = bfad_iocmd_qos_get_vc_attr(bfad, iocmd);
		break;

	case IOCMD_QOS_GET_STATS:
		rc = bfad_iocmd_qos_get_stats(bfad, iocmd);
		break;
	case IOCMD_QOS_RESET_STATS:
		rc = bfad_iocmd_qos_reset_stats(bfad, iocmd);
		break;
	case IOCMD_QOS_SET_BW:
		rc = bfad_iocmd_qos_set_bw(bfad, iocmd);
		break;

	/*
	 * RATELIM IOCTLs
	 */
	case IOCMD_RATELIM_ENABLE:
	case IOCMD_RATELIM_DISABLE:
		rc = bfad_iocmd_ratelim(bfad, cmd, iocmd);
		break;
	case IOCMD_RATELIM_DEF_SPEED:
		rc = bfad_iocmd_ratelim_speed(bfad, cmd, iocmd);
		break;
	/*
	 * FCPIM IOCTLs
	 */
	case IOCMD_FCPIM_FAILOVER:
		rc = bfad_iocmd_fcpim_failover(bfad, iocmd);
		break;
	case IOCMD_FCPIM_QDEPTH:
		rc = bfad_iocmd_fcpim_qdepth(bfad, iocmd);
		break;
	case IOCMD_FCPIM_MODSTATS:
		rc = bfad_iocmd_fcpim_get_modstats(bfad, iocmd);
		break;
	case IOCMD_FCPIM_MODSTATSCLR:
		rc = bfad_iocmd_fcpim_clr_modstats(bfad, iocmd);
		break;
	case IOCMD_FCPIM_DEL_ITN_STATS:
		rc = bfad_iocmd_fcpim_get_del_itn_stats(bfad, iocmd);
		break;
	case IOCMD_FCPIM_LUNMASK_ENABLE:
		rc = bfad_iocmd_lunmask_enable(bfad, iocmd);
		break;
	case IOCMD_FCPIM_LUNMASK_DISABLE:
		rc = bfad_iocmd_lunmask_disable(bfad, iocmd);
		break;
	case IOCMD_FCPIM_LUNMASK_QUERY:
		rc = bfad_iocmd_fcpim_lunmask_query(bfad, iocmd);
		break;
	case IOCMD_FCPIM_LUNMASK_ADD:
		rc = bfad_iocmd_fcpim_lunmask_add(bfad, iocmd);
		break;
	case IOCMD_FCPIM_LUNMASK_DELETE:
		rc = bfad_iocmd_fcpim_lunmask_delete(bfad, iocmd);
		break;
	case IOCMD_FCPIM_LUNMASK_CLEAR:
		rc = bfad_iocmd_fcpim_lunmask_clear(bfad, iocmd);
		break;

	case IOCMD_FCPIM_THROTTLE_QUERY:
		rc = bfad_iocmd_fcpim_throttle_query(bfad, iocmd);
		break;
	case IOCMD_FCPIM_THROTTLE_SET:
		rc = bfad_iocmd_fcpim_throttle_set(bfad, iocmd);
		break;

	case IOCMD_FCPIM_IOPF_START:
		rc = bfad_iocmd_fcpim_iopf_start(bfad, iocmd);
		break;
	case IOCMD_FCPIM_IOPF_STOP:
		rc = bfad_iocmd_fcpim_iopf_stop(bfad, iocmd);
		break;
	case IOCMD_FCPIM_IOPF_SHOW:
		rc = bfad_iocmd_fcpim_iopf_show(bfad, iocmd);
		break;
	case IOCMD_FCPIM_IOPF_QUERY:
		rc = bfad_iocmd_fcpim_iopf_query(bfad, iocmd);
		break;
	case IOCMD_FCPIM_IOPF_RESET:
		rc = bfad_iocmd_fcpim_iopf_reset(bfad, iocmd);
		break;

	/*
	 * BOOT IOCTLs
	 */
	case IOCMD_BOOT_CFG:
		rc = bfad_iocmd_boot_cfg(bfad, iocmd);
		break;
	case IOCMD_BOOT_QUERY:
		rc = bfad_iocmd_boot_query(bfad, iocmd);
		break;
	case IOCMD_PREBOOT_QUERY:
		rc = bfad_iocmd_preboot_query(bfad, iocmd);
		break;
	/*
	 * ITNIM IOCTLs
	 */
	case IOCMD_ITNIM_GET_ATTR:
		rc = bfad_iocmd_itnim_get_attr(bfad, iocmd);
		break;
	case IOCMD_ITNIM_GET_IOSTATS:
		rc = bfad_iocmd_itnim_get_iostats(bfad, iocmd);
		break;
	case IOCMD_ITNIM_RESET_STATS:
		rc = bfad_iocmd_itnim_reset_stats(bfad, iocmd);
		break;
	case IOCMD_ITNIM_GET_ITNSTATS:
		rc = bfad_iocmd_itnim_get_itnstats(bfad, iocmd);
		break;
	case IOCMD_ITNIM_TRS_ENABLE:
		rc = bfad_iocmd_itnim_trs_en_dis(bfad, iocmd, BFA_FALSE);
		break;
	case IOCMD_ITNIM_TRS_DISABLE:
		rc = bfad_iocmd_itnim_trs_en_dis(bfad, iocmd, BFA_TRUE);
		break;
	case IOCMD_ITNIM_TRS_CLEAR:
		rc = bfad_iocmd_itnim_trs_clear(bfad, iocmd);
		break;
	case IOCMD_ITNIM_TRS_QUERY:
		rc = bfad_iocmd_itnim_trs_query(bfad, iocmd, payload_len);
		break;
	/*
	 * AUTH IOCTLs
	 */
	case IOCMD_AUTH_GET_ATTR:
		rc = bfad_iocmd_auth_get_attr(bfad, iocmd);
		break;
	case IOCMD_AUTH_GET_STATS:
		rc = bfad_iocmd_auth_get_stats(bfad, iocmd);
		break;
	case IOCMD_AUTH_REINIT:
		rc = bfad_iocmd_auth_reinit(bfad, iocmd);
		break;
	case IOCMD_AUTH_SET_POLICY:
		rc = bfad_iocmd_auth_set_policy(bfad, iocmd);
		break;
	case IOCMD_AUTH_SET_ALGO:
		rc = bfad_iocmd_auth_set_algo(bfad, iocmd);
		break;
	case IOCMD_AUTH_SET_GROUP:
		rc = bfad_iocmd_auth_set_group(bfad, iocmd);
		break;
	case IOCMD_AUTH_SET_SECRETSTR:
		rc = bfad_iocmd_auth_set_secretstring(bfad, iocmd);
		break;
	case IOCMD_AUTH_SET_SECRETSRC:
		rc = bfad_iocmd_auth_set_secretsource(bfad, iocmd);
		break;
	case IOCMD_AUTH_RESET_STATS:
		rc = bfad_iocmd_auth_reset_stats(bfad, iocmd);
		break;
	case IOCMD_FCPORT_GET_STATS:
		rc = bfad_iocmd_fcport_get_stats(bfad, iocmd);
		break;
	case IOCMD_FCPORT_RESET_STATS:
	 	rc = bfad_iocmd_fcport_reset_stats(bfad, iocmd);
		break;
	case IOCMD_FCPORT_ENABLE:
		rc = bfad_iocmd_fcport_enable(bfad, iocmd);
		break;
	case IOCMD_FCPORT_DISABLE:
		rc = bfad_iocmd_fcport_disable(bfad, iocmd);
		break;
	case IOCMD_TRUNK_ENABLE:
		rc = bfad_iocmd_trunk_enable(bfad, iocmd);
		break;
	case IOCMD_TRUNK_DISABLE:
		rc = bfad_iocmd_trunk_disable(bfad, iocmd);
		break;
	case IOCMD_TRUNK_GET_ATTR:
		rc = bfad_iocmd_trunk_get_attr(bfad, iocmd);
		break;
	case IOCMD_TARGET_MODE:
		rc = bfad_iocmd_target_mode(bfad, iocmd);
		break;
	/* PCIFN */
	case IOCMD_PCIFN_CREATE:
		rc = bfad_iocmd_pcifn_create(bfad, iocmd);
		break;
	case IOCMD_PCIFN_DELETE:
		rc = bfad_iocmd_pcifn_delete(bfad, iocmd);
		break;
	case IOCMD_PCIFN_BW:
		rc = bfad_iocmd_pcifn_bw(bfad, iocmd);
		break;
	/* VHBA */
	case IOCMD_VHBA_QUERY:
		rc = bfad_iocmd_vhba_query(bfad, iocmd);
		break;
	case IOCMD_VHBA_ENABLE:
		rc = bfad_iocmd_vhba_enable(bfad, iocmd);
		break;
	case IOCMD_VHBA_DISABLE:
		rc = bfad_iocmd_vhba_disable(bfad, iocmd);
		break;
	case IOCMD_FAA_QUERY:
		rc = bfad_iocmd_faa_query(bfad, iocmd);
		break;
	case IOCMD_ETHBOOT_CFG:
		rc = bfad_iocmd_ethboot_cfg(bfad, iocmd);
		break;
	case IOCMD_ETHBOOT_QUERY:
		rc = bfad_iocmd_ethboot_query(bfad, iocmd);
		break;
	/* Adapter/Port Mode */
	case IOCMD_ADAPTER_CFG_MODE:
		rc = bfad_iocmd_adapter_cfg_mode(bfad, iocmd);
		break;
	case IOCMD_PORT_CFG_MODE:
		rc = bfad_iocmd_port_cfg_mode(bfad, iocmd);
		break;
	/* TFRU */
	case IOCMD_TFRU_READ:
		rc = bfad_iocmd_tfru_read(bfad, iocmd);
		break;
	case IOCMD_TFRU_WRITE:
		rc = bfad_iocmd_tfru_write(bfad, iocmd);
		break;
	/* FRU */
	case IOCMD_FRUVPD_READ:
		rc = bfad_iocmd_fruvpd_read(bfad, iocmd);
		break;
	case IOCMD_FRUVPD_UPDATE:
		rc = bfad_iocmd_fruvpd_update(bfad, iocmd);
		break;
	case IOCMD_FRUVPD_GET_MAX_SIZE:
		rc = bfad_iocmd_fruvpd_get_max_size(bfad, iocmd);
		break;
	default:
		rc = EINVAL;
		break;
	}

	return rc;
}

